package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.model.AlarmPatchParam;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.common.utils.ResultTools;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 告警
 *
 * @author zhangyf
 * @since 2019-02-18
 */
@Api(value = "告警", tags = "告警")
@RestController
@RequestMapping("/v1/alarms")
public class AlarmController {

    private final String CONFIRM = "confirm";
    private final String SOLVE = "solve";

    @Autowired
    private AlarmService alarmService;

    @GetMapping
    @ApiOperation(value = "告警列表查询", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return new BocloudResult(true, alarmService.list(pager, context), "查询成功");
    }

    /**
     * 用户首页显示
     */
    @GetMapping("/portal")
    @ApiOperation(value = "首页告警列表查询", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult portal(@Value(Common.REQ_CONTEXT) RequestContext context) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("eventAlarm", 0), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("status", "SOLVED"), Sign.UEQ));
        Pager pager = new Pager(1, 10, params, null);
        return new BocloudResult(true, alarmService.list(pager, context), "查询成功");
    }

    /**
     * 详情查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "告警详情查询", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return alarmService.detail(id);
    }


    /**
     * 告警确认、告警解决
     *
     * @param param
     * @param context
     * @return
     */
    @PatchMapping
    @ApiOperation(value = "告警解决/告警确认", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult patch(@Value(Common.REQ_CONTEXT) RequestContext context,
                                 @RequestBody AlarmPatchParam param) {
        if (CONFIRM.equals(param.getAction())) {
            try {
                return alarmService.confirm(param.getIds(), context);
            } catch (RuntimeException e) {
                return new BocloudResult(false, "确认告警失败：" + e.getMessage());
            }
        }
        if (SOLVE.equals(param.getAction())) {
            try {
                alarmService.solve(param.getIds(), context, param.getDealResult());
                return new BocloudResult(true, "告警已解决");
            } catch (RuntimeException e) {
                return new BocloudResult(false, "解决告警失败：" + e.getMessage());
            }
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 告警统计
     *
     * @param paramList
     * @param context
     * @return
     */
    @ApiOperation(value = "告警统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/stats")
    public BocloudResult stats(@RequestBody List<Param> paramList,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != paramList) {
            return alarmService.stats(paramList, context);
        } else {
            return new BocloudResult(false, "参数格式不正确");
        }
    }

    /**
     * 告警统计数据饼状图、告警数量柱状堆积图
     * @param vendorId
     * @param target
     * @param type
     * @param vendorType
     * @param start
     * @param end
     * @param action
     * @param context
     * @return
     */
    @GetMapping("/chart")
    @ApiOperation(value = "告警统计数据饼状图、告警数量柱状堆积图", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult alarmLevel(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                    @RequestParam(value = "target", required = false) Object target,
                                    @RequestParam(value = "type", required = false) String type,
                                    @RequestParam(value = "vendorType", required = false) String vendorType,
                                    @RequestParam(value = "start", required = false) Long start,
                                    @RequestParam(value = "end", required = false) Long end,
                                    @RequestParam(value = Common.ACTION, required = false) String action,
                                    @RequestParam(value = "time", required = false) String time,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (action) {
            //pie chart - 饼状图
            case "pieChart":
                return this.alarmService.alarmLevel(context, start, end, vendorId, target, type, vendorType,time);
            //bar chart - 柱状图
            case "barChart":
                return alarmService.alarmStats(context, type, vendorId, target, start, end, vendorType);
            case "vendorChart":
                return alarmService.vendorAlarmStats(context, start, end, vendorType);
            default:
                return new BocloudResult(false, "参数格式不正确");
        }
    }
}
