package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.AlarmTemplate;
import com.bocloud.cms.entity.DeleteIdList;
import com.bocloud.cms.intf.AlarmTemplateService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 告警模板
 * @author fanzx
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
@RestController
@Api(value = "告警模板", tags = "告警模板")
@RequestMapping("/v1/templates")
public class AlarmTemplateController {

    @Autowired
    private AlarmTemplateService alarmTemplateService;

    /**
     * 告警模板新增
     *
     * @return
     */
    @PostMapping
    @ApiOperation(value = "新增告警模板", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@RequestBody AlarmTemplate alarmTemplate,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.create(alarmTemplate, context);
    }

    /**
     * 告警模板修改
     *
     * @return
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "修改告警模板", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@RequestBody AlarmTemplate alarmTemplate,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.modify(alarmTemplate, context);
    }

    /**
     * 告警模板删除
     *
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除告警模板", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.remove(id, context);
    }

    /**
     * 获取告警模板列表
     *
     * @param pager
     * @param context 操作者信息
     * @return 数据结果集
     */
    @GetMapping
    @ApiOperation(value = "告警模板列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.list(pager, context);
    }

    /**
     * 告警模板详情
     *
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "告警模板详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return alarmTemplateService.detail(id);
    }

    /**
     * 批量删除告警模板
     *
     * @param param
     * @return
     */
    @DeleteMapping
    @ApiOperation(value = "批量删除告警模板", httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult batchRemove(@RequestBody DeleteIdList param,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.batchRemove(param.getIds(), context.getTarget());
    }
}
