package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.cms.intf.AnalysisRuleService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 监控分析规则Controller
 *
 * @author zhangdi
 * @since Aug 15,2019
 */

@RestController
@Api(value = "监控分析规则", tags = "监控分析规则")
@RequestMapping("/v1/analysis/rules")
public class AnalysisRuleController {

    private final AnalysisRuleService analysisRuleService;

    @Autowired
    public AnalysisRuleController(AnalysisRuleService analysisRuleService) {
        this.analysisRuleService = analysisRuleService;
    }

    /**
     * 监控分析规则列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @ApiOperation(value = "监控分析规则列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.list(pager, context);
    }

    /**
     * 新增监控分析规则
     *
     * @param analysisRule
     * @param context
     * @return
     */
    @PostMapping
    @ApiOperation(value = "创建监控分析规则", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@RequestBody MonitorAnalysisRule analysisRule,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.create(analysisRule, context);
    }

    /**
     * 修改监控分析规则
     *
     * @param analysisRule
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "监控分析规则修改", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@RequestBody MonitorAnalysisRule analysisRule,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.modify(analysisRule, context);
    }

    /**
     * 删除监控分析规则
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "监控分析规则删除", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.remove(id, context.getTarget());
    }

    /**
     * 监控分析规则详情展示
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "监控分析规则详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return analysisRuleService.detail(id);
    }

}
