package com.bocloud.cms.booter.controller;

import com.bocloud.cms.common.utils.ExcelChartUtil;
import com.bocloud.cms.intf.AnalysisService;
import com.bocloud.cms.intf.AnalysisSnapshotService;
import com.bocloud.cms.model.SnapshotParam;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 监控分析
 *
 * @author zhangyf
 * @since 2019-08-16
 */
@RestController
@Api(value = "监控分析快照", tags = "监控分析快照")
@RequestMapping("/v1/analysis/snapshots")
public class AnalysisSnapshotController {

    @Autowired
    private AnalysisSnapshotService analysisSnapshotService;
    @Autowired
    private AnalysisService analysisService;

    @GetMapping
    @ApiOperation(value = "监控分析快照列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager,
                              @RequestParam(value = Common.SIMPLE, required = false) Boolean simple,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisSnapshotService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(), simple, context);
    }

    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除监控分析快照", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisSnapshotService.remove(id, context.getTarget());
    }

    @GetMapping("/{id}")
    @ApiOperation(value = "监控分析快照详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return analysisSnapshotService.detail(id);
    }

    /**
     * 监控分析
     *
     * @param snapshotParam
     * @param context
     * @return
     */
    @PostMapping
    @ApiOperation(value = "执行监控分析任务", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult handle(@RequestBody SnapshotParam snapshotParam,
                                @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return analysisService.handle(snapshotParam.getId(), snapshotParam.getVendorId(), snapshotParam.getStartTime(),
                snapshotParam.getEndTime(), context, request.getHeader(Common.TOKEN));
    }

    /**
     * 监控报表下载
     *
     * @param id
     * @param request
     * @param response
     * @return
     */
    @GetMapping("/{id}/download")
    @ApiOperation(value = "监控报表下载", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult download(@PathVariable(value = Common.ID) Long id, HttpServletRequest request, HttpServletResponse response) {
        BocloudResult result = analysisSnapshotService.detail(id);
        Assert.isTrue(result.isSuccess(),"无法创建报表:"+result.getMessage());
        return ExcelChartUtil.createMonitorCharExcel(result, request, response);
    }

}
