package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.CmsService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * 消息队列监控
 *
 * @author zhangdi
 * @since Apr 7th,2020
 */

@RestController
@Api(value = "消息队列监控", tags = "消息队列监控")
@RequestMapping("/v1/cms")
public class CmsController {

    private final CmsService cmsService;

    @Autowired
    public CmsController(CmsService cmsService) {
        this.cmsService = cmsService;
    }

    @GetMapping
    @ApiOperation(value = "消队列列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, String catalog, @Value(Common.REQ_CONTEXT) RequestContext context,
                              HttpServletRequest request) {
        return cmsService.list(pager, catalog, context, request);
    }

    @GetMapping("/kafka/{id}/topics")
    @ApiOperation(value = "Kafka Topic列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult topicsOfKafka(@PathVariable(value = Common.ID) Long id) {
        return new BocloudResult(true, cmsService.topicsOfKafka(id), "SUCCESS");
    }

    @GetMapping("/kafka/{id}/consumergroups")
    @ApiOperation(value = "Kafka消费组列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult consumerGroupsOfKafka(@PathVariable(value = Common.ID) Long id) {
        return new BocloudResult(true, cmsService.consumerGroupsOfKafka(id), "SUCCESS");
    }
}
