package com.bocloud.cms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.intf.FusionCloudService;
import com.bocloud.cms.intf.external.RemoteManageOneService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * FileSystem
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Api(value = "FusionCloud监控", tags = "FusionCloud监控")
@RestController
@RequestMapping("/v1/fusioncloud")
public class FusionCloudController {

    @Autowired
    private FusionCloudService fusionCloudService;

    @Autowired
    private RemoteManageOneService remoteManageOneService;

    /**
     * 华为云云主机列表查询
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/hosts")
    @ApiOperation(value = "宿主机列表展示", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listOfHost(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return fusionCloudService.listOfHost(pager, context);
    }

    @GetMapping("/resource-pools")
    @ApiOperation(value = "资源池列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourcePools(Long vendorId) {
        List<JSONObject> resourcePools = remoteManageOneService.listInstance(vendorId,"SYS_ResourcePool",null,null);
        List<JSONObject> azones = remoteManageOneService.listInstance(vendorId,"SYS_Azone",null,null);
        List<Map<String,Object>> resourcePoolList = new ArrayList<>();
        Optional.ofNullable(resourcePools).orElse(new ArrayList<>()).forEach(resourcePool->{
            Map<String, Object> resourcePoolMap = new HashMap<>(16, 0.75F);
            resourcePoolMap.put("label",resourcePool.getString("name"));
            resourcePoolMap.put("value",resourcePool.getString("resId"));
            resourcePoolMap.put("tag","resourcePool");
            List<Map<String,Object>> children = new ArrayList<>();
            resourcePoolMap.put("children",children);
            Optional.ofNullable(azones).orElse(new ArrayList<>()).forEach(azone->{
                Map<String, Object> azoneMap = new HashMap<>(16, 0.75F);
                azoneMap.put("label",azone.getString("name"));
                azoneMap.put("value",azone.getString("resId"));
                azoneMap.put("tag","azone");
                children.add(azoneMap);
            });
            resourcePoolList.add(resourcePoolMap);
        });
        return new BocloudResult(true, resourcePoolList, "success");
    }

    @GetMapping("/resource-pools/{id}")
    @ApiOperation(value = "资源池详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detailResPool(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.getInstanceDetail(vendorId, "SYS_ResourcePool", id), "success");
    }

    @GetMapping("/azones/{id}")
    @ApiOperation(value = "可用区详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detailAzones(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.getInstanceDetail(vendorId, "SYS_Azone", id), "success");
    }

    @GetMapping("/capacity/resource-pools/{id}/statistics")
    @ApiOperation(value = "查询资源池的云主机、物理机状态数目统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourcePoolStatistics(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.resourcePoolStatistics(vendorId, id), "success");
    }

    @GetMapping("/capacity/azones/{id}/statistics")
    @ApiOperation(value = "查询可用区的云主机、物理机状态数目统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult azoneStatistics(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.azoneStatistics(vendorId, id), "success");
    }

    @GetMapping("/capacity/resource-pools/{id}/current-capacities")
    @ApiOperation(value = "资源池的当前容量", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourcePoolCurrentCapacities(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.resourcePoolCurrentCapacities(vendorId, id), "success");
    }

    @GetMapping("/capacity/azones/{id}/current-capacities")
    @ApiOperation(value = "可用区的当前容量", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult azoneCurrentCapacities(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.azoneCurrentCapacities(vendorId, id), "success");
    }

    @GetMapping("/capacity/resource-pools/{id}/resource-types/storage-pool/capacities")
    @ApiOperation(value = "资源池存储资源当前容量", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourcePoolStoragePoolCapacities(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.resourcePoolStoragePoolCapacities(vendorId, id), "success");
    }

    @GetMapping("/capacity/azones/{id}/resource-types/storage-pool/capacities")
    @ApiOperation(value = "可用区存储资源当前容量", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult azoneStoragePoolCapacities(@PathVariable String id, Long vendorId) {
        return new BocloudResult(true, remoteManageOneService.azoneStoragePoolCapacities(vendorId, id), "success");
    }
}
