package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.MonitorHostIpmi;
import com.bocloud.cms.intf.HostService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 宿主机监控
 *
 * @author zhangdi
 * @since June 12,2019
 */

@RestController
@Api(value = "宿主机监控", tags = "宿主机监控")
@RequestMapping("/v1/hosts")
public class HostController {

    @Autowired
    private HostService hostService;

    /**
     * 宿主机列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @ApiOperation(value = "宿主机列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hostService.list(pager, context);
    }

    /**
     * 宿主机基本信息详情
     *
     * @param id
     * @param type
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "宿主机基本信息详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult hostOverview(@PathVariable(value = Common.ID) Long id,
                                      @RequestParam(value = "type") String type) {
        return hostService.hostOverview(id, type);
    }

    /**
     * 宿主机Cpu或内存或磁盘信息
     *
     * @param id
     * @param type
     * @return
     */
    @GetMapping("/{id}/metrics")
    @ApiOperation(value = "主机Cpu/内存/磁盘基本信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult hostInfo(@PathVariable(value = Common.ID) Long id,
                                  @RequestParam(value = "type") String type) {
        return hostService.hostInfo(id, type);
    }

    /**
     * 配置宿主机ipmi信息
     *
     * @param hostIpmi
     * @return
     */
    @PatchMapping("/ipmi/binding")
    @ApiOperation(value = "宿主机绑定ipmi信息", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult binding(@RequestBody MonitorHostIpmi hostIpmi,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hostService.binding(hostIpmi, context);
    }

    /**
     * 删除宿主机ipmi信息
     *
     * @param id
     * @param context
     * @return
     */
    @PatchMapping("/ipmi/unbind")
    @ApiOperation(value = "删除宿主机ipmi信息", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult unbind(Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hostService.unbind(id, context.getTarget());
    }

    /**
     * 获取宿主机ipmi信息
     *
     * @param id
     * @return
     */
    @GetMapping("/ipmi/{id}")
    @ApiOperation(value = "宿主机ipmi信息详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return hostService.detail(id);
    }


}
