package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.KubernetesService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Kubernetes监控
 *
 * @author zhangdi
 * @since Sept 3,2019
 */

@RestController
@Api(value = "Kubernetes监控", tags = "Kubernetes监控")
@RequestMapping("/v1/kubernetes")
public class KubernetesController {

    @Autowired
    private KubernetesService kubernetesService;

    /**
     * 节点列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/nodes")
    @ApiOperation(value = "节点列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listOfNode(Pager pager,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kubernetesService.listOfNode(pager, context);
    }

    /**
     * 容器组列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/pods")
    @ApiOperation(value = "容器组列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listOfPod(Pager pager,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kubernetesService.listOfPod(pager, context);
    }

    /**
     * 节点Node基本信息
     *
     * @param id
     * @return
     */
    @GetMapping("/nodes/overview")
    @ApiOperation(value = "节点基本信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult nodeOverview(Long id) {
        return kubernetesService.detailOfNode(id);
    }

    /**
     * 容器组基本信息
     *
     * @param name
     * @return
     */
    @GetMapping("/pods/overview")
    @ApiOperation(value = "容器组基本信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult podOverview(String name) {
        return kubernetesService.detailOfPod(name);
    }


}
