package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.DeleteIdList;
import com.bocloud.cms.entity.Metric;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.cms.intf.MetricService;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 监控指标
 *
 * @author zhangdi
 * @since May 28,2019
 * @updateDate 2022/2/21 18:01 fanzhenxi 增加CRUD功能并可与监控资源类型关联
 * @version 5.6.0-SNAPSHOT
 */
@Api(value = "监控指标", tags = "监控指标")
@RestController
@RequestMapping("/v1/metrics")
class MetricController {

    @Autowired
    private MetricService metricService;
    /**
     * 查询监控指标列表 获取到的指标为开启状态
     * @param resourceType
     * @return
     */
    @GetMapping
    @ApiOperation(value = "监控指标列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(String resourceType) {
        return metricService.list(resourceType);
    }

    /**
     * 监控配置展示指标列表
     * @param pager
     * @return
     */
    @GetMapping("/monitor/list")
    @ApiOperation(value = "监控指标列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult monitorMetricList(Pager pager) {
        return metricService.monitorMetricList(pager);
    }

    /**
     * 监控指标新增
     * @param metric
     * @param context
     * @return
     */
    @PostMapping
    @ApiOperation(value = "新增监控指标", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@RequestBody Metric metric,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.create(metric, context);
    }

    /**
     * 监控指标修改
     * @param metric
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "修改监控指标", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@RequestBody Metric metric,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.modify(metric, context);
    }

    /**
     * 监控指标删除
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除监控指标", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.remove(id, context);
    }

    /**
     * 批量删除监控指标
     * @param param
     * @param context
     * @return
     */
    @DeleteMapping
    @ApiOperation(value = "批量删除监控指标", httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult batchRemove(@RequestBody DeleteIdList param,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.batchRemove(param.getIds(), context);
    }

    /**
     * 获取该资源类型下所有status为开启的指标列表
     * @param resourceType
     * @return
     */
    @GetMapping("/state/ids")
    @ApiOperation(value = "获取该资源类型下所有状态为开启的指标列表", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult getByResourceType(String resourceType) {
        return metricService.getByResourceType(resourceType);
    }
}