package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.DeleteIdList;
import com.bocloud.cms.entity.MonitorResourceType;
import com.bocloud.cms.intf.MonitorResourceTypeService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 告警配置资源类型
 * @author fanzhenxi
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@RestController
@Api(value = "监控类型", tags = "监控类型")
@RequestMapping("/v1/monitor/resourcetype")
public class MonitorResourceTypeController {

    @Autowired
    private MonitorResourceTypeService monitorResourceTypeService;

    /**
     * 新增监控资源类型
     * @param monitorResourceType   监控类型对象
     * @param context
     * @return  成功/失败
     */
    @PostMapping
    @ApiOperation(value = "新增监控资源类型", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@RequestBody MonitorResourceType monitorResourceType,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.create(monitorResourceType, context);
    }

    /**
     * 获取监控资源类型列表
     * @param pager
     * @param context
     * @return  监控资源类型列表
     */
    @GetMapping
    @ApiOperation(value = "获取监控资源类型列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.list(pager, context);
    }

    /**
     * 修改监控资源类型
     * @param monitorResourceType   监控类型对象
     * @param context
     * @return  成功/失败
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "修改监控资源类型", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody MonitorResourceType monitorResourceType,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.modify(id,monitorResourceType, context);
    }

    /**
     * 删除监控资源类型
     * @param id       监控类型对象id
     * @param context  上下文对象
     * @return  成功/失败
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除监控资源类型", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.remove(id, context);
    }

    /**
     * 批量删除监控资源类型
     *
     * @param deleteIdList    监控类型对象id列表
     * @param context   上下文对象
     * @return 成功/失败 个数
     */
    @DeleteMapping("/remove")
    @ApiOperation(value = "批量删除监控资源类型", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult batchRemove(@RequestBody DeleteIdList deleteIdList,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.batchRemove(deleteIdList.getIds(), context.getTarget());
    }

    /**
     * 资源类型绑定监控指标操作
     * @param monitorResourceType
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "资源类型绑定监控指标", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/binding/metrics")
    public Result bindingPlugins(@RequestBody MonitorResourceType monitorResourceType,
                                 @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return monitorResourceTypeService.bindingMetrics(monitorResourceType.getMetricIds(),monitorResourceType.getResourceType(), requestContext);
    }
}
