package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Set;

/**
 * 查询prometheus监控数据
 *
 * @author zhangyf
 * @since 2019-09-06
 */

@RestController
@Api(value = "prometheus", tags = "prometheus")
@RequestMapping("/v1/prometheus")
public class PrometheusController {

    private static final Logger logger = LoggerFactory.getLogger(PrometheusController.class);

    private final PrometheusService prometheusService;

    @Autowired
    public PrometheusController(PrometheusService prometheusService) {
        this.prometheusService = prometheusService;
    }

    /**
     * prometheus监控数据查询
     *
     * @return
     */
    @GetMapping
    @ApiOperation(value = "prometheus监控数据查询", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult query(@RequestParam(value = "expr") String expr,
                               @RequestParam(value = "time", required = false) String time,
                               @RequestParam(value = "start", required = false) String start,
                               @RequestParam(value = "end", required = false) String end,
                               @RequestParam(value = Common.SIMPLE, required = false, defaultValue = "false") Boolean simple,
                               @RequestParam(value = "step", required = false) String step) {
        try {
            if (simple) {
                Object value = prometheusService.simpleQuery(expr, time);
                return new BocloudResult(true, value, "success");
            }
            List<PrometheusResult> results;
            if (StringUtils.isBlank(start)) {
                results = prometheusService.query(expr, time);
            } else {
                results = prometheusService.queryRange(expr, start, end, step);
            }
            return new BocloudResult(true, results, "success");
        } catch (RuntimeException e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    /**
     * 过滤条件集合 用于图表的下拉框
     *
     * @param expr
     * @param filter
     * @return
     */
    @GetMapping("/filter")
    @ApiOperation(value = "prometheus过滤条件集合", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult filter(@RequestParam(value = "expr") String expr,
                                @RequestParam(value = "filter") String filter) {
        Set<String> set = prometheusService.filter(expr, filter);
        return new BocloudResult(true, set, "success");
    }
}
