package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.AlarmSendPolicy;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * @author lyy
 * @Version 1.0
 * @since 2017年7月13日
 */
@RestController
@Api(value = "分发策略", tags = "分发策略")
@RequestMapping("/v1/distributions")
public class SendPolicyController {

    private final AlarmSendPolicyService alarmSendPolicyService;

    @Autowired
    public SendPolicyController(AlarmSendPolicyService alarmSendPolicyService) {
        this.alarmSendPolicyService = alarmSendPolicyService;
    }

    /**
     * 分发策略新增
     *
     * @return
     */
    @PostMapping
    @ApiOperation(value = "新增分发策略", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@RequestBody AlarmSendPolicy alarmSendPolicy,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.create(alarmSendPolicy, context);
    }

    /**
     * 分发策略修改
     *
     * @return
     */
    @PutMapping("/{id}")
    @ApiOperation(value = "修改分发策略", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@RequestBody AlarmSendPolicy alarmSendPolicy,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.modify(alarmSendPolicy, context);
    }

    /**
     * 分发策略删除
     *
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(value = "删除分发策略", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.remove(id, context);
    }

    /**
     * 获取应用列表
     *
     * @param pager
     * @param context 操作者信息
     * @return 数据结果集
     */
    @GetMapping
    @ApiOperation(value = "分发策略列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmSendPolicyService.list(pager, context);
    }

    /**
     * 分发策略详情
     *
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "分发策略详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return alarmSendPolicyService.detail(id);
    }


}
