package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.StorageService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * 华为存储监控
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@RestController
@Api(value = "存储监控", tags = "存储监控")
@RequestMapping("/v1")
public class StorageController {
    @Autowired
    private StorageService storageService;

    /**
     * 存储设备列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/storages")
    @ApiOperation(value = "存储设备列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listDevices(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return storageService.listDevices(pager, context, request);
    }

    /**
     * 硬盘列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/disks")
    @ApiOperation(value = "硬盘列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listDisks(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return storageService.listDisks(pager, context, request);
    }

    /**
     * 存储卷列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/volumes")
    @ApiOperation(value = "存储卷列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listVolumes(Pager pager, @RequestParam(value = "action", required = false) String action,
                                     @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return storageService.listVolumes(pager, context, action, request);
    }
}
