package com.bocloud.cms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.booter.config.MySqlConfiguration;
import com.bocloud.cms.booter.monitor.ComponentModel;
import com.bocloud.cms.booter.monitor.PluginsModel;
import com.bocloud.cms.booter.monitor.ServicesModel;
import com.bocloud.cms.booter.monitor.prometheus.AlertManagerInfo;
import com.bocloud.cms.booter.monitor.prometheus.PrometheusInfo;
import com.bocloud.cms.booter.monitor.prometheus.PrometheusMonitor;
import com.bocloud.cms.booter.monitor.rabbitmq.RabbitmqMonitor;
import com.bocloud.cms.booter.monitor.redis.RedisInfo;
import com.bocloud.cms.booter.monitor.redis.RedisMonitor;
import com.bocloud.cms.booter.monitor.zookeeper.ZooKeeper;
import com.bocloud.cms.booter.monitor.zookeeper.ZooKeeperMonitor;
import com.bocloud.cms.common.config.PrometheusConfig;
import com.bocloud.cms.common.utils.MathUtils;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.QueryService;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.SystemBean;
import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.License;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.RegistryService;
import com.bocloud.service.core.Service;
import com.bocloud.service.license.LicenseDaemon;
import com.bocloud.service.registry.ZooKeeperClient;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageProperties;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.client.ServiceInstance;
import org.springframework.cloud.client.discovery.DiscoveryClient;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.connection.RedisClusterNode;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisSentinelConnection;
import org.springframework.data.redis.connection.RedisServer;
import org.springframework.data.redis.connection.jedis.JedisClusterConnection;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.connection.jedis.JedisSentinelConnection;
import org.springframework.data.redis.connection.lettuce.LettuceClusterConnection;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettuceSentinelConnection;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.lang.NonNull;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author tyl
 * @time 2019/12/17 周二 9:50
 * @bugfix 2021/11/19 集群环境Redis宕机后查询系统监控失败问题 fanzx
 */
@Api(value = "系统监控", tags = "系统监控")
@RestController
@RequestMapping("/v1/system")
public class SystemController {
    private static final Logger logger = LoggerFactory.getLogger(SystemController.class);
    private static final String NODE_EXPORTER = "node_exporter";
    private static final String TELEGRAF = "telegraf";
    private static final String BLACK_BOX = "blackbox";
    private static final String MASTER = "master";
    private static final String ERROR = "error";
    private static final String WARNING = "warning";
    private static final String RUNNING = "running";
    /**
     * 声明超时时间
     */
    private static final Long TIME_OUT = (long) (1000 * 10);
    private final LicenseDaemon licenseDaemon;

    @Autowired
    private RegistryService registry;
    @Autowired
    private RabbitAdmin rabbitAdmin;
    @Autowired
    private ZooKeeperClient client;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private Environment env;
    @Autowired
    private PrometheusMonitor prometheusMonitor;
    @Autowired
    private PrometheusConfig prometheusConfig;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private QueryService queryService;
    @Autowired
    private DiscoveryClient discoveryClient;
    @Autowired
    private RegistryService registryService;
    @Autowired
    private MySqlConfiguration mySqlConfiguration;

    public SystemController(CurrentService currentService) {
        this.licenseDaemon = LicenseDaemon.instance(currentService);
    }

    /**
     * 查询主机状态统计图
     */
    @ApiOperation(value = "查询主机状态统计信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/charts")
    public BocloudResult getChars(String start, String end, String ip, String step) {
        String cpuParam = "round((1 - avg(irate(node_cpu_seconds_total{instance=\"" + ip
                + "\",mode=\"idle\"}[5m])) by (instance))*100,0.01)";
        String memParam = "round(100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes)/node_memory_MemTotal_bytes{instance=\"" + ip
                + "\"}) * 100,0.01)";
        String diskParam = "round(100 - sum(node_filesystem_free_bytes{ instance=\"" + ip
                + "\",fstype!~\"rootfs|selinuxfs|autofs|rpc_pipefs|tmpfs|udev|none|devpts|sysfs|debugfs|fuse.*\"}) "
                + "/ sum(node_filesystem_size_bytes{ instance=\"" + ip
                + "\",fstype!~\"rootfs|selinuxfs|autofs|rpc_pipefs|tmpfs|udev|none|devpts|sysfs|debugfs|fuse.*\"}) * 100,0.01)";
        String networkParam1 = "sum(round(1/1024*irate(node_network_receive_bytes_total{instance=\"" + ip
                + "\",device=~\"en.*|em.*|eth.*|ten.*|one.*\"}[5m])))";
        String networkParam2 = "sum(round(1/1024*irate(node_network_transmit_bytes_total{instance=\"" + ip
                + "\",device=~\"en.*|em.*|eth.*|ten.*|one.*\"}[5m])))";
        Map<String, Object> charsMap = new HashMap<>(16, 0.75F);
        charsMap.put("cpu", queryService.chart(Collections.singletonList(cpuParam), Collections.singletonList("cpu使用率"), start, end, "20", "%"));
        charsMap.put("mem", queryService.chart(Collections.singletonList(memParam), Collections.singletonList("内存使用率"), start, end, "20", "%"));
        charsMap.put("disk", queryService.chart(Collections.singletonList(diskParam), Collections.singletonList("磁盘使用率"), start, end, "20", "%"));
        charsMap.put("network", queryService.chart(Arrays.asList(networkParam1, networkParam2), Arrays.asList("网络流入速率", "网络流出速率"), start, end, "20", "kb/s"));
        return new BocloudResult(true, charsMap, "success");
    }

    /**
     * 查询主机状态信息
     *
     * @return 主机状态信息
     */
    @ApiOperation(value = "查询系统状态信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/servers")
    public BocloudResult servers() {
        Set<String> instances = prometheusService.filter("up{tag='cmp'}", "instance");
        List<Map<String, Object>> servers = Lists.newArrayList();
        Optional.ofNullable(instances).orElse(new HashSet<>()).stream().forEach(host -> {
            // 先判断主机是否启动
            Double value = prometheusService.simpleQuery("up{tag='cmp'}", Collections.singletonMap("instance", host));
            Map<String, Object> data = new HashMap<>(16, 0.75F);
            Double cpuUsed = prometheusService.simpleQuery("round((1 - avg(irate(node_cpu_seconds_total{mode='idle'}[5m])) by (instance))*100,0.01)", Collections.singletonMap("instance", host));
            Double memUsed = prometheusService.simpleQuery("round(100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes)/node_memory_MemTotal_bytes{}) * 100,0.01)", Collections.singletonMap("instance", host));
            Double diskUsed = prometheusService.simpleQuery("round(100 - sum(node_filesystem_free_bytes{ instance='" + host
                    + "',fstype!~'rootfs|selinuxfs|autofs|rpc_pipefs|tmpfs|udev|none|devpts|sysfs|debugfs|fuse.*'})"
                    + "/ sum(node_filesystem_size_bytes{instance='" + host
                    + "',fstype!~'rootfs|selinuxfs|autofs|rpc_pipefs|tmpfs|udev|none|devpts|sysfs|debugfs|fuse.*'}) * 100,0.01)");
            data.put("host", host);
            data.put("cpuUsed", cpuUsed);
            data.put("memUsed", memUsed);
            data.put("diskUsed", diskUsed);
            data.put("status", (1D == value) ? RUNNING : ERROR);
            servers.add(data);
        });
        return new BocloudResult(true, servers, "查询成功");
    }

    /**
     * 查询服务状态信息
     *
     * @return 返回服务状态信息
     */
    @ApiOperation(value = "查询服务状态信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/services")
    public BocloudResult service() {
        ServicesModel servicesModel = new ServicesModel();
        License catalog = licenseDaemon.content();
        List<String> services = catalog.getService();
        Integer hosts = catalog.getCatalog().getHosts();
        // 从Licenses里面获取AIO/STD
        servicesModel.setHosts(hosts);
        List<ServicesModel.Service> serviceList = Lists.newArrayList();
        if (services != null) {
            if (services.contains(BoCloudService.IMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.IMS, "IAAS服务"));
            }
            if (services.contains(BoCloudService.PMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.PMS, "PAAS服务"));
            }
            if (services.contains(BoCloudService.NMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.NMS, "网络服务"));
            }
            if (services.contains(BoCloudService.DMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.DMS, "存储服务"));
            }
            if (services.contains(BoCloudService.SMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.SMS, "系统管理服务"));
            }
            if (services.contains(BoCloudService.CMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.CMS, "监控服务"));
            }
            if (services.contains(BoCloudService.COP.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.COP, "运维服务"));
            }
            if (services.contains(BoCloudService.COS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.COS, "运营服务"));
            }
            // @version 5.6.0-SNAPSHOT服务拆分添加
            if (services.contains(BoCloudService.AMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.AMS, "资产服务"));
            }
            if (services.contains(BoCloudService.BMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.BMS, "裸金属服务"));
            }
            if (services.contains(BoCloudService.FMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.FMS, "流程服务"));
            }
            if (services.contains(BoCloudService.RMS.getName())) {
                serviceList.add(buildServiceModel(BoCloudService.RMS, "报表服务"));
            }
        }
        servicesModel.setService(serviceList);
        return new BocloudResult(true, servicesModel, "查询成功");
    }

    /**
     * 系统状态信息
     *
     * @param host 服务host地址（带端口）
     * @return 系统主机状态信息
     */
    @ApiOperation(value = "服务状态信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/status")
    public BocloudResult status(@RequestParam(value = Common.VALUE) String host) {
        HttpClient client = new HttpClient();
        String value = "http://" + host + "/status";
        Result result = client.get(value);
        return result.isSuccess() ? JSONObject.parseObject(result.getMessage(), BocloudResult.class)
                : new BocloudResult(false, result.getMessage());
    }

    /**
     * 获取插件的状态信息
     *
     * @return 插件列表
     */
    @ApiOperation(value = "查询插件状态信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/plugins")
    public BocloudResult plugins() {
        List<PluginsModel> pluginsModel = Lists.newArrayList();
        License catalog = licenseDaemon.content();
        List<String> services = catalog.getService();
        if (services != null) {
            if (services.contains(BoCloudService.WORKER.getName())) {
                pluginsModel.add(buildPluginsModel(BoCloudService.WORKER, "作业代理"));
            }
            if (services.contains(BoCloudService.PONTUS.getName())) {
                pluginsModel.add(buildPluginsModel(BoCloudService.PONTUS, "存储代理"));
            }
        }
        return new BocloudResult(true, pluginsModel, "查询成功");
    }

    /**
     * 查询组件状态信息
     *
     * @return 组件状态信息
     */
    @ApiOperation(value = "查询组件状态信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/components")
    public BocloudResult components() {
        List<ComponentModel> componentModels = Lists.newArrayList();
        String druidUrl = env.getProperty("spring.datasource.druid.url");
        if (druidUrl.contains("mysql")) {
            componentModels.add(buildComponentMysql());
        }
        componentModels.add(buildComponentPrometheus());
        componentModels.add(buildComponentZookeeper());
        componentModels.add(buildComponentRabbitmq());
        componentModels.add(buildComponentAlertManager());
        componentModels.add(buildComponentRedis());
        return new BocloudResult(true, componentModels, "查询成功");
    }

    /**
     * 查询网关状态信息
     *
     * @return 组件状态信息
     */
    @ApiOperation(value = "查询网关状态信息", httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/gateway")
    public BocloudResult gateWayCompnents() {
        List<ComponentModel> componentModels = Lists.newArrayList();
        componentModels.add(buildComponentGateway());
        return new BocloudResult(true, componentModels, "查询成功");
    }

    /**
     * 获取服务对应实例地址
     *
     * @param service 服务
     * @return 实例地址集合
     */
    private List<String> getServices(BoCloudService service) {
        try {
            List<String> serviceInstances = registry.getServices(service);
            return ListTool.isEmpty(serviceInstances) ? new ArrayList<>() : serviceInstances;
        } catch (RuntimeException e) {
            logger.error("get services error:" + e.getMessage());
            return new ArrayList<>();
        }
    }

    /**
     * 获取服务数据
     *
     * @param service 服务
     * @param alias   服务名
     * @return 服务数据
     */
    private ServicesModel.Service buildServiceModel(@NonNull BoCloudService service, @NonNull String alias) {
        List<ServicesModel.Service.Instance> data = Lists.newArrayList();
        try {
            String status = RUNNING;
            // 获取license里面规定的实例个数(从mac地址中获取)
            int instances = licenseDaemon.content().getAddress().size();
            List<String> servers = getServices(service);
            logger.info(service + " 获取到服务列表：" + servers);
            int errorNum = 0;
            for (String server : servers) {
                server = server.replaceAll("http://", "").replaceAll("https://", "");
                String state = this.registryService.getServiceState(service.getName(), server);
                SystemBean systemBean;
                ServicesModel.Service.Instance serviceInstance = new ServicesModel.Service.Instance();
                try {
                    systemBean = JSONObject.parseObject(state).toJavaObject(SystemBean.class);
                } catch (Exception e) {
                    logger.error(service + "获取服务信息失败！" + e);
                    errorNum++;
                    serviceInstance.setHost(server);
                    serviceInstance.setStatus(ERROR);
                    data.add(serviceInstance);
                    continue;
                }
                serviceInstance.setHost(systemBean.getServiceInstance());
                serviceInstance.setStatus(RUNNING);
                serviceInstance.setCpuUsed(MathUtils.round(BigDecimal.valueOf(systemBean.getJvmCpuUsage()).doubleValue() * 100d, 2));
                serviceInstance.setMemUsed(MathUtils.round(BigDecimal.valueOf(txfloat(systemBean.getJvmMemoryUsed(), systemBean.getJvmMemoryMax())).doubleValue() * 100d, 2));
                serviceInstance.setLogger(service.getName() + ".server");
                serviceInstance.setIsLeader(systemBean.getIsLeader());
                data.add(serviceInstance);
            }
            if (0 < errorNum || data.size() < instances) {
                if (data.size() <= instances) {
                    int difference = instances - data.size();
                    for (int i = 0; i < difference; i++) {
                        ServicesModel.Service.Instance errorInstance = new ServicesModel.Service.Instance();
                        errorInstance.setStatus(ERROR);
                        data.add(errorInstance);
                    }
                }
                status = WARNING;
            }
            if (errorNum >= data.size() || data.size() <= 0) {
                status = ERROR;
            }
            return new ServicesModel.Service(alias, status, data);
        } catch (Exception e) {
            logger.error("系统监控服务查询失败！" + service);
            return new ServicesModel.Service(alias, ERROR, data);
        }
    }

    /**
     * 重载方法，一个是参数只有 插件名，从Prometheus里获取其状态 另一个是有插件名，和oCloudService
     * service，从getServices获取IP，后确定其状态
     *
     * @param name 插件名
     * @return 插件数据
     */
    private PluginsModel buildPluginsModel(String name) {
        String pluginsName;
        switch (name) {
            case "采集组件":
                pluginsName = TELEGRAF;
                break;
            case "通断监控":
                pluginsName = BLACK_BOX;
                break;
            default:
                pluginsName = NODE_EXPORTER;
        }
        PluginsModel pluginsModel = new PluginsModel();
        try {
            List<PrometheusResult> results = prometheusService.query("up{job=\"" + pluginsName + "\"}");
            List<PluginsModel.Plugin> data = Lists.newArrayList();
            boolean status = !ListTool.isEmpty(results);
            // 活跃个数应该与服务器个数相同，介于 0 和
            int runningNums = Optional.ofNullable(results).orElse(new ArrayList<>()).size();
            // size之间为 warning，0 为 error
            for (PrometheusResult obj : results) {
                PluginsModel.Plugin plugin = new PluginsModel.Plugin();
                String host = String.valueOf(obj.getMetric().get("instance"));
                boolean active = "1".equals(obj.getValue()[1]);
                if (!active) {
                    runningNums--;
                    // 只要有不存活的服务器，status就是false；即不可能是running状态
                    status = false;
                }
                plugin.setStatus(active ? RUNNING : ERROR);
                if (!NODE_EXPORTER.equals(pluginsName)) {
                    plugin.setHost(host + "(" + name + ")");
                } else {
                    plugin.setHost(host);
                }
                plugin.setLogger(pluginsName + ".server");
                data.add(plugin);
            }
            pluginsModel.setName(pluginsName + "(" + data.size() + "）");
            pluginsModel.setStatus(runningNums == 0 ? ERROR : status ? RUNNING : WARNING);
            pluginsModel.setInstance(data);
            return pluginsModel;
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            pluginsModel.setName(pluginsName + "(0)");
            pluginsModel.setStatus(ERROR);
            pluginsModel.setInstance(null);
            return pluginsModel;
        }
    }

    /**
     * 重载方法，一个是参数只有 插件名，从Prometheus里获取其状态 另一个是有插件名，和BoCloudService
     * service，从getServices获取IP，后确定其状态
     *
     * @param service 插件服务
     * @param alias   插件服务名
     * @return 插件数据
     */
    private PluginsModel buildPluginsModel(@NonNull BoCloudService service, @NonNull String alias) {
        List<String> servers = getServices(service);
        PluginsModel pluginsModel = new PluginsModel();
        List<PluginsModel.Plugin> data = Lists.newArrayList();
        boolean status = true;
        if (servers.isEmpty()) {
            status = false;
        }
        // 活跃个数应该与服务器个数相同，介于 0 和 size之间为 warning，0 为 error
        int runningNums = servers.size();
        for (String server : servers) {
            PluginsModel.Plugin plugin = new PluginsModel.Plugin();
            if (service == BoCloudService.MCS) {
                //  fixBug:不兼容的类型: com.bocloud.service.core.Service无法转换为java.lang.String
                Map<String, String> serviceStatus = registry.getServiceStatus(String.valueOf(Service.create(service)));
                plugin.setStatus(serviceStatus.isEmpty() ? ERROR : RUNNING);
            } else {
//                boolean active = isActive(server);
//                if (!active) {
//                    runningNums--;
//                    status = false; // 只要有不存活的服务器，status就是false；即不可能是running状态
//                }
                plugin.setStatus(RUNNING);
            }
            server = server.replaceAll("http://", "").replaceAll("https://", "");
            if (service == BoCloudService.MCS) {
                plugin.setHost(server + "(" + alias + ")");
            } else {
                plugin.setHost(server);
            }
            plugin.setLogger(service.getName() + ".server");
            data.add(plugin);
        }
        // 将telegraf和blackbox放入EXPORTER里面，作为一个个体
        if (service == BoCloudService.MCS) {
            // 为了函数能返回两个值，放进一个集合，index 0为Boolean，index 1为Integer类型，不能打乱顺序，已经写死，后面根据索引取出
            List<Object> list = new ArrayList<>(2);
            list.add(status);
            list.add(runningNums);
            getPluginInstance("通断监控(" + data.size() + ")", list, data);
            getPluginInstance("采集组件(" + data.size() + ")", list, data);
            status = (boolean) list.get(0);
            runningNums = (int) list.get(1);
        }
        pluginsModel.setName(alias);
        String setStatus = runningNums == 0 ? ERROR : status ? RUNNING : WARNING;
        pluginsModel.setStatus(setStatus);
        pluginsModel.setInstance(data);
        return pluginsModel;
    }

    /**
     * telegraf和blackbox放入了EXPORTER，该函数为在EXPORTER中单独获取telegraf和blackbox
     *
     * @param name
     * @param list
     * @param data
     */
    private void getPluginInstance(String name, List<Object> list, List<PluginsModel.Plugin> data) {
        boolean status = (boolean) list.get(0);
        int runningNums = (int) list.get(1);
        PluginsModel model = buildPluginsModel(name);
        if (model.getInstance() != null) {
            for (PluginsModel.Plugin plugin : model.getInstance()) {
                if (RUNNING.equals(model.getStatus())) {
                    runningNums++;
                } else {
                    status = false;
                }
                data.add(plugin);
            }
        } else {
            PluginsModel.Plugin plugin = new PluginsModel.Plugin();
            plugin.setStatus(ERROR);
            plugin.setHost(name);
            status = false;
            data.add(plugin);
        }
        list.set(0, status);
        list.set(1, runningNums);
    }

    /**
     * 组件添加mysql
     *
     * @return
     */
    private ComponentModel buildComponentMysql() {
        try {
            BocloudResult result = mySqlConfiguration.dbStatus();
            logger.info("mySqlExecuteResult: " + result);
            List<ComponentModel.Component> components = Lists.newArrayList();
            int errorNum = 0;
            if (result.isSuccess()) {
                JSONObject mysqlInfo = JSONObject.parseObject(result.getData().toString());
                if (mysqlInfo.getString("mysqlInfos") != null) {
                    List<String> masterHosts = JSONArray.parseArray(mysqlInfo.getJSONArray("masterHosts").toJSONString(), String.class);
                    JSONArray mysqlInfos = mysqlInfo.getJSONArray("mysqlInfos");
                    // 主节点没有slaveStatus这个属性
                    for (int i = 0; i < mysqlInfos.size(); i++) {
                        JSONObject info = mysqlInfos.getJSONObject(i);
                        String host = info.getString("host");
                        String status;
                        if (info.getJSONArray("ThreadsGlobalStatus") != null) {
                            status = RUNNING;
                        } else {
                            status = ERROR;
                            errorNum++;
                        }
                        components.add(new ComponentModel.Component(host, status, null, masterHosts.contains(host), info));
                    }
                }
            }
            if (errorNum == components.size()) {
                return new ComponentModel("数据库", ERROR, components);
            } else if (errorNum > 0) {
                return new ComponentModel("数据库", WARNING, components);
            }
            return new ComponentModel("数据库", RUNNING, components);
        } catch (Exception e) {
            logger.error("获取数据库状态失败:" + e.getMessage(), e);
            return new ComponentModel("数据库", ERROR, Lists.newArrayList());
        }
    }

    /**
     * 组件添加Prometheus
     *
     * @return
     */
    private ComponentModel buildComponentPrometheus() {
        String addr = null;
        List<ComponentModel.Component> servers = Lists.newArrayList();
        try {
            String[] addresses = Optional.ofNullable(prometheusConfig.getNodeUrls()).orElse("").split(",");
            boolean status = true;
            if (addresses.length == 0) {
                status = false;
            }
            // 活跃个数应该与服务器个数相同，介于 0 和 size之间为 warning，0 为 error
            int runningNums = addresses.length;
            for (String address : addresses) {
                addr = address;
                PrometheusInfo prometheusInfo = new PrometheusInfo();
                try {
                    boolean conn = prometheusMonitor.isPrometheusConn(address);
                    prometheusInfo = prometheusMonitor.prometheus(address);
                    if (!conn) {
                        runningNums--;
                        // 只要有不存活的服务器，status就是false；即不可能是running状态
                        status = false;
                    }
                    servers.add(new ComponentModel.Component(address.replaceAll("https?://", ""), conn ? RUNNING : ERROR, null,
                            true, prometheusInfo));
                } catch (Exception e) {
                    logger.error("获取Prometheus组件信息失败： " + address);
                    servers.add(new ComponentModel.Component(address.replaceAll("https?://", ""), ERROR, null,
                            false, prometheusInfo));
                }
            }
            ComponentModel componentModel = new ComponentModel();
            componentModel.setName("监控中心");
            componentModel.setStatus(runningNums == 0 ? ERROR : status ? RUNNING : WARNING);
            componentModel.setInstance(servers);
            return componentModel;
        } catch (Exception e) {
            logger.error("获取Prometheus组件信息失败： " + addr);
            ComponentModel componentModel = new ComponentModel();
            componentModel.setName("监控中心");
            componentModel.setStatus(ERROR);
            componentModel.setInstance(servers);
            return componentModel;
        }
    }

    /**
     * 组件添加zookeeper
     *
     * @return
     */
    private ComponentModel buildComponentZookeeper() {
        try {
            boolean connected = client.getClient().getZookeeperClient().isConnected();
            String[] serverUrls = client.getProperties().getServerUrl().split(",");
            ComponentModel componentModel = new ComponentModel();
            List<ComponentModel.Component> servers = Lists.newArrayList();
            boolean warning = false;
            componentModel.setName("注册中心");
            for (String serverUrl : serverUrls) {
                String host = serverUrl.split(":")[0];
                String port = serverUrl.split(":")[1];
                ZooKeeper zk = ZooKeeperMonitor.getZooKeeperMonitor(host, Integer.parseInt(port));
                // 获取状态
                String status = zk.getStatus();
                String state = "";
                if (zk.getMonitorInfo() != null) {
                    // 获取角色leader，follow
                    state = zk.getMonitorInfo().getString("zk_server_state");
                }
                // 检测单个节点是否挂掉了
                if (ERROR.equals(status)) {
                    warning = true;
                }
                ZooKeeper keeper = ZooKeeperMonitor.getZooKeeperMonitor(host, Integer.parseInt(port));
                servers.add(new ComponentModel.Component(host + ":" + port, status, null,
                        serverUrls.length == 1 || "leader".equals(state), keeper));
            }
            componentModel.setInstance(servers);
            String setStatus = connected ? warning ? WARNING : RUNNING : ERROR;
            componentModel.setStatus(setStatus);
            return componentModel;
        } catch (RuntimeException e) {
            logger.error("build zookeeper component error:" + e.getMessage(), e);
            return new ComponentModel("注册中心", ERROR, null);
        }
    }

    /**
     * 组件添加rabbitMq
     *
     * @return
     */
    private ComponentModel buildComponentRabbitmq() {
        try {
            String queue = IDFactory.instance().uuid();
            String address = env.getProperty("spring.rabbitmq.addresses");
            String mqHost = env.getProperty("spring.rabbitmq.host");
            String rabbitmqHost = address != null ? address : mqHost;
            int port = rabbitAdmin.getRabbitTemplate().getConnectionFactory().getPort();
            String[] hosts = Objects.requireNonNull(rabbitmqHost).split(",");
            boolean success = false;
            try {
                rabbitAdmin.declareQueue(new Queue(queue, false, false, true));
                rabbitAdmin.declareBinding(new Binding(queue, Binding.DestinationType.QUEUE,
                        BoCloudExchange.Default.DIRECT.getExchange(), queue, null));
                String testContent = "This message is used for testing.";
                rabbitAdmin.getRabbitTemplate().send(queue, new Message(testContent.getBytes(), new MessageProperties()));
                //fix Bug Cast one of the operands of this multiplication operation to a "long"
                Message message = rabbitAdmin.getRabbitTemplate().receive(queue, TIME_OUT);
                if (message != null && message.getBody().length > 0) {
                    String content = new String(message.getBody());
                    if (testContent.equals(content)) {
                        success = true;
                    }
                }
            } catch (Exception e) {
                logger.error("detect RabbitMQ connection exception:" + e.getMessage());
            } finally {
                this.rabbitAdmin.deleteQueue(queue);
            }
            List<ComponentModel.Component> servers = Lists.newArrayList();
            for (String host : hosts) {
                String username = env.getProperty("spring.rabbitmq.username");
                String password = env.getProperty("spring.rabbitmq.password");
                // RabbitMQ的Http API端口都是 15672 但是配置文件中没有 直接用固定值
                assert username != null;
                JSONObject overview = RabbitmqMonitor.getOverview(host.split(":")[0], username, password);
                if (overview == null) {
                    if (address != null) {
                        servers.add(new ComponentModel.Component(host, ERROR, null, false, null));
                    } else {
                        servers.add(new ComponentModel.Component(host + ":" + port, ERROR, null, false, null));
                    }
                    continue;
                }
                String clusterName = overview.getString("cluster_name");
                String node = overview.getString("node");
                JSONObject nodeInfo = RabbitmqMonitor.getNodeMonitor(host.split(":")[0], username, password, node);
                JSONObject rabbitmq = new JSONObject();
                rabbitmq.put("overview", overview);
                rabbitmq.put("nodeInfo", nodeInfo);
                if (address != null) {
                    servers.add(new ComponentModel.Component(host, success ? RUNNING : ERROR, null,
                            clusterName.equals(node), rabbitmq));
                } else {
                    servers.add(new ComponentModel.Component(host + ":" + port, success ? RUNNING : ERROR, null,
                            clusterName.equals(node), rabbitmq));
                }
            }
            return new ComponentModel("消息队列", success ? RUNNING : ERROR, servers);
        } catch (RuntimeException e) {
            logger.error("build zookeeper component error:" + e.getMessage(), e);
            return new ComponentModel("消息队列", ERROR, null);
        }
    }

    /**
     * 组件添加alertManager告警分发中心
     *
     * @return
     */
    private ComponentModel buildComponentAlertManager() {
        try {
            String[] addresses = Optional.ofNullable(prometheusConfig.getAlertmanagerUrl()).orElse("").split(",");
            boolean status = true;
            if (addresses.length == 0) {
                status = false;
            }
            // 活跃个数应该与服务器个数相同，介于 0 和 size之间为 warning，0 为 error
            int runningNums = addresses.length;
            List<ComponentModel.Component> componentList = Lists.newArrayList();
            for (String address : addresses) {
                AlertManagerInfo alertManagerInfo = new AlertManagerInfo();
                try {
                    boolean conn = prometheusMonitor.isAlertManagerConn(address);
                    alertManagerInfo = prometheusMonitor.alertManager(address);
                    if (!conn) {
                        runningNums--;
                        // 只要有不存活的服务器，status就是false；即不可能是running状态
                        status = false;
                    }
                    componentList.add(new ComponentModel.Component(address.replaceAll("https?://", ""), conn ? RUNNING : ERROR,
                            null, true, alertManagerInfo));
                } catch (Exception e) {
                    logger.error("获取AlertManager组件信息失败： " + address);
                    componentList.add(new ComponentModel.Component(address.replaceAll("https?://", ""), ERROR,
                            null, false, alertManagerInfo));
                }
            }
            ComponentModel componentModel = new ComponentModel();
            componentModel.setName("告警中心");
            String setStatus = runningNums == 0 ? ERROR : status ? RUNNING : WARNING;
            componentModel.setStatus(setStatus);
            componentModel.setInstance(componentList);
            return componentModel;
        } catch (RuntimeException e) {
            logger.error("build alertManager component error:" + e.getMessage(), e);
            return new ComponentModel("告警中心", ERROR, null);
        }
    }

    /**
     * 组件添加Redis
     *
     * @return
     */
    private ComponentModel buildComponentRedis() {
        try {
            boolean success = false;
            try {
                boolean closed = redisTemplate.getRequiredConnectionFactory().getConnection().isClosed();
                if (!closed) {
                    String uuidKey = UUID.randomUUID().toString();
                    redisTemplate.opsForValue().set(uuidKey, uuidKey, 10, TimeUnit.SECONDS);
                    redisTemplate.delete(uuidKey);
                    success = true;
                }
            } catch (RuntimeException e) {
                logger.error("detect redis connection error:" + e.getMessage());
            }
            List<ComponentModel.Component> servers = Lists.newArrayList();
            RedisConnectionFactory factory = this.redisTemplate.getConnectionFactory();
            if (factory instanceof JedisConnectionFactory) {
                JedisConnectionFactory jedisConnectionFactory = (JedisConnectionFactory) factory;
                if (jedisConnectionFactory.isRedisClusterAware()) {
                    JedisClusterConnection jedisClusterConnection = (JedisClusterConnection) jedisConnectionFactory
                            .getClusterConnection();
                    Set<RedisClusterNode> clusterNodes = jedisClusterConnection.clusterGetNodes();
                    for (RedisClusterNode clusterNode : clusterNodes) {
                        RedisInfo redisInfo = RedisMonitor.getRedisMonitor(clusterNode.getHost(), Objects.requireNonNull(clusterNode.getPort()),
                                jedisConnectionFactory.getPassword());
                        servers.add(new ComponentModel.Component(clusterNode.getHost() + ":" + clusterNode.getPort(),
                                success ? RUNNING : ERROR, null, MASTER.equals(redisInfo.getReplicationInfo().getRole()),
                                redisInfo));
                    }
                } else if (jedisConnectionFactory.isRedisSentinelAware()) {
                    buildSentinelInfo(servers, success, jedisConnectionFactory.getSentinelConnection());
                } else {
                    RedisInfo redisInfo = RedisMonitor.getRedisMonitor(jedisConnectionFactory.getHostName(),
                            jedisConnectionFactory.getPort(), jedisConnectionFactory.getPassword());
                    servers.add(new ComponentModel.Component(
                            jedisConnectionFactory.getHostName() + ":" + jedisConnectionFactory.getPort(),
                            success ? RUNNING : ERROR, null, MASTER.equals(redisInfo.getReplicationInfo().getRole()),
                            redisInfo));
                }
            } else if (factory instanceof LettuceConnectionFactory) {
                LettuceConnectionFactory lettuceConnectionFactory = (LettuceConnectionFactory) factory;
                if (lettuceConnectionFactory.isClusterAware()) {
                    try {
                        LettuceClusterConnection lettuceClusterConnection = (LettuceClusterConnection) lettuceConnectionFactory
                                .getClusterConnection();
                        List<RedisClusterNode> clusterNodes = lettuceClusterConnection.clusterGetNodes();
                        for (RedisClusterNode clusterNode : clusterNodes) {
                            RedisInfo redisInfo = RedisMonitor.getRedisMonitor(clusterNode.getHost(), Objects.requireNonNull(clusterNode.getPort()),
                                    lettuceConnectionFactory.getPassword());
                            servers.add(new ComponentModel.Component(clusterNode.getHost() + ":" + clusterNode.getPort(),
                                    success ? RUNNING : ERROR, null, MASTER.equals(redisInfo.getReplicationInfo().getRole()),
                                    redisInfo));
                        }
                    } catch (Exception e) {
                        return new ComponentModel("集群缓存", ERROR, servers);
                    }
                } else if (lettuceConnectionFactory.isRedisSentinelAware()) {
                    buildSentinelInfo(servers, success, lettuceConnectionFactory.getSentinelConnection());
                } else {
                    RedisInfo redisInfo = RedisMonitor.getRedisMonitor(lettuceConnectionFactory.getHostName(),
                            lettuceConnectionFactory.getPort(), lettuceConnectionFactory.getPassword());
                    servers.add(new ComponentModel.Component(
                            lettuceConnectionFactory.getHostName() + ":" + lettuceConnectionFactory.getPort(),
                            success ? RUNNING : ERROR, null, MASTER.equals(redisInfo.getReplicationInfo().getRole()),
                            redisInfo));
                }
            }
            return new ComponentModel("缓存", RUNNING, servers);
        } catch (RuntimeException e) {
            logger.error("build redis component error:" + e.getMessage(), e);
            return new ComponentModel("缓存", ERROR, null);
        }
    }

    /**
     * 组件添加网关
     *
     * @return
     */
    private ComponentModel buildComponentGateway() {
        try {
            ComponentModel componentModel = new ComponentModel();
            componentModel.setName("服务网关");
            List<ComponentModel.Component> servers = Lists.newArrayList();
            List<String> services = discoveryClient.getServices();
            List<ServiceInstance> gwsService = new ArrayList<>();
            for (String service : services) {
                if ("gws".equalsIgnoreCase(service)) {
                    gwsService = discoveryClient.getInstances(service);
                }
            }
            Optional.ofNullable(gwsService).orElse(new ArrayList<>()).forEach(serviceInstance -> {
                Boolean status = false;
                Map<String, String> metadata = serviceInstance.getMetadata();
                if (metadata.get("instance_status") != null) {
                    status = "UP".equals(metadata.get("instance_status"));
                }
                servers.add(new ComponentModel.Component(serviceInstance.getHost() + ":" + serviceInstance.getPort(),
                        status.toString(), null, false, serviceInstance));
                componentModel.setInstance(servers);
                componentModel.setStatus(status.toString());
            });
            return componentModel;
        } catch (RuntimeException e) {
            logger.error("build gateway component error:" + e.getMessage(), e);
            return new ComponentModel("服务网关", ERROR, null);
        }
    }

    /**
     * 组装redis哨兵数据(密码得在环境里面拿)
     *
     * @param servers
     * @param success
     * @param sentinelConnection
     */
    private void buildSentinelInfo(List<ComponentModel.Component> servers, boolean success,
                                   RedisSentinelConnection sentinelConnection) {
        // 所有master节点
        if (sentinelConnection instanceof LettuceSentinelConnection) {
            LettuceSentinelConnection lettuceSentinelConnection = (LettuceSentinelConnection) sentinelConnection;
            List<RedisServer> masters = lettuceSentinelConnection.masters();
            try {
                for (RedisServer master : masters) {
                    RedisInfo masterInfo = RedisMonitor.getRedisMonitor(master.getHost(), Objects.requireNonNull(master.getPort()),
                            env.getProperty("spring.redis.password"));
                    servers.add(new ComponentModel.Component(master.getHost() + ":" + master.getPort(),
                            success ? RUNNING : ERROR, null, MASTER.equals(masterInfo.getReplicationInfo().getRole()),
                            masterInfo));
                    // 查找所有slave节点
                    List<RedisServer> slaves = lettuceSentinelConnection.slaves(Objects.requireNonNull(master.getName()));
                    for (RedisServer slave : slaves) {
                        RedisInfo slaveInfo = RedisMonitor.getRedisMonitor(slave.getHost(), Objects.requireNonNull(slave.getPort()),
                                env.getProperty("spring.redis.password"));
                        servers.add(new ComponentModel.Component(slave.getHost() + ":" + slave.getPort(),
                                success ? RUNNING : ERROR, null, MASTER.equals(slaveInfo.getReplicationInfo().getRole()),
                                slaveInfo));
                    }
                }
            }catch (Exception e){
                logger.error("组装redis哨兵数据失败：" + masters,e);
                servers.add(new ComponentModel.Component(null, ERROR, null,false,null));
            }
        } else if (sentinelConnection instanceof JedisSentinelConnection) {
            JedisSentinelConnection jedisSentinelConnection = (JedisSentinelConnection) sentinelConnection;
            List<RedisServer> masters = jedisSentinelConnection.masters();
            try {
                for (RedisServer master : masters) {
                    RedisInfo masterInfo = RedisMonitor.getRedisMonitor(master.getHost(), Objects.requireNonNull(master.getPort()),
                            env.getProperty("spring.redis.password"));
                    servers.add(new ComponentModel.Component(master.getHost() + ":" + master.getPort(),
                            success ? RUNNING : ERROR, null, MASTER.equals(masterInfo.getReplicationInfo().getRole()),
                            masterInfo));
                    // 查找所有slave节点
                    List<RedisServer> slaves = jedisSentinelConnection.slaves(Objects.requireNonNull(master.getName()));
                    for (RedisServer slave : slaves) {
                        RedisInfo slaveInfo = RedisMonitor.getRedisMonitor(slave.getHost(), Objects.requireNonNull(slave.getPort()),
                                env.getProperty("spring.redis.password"));
                        servers.add(new ComponentModel.Component(slave.getHost() + ":" + slave.getPort(),
                                success ? RUNNING : ERROR, null, MASTER.equals(slaveInfo.getReplicationInfo().getRole()),
                                slaveInfo));
                    }
                }
            }catch (Exception e){
                logger.error("组装redis哨兵数据失败：" + masters,e);
                servers.add(new ComponentModel.Component(null, ERROR, null,false,null));
            }
        }
    }

    /**
     * 除法运算，保留小数
     *
     * @param a 被除数
     * @param b 除数
     * @return 商
     * @author fzx
     */
    public Double txfloat(int a, int b) {
        try {
            if (a == 0 || b == 0) {
                return 0.0;
            }
            //设置保留位数
            DecimalFormat df = new DecimalFormat("0.00");
            String result = df.format((double) a / b);
            return Double.parseDouble(result);
        } catch (Exception e) {
            logger.error("错误除法运算！" + e);
            return 0.0;
        }
    }

}
