package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.TopService;
import com.bocloud.cms.model.LineChartResult;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * 监控TOP
 *
 * @author zhangyf
 * @since 2018-12-27
 */
@Api(value = "监控TOPN", tags = "监控TOPN")
@RestController
@RequestMapping("/v1/tops")
public class TopController {

    private static final Logger logger = LoggerFactory.getLogger(TopController.class);

    @Autowired
    private TopService topService;

    /**
     * 宿主机、云主机Top5
     *
     * @param type
     * @param vendorId
     * @param limit
     * @param context
     * @return
     */
    @GetMapping
    @ApiOperation(value = "宿主机/云主机TopN图表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult top(@RequestParam(value = "type", required = false) String type,
                             @RequestParam(value = "vendorId", required = false) Long vendorId,
                             @RequestParam(value = "limit", required = false,defaultValue = "5") Integer limit,
                             @Value(Common.REQ_CONTEXT) RequestContext context,
                             HttpServletRequest request) {
        LineChartResult chart = null;
        try {
            switch (type) {
                case "hostCpu":
                    chart = this.topService.topOfHostCpu(vendorId, limit);
                    break;
                case "hostMem":
                    chart = this.topService.topOfHostMem(vendorId, limit);
                    break;
                case "vmCpu":
                    chart = this.topService.topOfvmCpu(context, vendorId, limit, request);
                    break;
                case "vmMem":
                    chart = this.topService.topOfvmMem(context, vendorId, limit, request);
                    break;
                default:
                    throw new IllegalArgumentException("unknow type " + type);
            }
            return new BocloudResult(true, chart, "success");
        } catch (RuntimeException e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "获取Top监控数据失败:" + e.getMessage());
        }
    }
}