package com.bocloud.cms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAllocationRatio;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.cms.intf.AllocationRatioService;
import com.bocloud.cms.intf.AuthService;
import com.bocloud.cms.intf.QueryService;
import com.bocloud.cms.intf.VendorService;
import com.bocloud.cms.model.VendorActionParam;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * 平台监控
 *
 * @author 王文凯
 * @since 2019-2-15
 */
@Api(value = "监控平台", tags = "监控平台")
@RestController
@RequestMapping("/v1/vendors")
public class VendorController {

    @Autowired
    private VendorService vendorService;
    @Autowired
    private AllocationRatioService allocationRatioService;
    @Autowired
    private AuthService authService;
    @Autowired
    private QueryService queryService;

    /**
     * 监控openstack平台概览
     *
     * @param id
     * @param uuid
     * @return
     */
    @GetMapping
    @ApiOperation(value = "OpenStack平台监控概览", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult vendorOverview(@RequestParam(value = "id", required = false) Long id) {
        return queryService.vendor(id);
    }

    /**
     * 开启/关闭监控
     *
     * @param param
     * @param context
     * @return
     */
    @PatchMapping
    @ApiOperation(value = "开启/关闭平台监控", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult open(@RequestBody VendorActionParam param,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        if ("open".equals(param.getAction())) {
            return vendorService.open(context, param.getIds());
        }
        if ("close".equals(param.getAction())) {
            return vendorService.close(context, param.getIds());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 配置超分比
     *
     * @param allocationRatio
     * @return
     */
    @PostMapping("/ratio")
    @ApiOperation(value = "配置OpenStack超分比", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult saveRatio(@RequestBody MonitorAllocationRatio allocationRatio) {
        return allocationRatioService.save(allocationRatio);
    }

    /**
     * 查询超分比详情
     *
     * @param vendorId
     * @return
     */
    @GetMapping("/ratio")
    @ApiOperation(value = "查询OpenStack超分比", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult queryRatio(Long vendorId) {
        return allocationRatioService.query(vendorId);
    }

    /**
     * 创建云平台监控运维认证信息
     *
     * @param id
     * @param monitorAuth
     * @param context
     * @return
     */
    @PostMapping("/{id}/authentications")
    @ApiOperation(value = "华为云运维认证信息", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult saveAuthInfo(@PathVariable(value = Common.ID) Long id,
                                      @RequestBody MonitorAuth monitorAuth,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) throws Exception {
        return authService.save(monitorAuth, context, id);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}/authentications")
    @ApiOperation(value = "华为云运维认证信息", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return new BocloudResult(true, authService.detail(id), "查询成功");
    }

    /**
     * 平台信息下载
     *
     * @param id
     * @param uuid
     * @return
     */
    @GetMapping("/download")
    @ApiOperation(value = "平台信息下载", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult download(Long id, String uuid) {
        return queryService.download(id, uuid);
    }

    /**
     * 配置xsky prometheus信息
     *
     * @param params
     * @return
     */
    @PostMapping("/{id}/monitor")
    @ApiOperation(value = "配置xsky prometheus信息", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult updateXskyMonitoringInfo(@PathVariable(value = Common.ID) Long id, @RequestParam(value = Common.PARAMS, required = false) String params) {
        JSONObject object = JSONObject.parseObject(params);
        if (null != object) {
            vendorService.updateXskyMonitoringInfo(id, object);
            return new BocloudResult(true, "success");
        }
        return ResultTools.formatErrResult();
    }
}

