package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.IpRelation;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cms.intf.VmService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.Map;

/**
 * 云主机监控
 *
 * @author zhangdi
 * @since June 12,2019
 */
@Api(value = "云主机监控", tags = "云主机监控")
@RestController
@RequestMapping("/v1/vms")
public class VmController {

    @Autowired
    private VmService vmService;

    @GetMapping
    @ApiOperation(value = "云主机列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return vmService.list(pager, context, request);
    }
    @GetMapping("/download")
    @ApiOperation(value = "云主机列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult download(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request, HttpServletResponse response) {

        BocloudResult<Map<String, Object>> bocloudResult = vmService.listDown(pager, context, request);
        if (bocloudResult.isSuccess()) {
            String content;
            try {
                Map<String, Object> result = bocloudResult.getData();
                String fileName = result.get("fileName").toString();
                if (fileName.contains("pdf")) {
                    response.setContentType("application/pdf");
                } else {
                    response.setContentType("application/vnd.ms-excel");
                }
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String)result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @GetMapping("/{id}")
    @ApiOperation(value = "云主机资源概览", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult vmOverview(@PathVariable(value = Common.ID) Long id) {
        return vmService.vmOverview(id);
    }

    @PostMapping("/{id}/ips")
    @ApiOperation(value = "设置云主机监控IP", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult ips(@Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(value = Common.ID) Long id,
                             @RequestBody IpRelation ipRelation) {
        vmService.saveMonitorIp(context, id, ipRelation.getIp());
        return new BocloudResult(true, "设置监控IP成功");
    }

    @GetMapping("/{id}/ips")
    @ApiOperation(value = "查询云主机监控IP", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult getIps(@Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(value = Common.ID) Long id) {
        String monitorIp = vmService.queryMonitorIp(context, id);
        return new BocloudResult(true, monitorIp, "云主机监控IP查询成功");
    }

}
