package com.bocloud.cms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.cms.intf.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * xsky监控
 *
 * @author zhangyf
 * @since 2020-07-08
 */
@Api(value = "XSKY存储监控", tags = "XSKY存储监控")
@RestController
@RequestMapping("/v1/xsky")
public class XskyController {

    private final XskyBucketService xskyBucketService;
    private final XskyObjectUserService xskyObjectUserService;
    private final XskyObjectRouterService xskyObjectRouterService;
    private final XskyDiskService xskyDiskService;
    private final XskyVolumeService xskyVolumeService;
    private final XskyVolumePoolService xskyVolumePoolService;
    private final XskyHostService xskyHostService;
    private final XskyCephOsdService xskyCephOsdService;
    private final XskyFileSystemService xskyFileSystemService;

    @Autowired
    public XskyController(XskyBucketService xskyBucketService, XskyObjectUserService xskyObjectUserService, XskyObjectRouterService xskyObjectRouterService, XskyDiskService xskyDiskService, XskyVolumeService xskyVolumeService, XskyVolumePoolService xskyVolumePoolService, XskyHostService xskyHostService, XskyCephOsdService xskyCephOsdService, XskyFileSystemService xskyFileSystemService) {
        this.xskyBucketService = xskyBucketService;
        this.xskyObjectUserService = xskyObjectUserService;
        this.xskyObjectRouterService = xskyObjectRouterService;
        this.xskyDiskService = xskyDiskService;
        this.xskyVolumeService = xskyVolumeService;
        this.xskyVolumePoolService = xskyVolumePoolService;
        this.xskyHostService = xskyHostService;
        this.xskyCephOsdService = xskyCephOsdService;
        this.xskyFileSystemService = xskyFileSystemService;
    }

    /**
     * 服务器列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/hosts")
    @ApiOperation(value = "服务器列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult hosts(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyHostService.list(pager, context);
    }


    /**
     * 对象用户列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/object/user")
    @ApiOperation(value = "对象用户列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult objectUsers(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyObjectUserService.list(pager, context);
    }

    /**
     * 对象路由列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/object/router")
    @ApiOperation(value = "对象路由列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult objectRouters(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyObjectRouterService.list(pager, context);
    }

    /**
     * 存储桶列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/buckets")
    @ApiOperation(value = "存储桶列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult buckets(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyBucketService.list(pager, context);
    }

    /**
     * 存储池列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/storages/pools")
    @ApiOperation(value = "存储池列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult volumePools(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyVolumePoolService.list(pager, context);
    }

    /**
     * 硬盘列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/ceph/osds")
    @ApiOperation(value = "硬盘列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult osds(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyCephOsdService.list(pager, context);
    }

    /**
     * 缓存磁盘列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/disks")
    @ApiOperation(value = "缓存磁盘列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult disks(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyDiskService.list(pager, context);
    }

    /**
     * 存储卷列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/volumes")
    @ApiOperation(value = "存储卷列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult volumes(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyVolumeService.list(pager, context);
    }

    /**
     * 文件系统列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping("/fileSystems")
    @ApiOperation(value = "文件系统列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult fileSystems(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return xskyFileSystemService.list(pager, context);
    }


}
