package com.bocloud.cms.booter.listener;

import java.io.File;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.OperateResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;

/**
 * 宿主机安装bocloud_expoter Listener
 *
 * @author zhangyf
 * @version 1.0
 * @since 2019-11-15
 */
@Component
public class ExporterInstallListener implements MessageListener {

    private Logger logger = LoggerFactory.getLogger(ExporterInstallListener.class);
    @Autowired
    StorageConfig storageConfig;
    @Autowired
    ImsServerService imsServerService;
    @Autowired
    ImsCloudVendorService imsCloudVendorService;
    @Autowired
    AmqpTemplate amqpTemplate;
    @Value("${bocloud.exporter.port:6003}")
    private int port;

    @RabbitListener(queues = "exporter.install.event", admin = "rabbitAdmin", autoStartup = "true")
    @Override
    public void onMessage(Message message) {
        try {
            JSONObject jsonObject = JSONObject.parseObject(new String(message.getBody()));
            Long hostId = jsonObject.getLong("serverId");
            //TODO 参数requestContext待验证
            RequestContext context = (RequestContext) jsonObject.get("requestContext");
            BocloudResult<Server> server = imsServerService.detail(hostId);
            BocloudResult<CloudVendor> vendor = imsCloudVendorService.detail(server.getData().getVendorId());
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("host_id", String.valueOf(hostId));
            labels.put("hostname", server.getData().getName());
            labels.put("vendor_id", String.valueOf(vendor.getData().getId()));
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(server.getData().getManageIp() + ":" + port));
            config.put("labels", labels);
            JSONArray array = new JSONArray();
            array.add(config);
            FileUtils.write(new File(storageConfig.getOpenstackDir(), "server_" + hostId + ".json"), array.toJSONString(), "UTF-8");

            //给前端发送消息以改变安装状态
            OperateResult operateResult = new OperateResult(true, "agent安装成功", "exporter.install.success", null,
                    OperateResult.OperateCategory.SiteMessage, context);
            this.amqpTemplate.convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#",
                    operateResult);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }
}
