package com.bocloud.cms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.*;
import com.bocloud.cms.service.MonitoringAbleRegistry;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.dms.entity.*;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.entity.hmc.HmcPartition;
import com.bocloud.ims.entity.hmc.HmcServer;
import com.bocloud.ims.entity.kubernetes.KubeNode;
import com.bocloud.ims.entity.kubernetes.KubePod;
import com.bocloud.ims.enums.CloudProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 资源（平台、宿主机、云主机）被删除后删除监控相关内容
 *
 * @author zhangyf
 * @since 2019年6月3日
 */
@Component
public class ResourceDeletedListener implements MessageListener {

    private Logger logger = LoggerFactory.getLogger(ResourceDeletedListener.class);

    @Autowired
    VendorService vendorService;
    @Autowired
    ImsCloudVendorService imsCloudVendorService;
    @Autowired
    TelegrafService telegrafService;
    @Autowired
    AlarmService alarmService;
    @Autowired
    VmService vmService;
    @Autowired
    HostService hostService;
    @Autowired
    KubernetesService kubernetesService;
    @Autowired
    StorageService storageService;
    @Autowired
    HmcService hmcService;
    @Autowired
    XskyHostService xskyHostService;

    /**
     * @param message {
     *                "type":"CloudServer"|"Server"|"CloudVendor"
     *                "data": entity对象
     *                }
     */
    @RabbitListener(queues = "cmp.resource.deleted.queue", admin = "rabbitAdmin", autoStartup = "true")
    @Override
    public void onMessage(Message message) {
        try {
            logger.info(message.toString());
            String content = new String(message.getBody());
            logger.info("cmp.resource.deleted.queue message:", content);
            JSONObject jsonObject = JSONObject.parseObject(content);
            String type = jsonObject.getString("type");
            logger.info("receive " + type + " deleted message");
            // 云主机
            if (CloudServer.class.getSimpleName().equals(type)) {
                CloudServer cloudServer = jsonObject.getObject("data", CloudServer.class);
                switch (CloudProvider.valueOf(cloudServer.getVendorType())) {
                    case VMWARE:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_VCENTER_VM.name());
                        break;
                    case OPENSTACK:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_OPENSTACK_VM.name());
                        break;
                    case MANAGEONE:
                    case FUSIONCLOUD:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name());
                        break;
                    case HUAWEI:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_HUAWEI_VM.name());
                        break;
                    case TIANYI:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_TIANYI_VM.name());
                        break;
                    case ALIYUN:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_ALIYUN_VM.name());
                        break;
                    case H3C:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_H3C_VM.name());
                        break;
                    case SMARTX:
                        vmService.destroy(cloudServer, MonitorResourceType.MONITOR_SMARTX_VM.name());
                        break;
                    default:
                        break;
                }
                // 宿主机
            } else if (Server.class.getSimpleName().equals(type)) {
                Server server = jsonObject.getObject("data", Server.class);
                if (CloudProvider.XSKY.name().equalsIgnoreCase(server.getVendorType())) {
                    xskyHostService.destroy(server);
                } else {
                    hostService.destroy(server);
                }
                // 平台
            } else if (CloudVendor.class.getSimpleName().equals(type)) {
                CloudVendor vendor = jsonObject.getObject("data", CloudVendor.class);
                logger.info("vendor id: " + vendor.getId());
                if (!CloudProvider.VMWARE.name().equals(vendor.getType())
                        && !CloudProvider.OPENSTACK.name().equals(vendor.getType())
                        && !CloudProvider.SMIS_STORAGE.name().equals(vendor.getType())
                        && !CloudProvider.XSKY.name().equals(vendor.getType())
                        && !CloudProvider.ALIYUN.name().equals(vendor.getType())
                        && !CloudProvider.HUAWEI.name().equals(vendor.getType())
                        && !CloudProvider.TIANYI.name().equals(vendor.getType())
                        && !CloudProvider.SMARTX.name().equals(vendor.getType())
                        && !CloudProvider.AZURE.name().equals(vendor.getType())
                        && !CloudProvider.AWS.name().equals(vendor.getType())
                        && !CloudProvider.HMC.name().equals(vendor.getType())
                        && !CloudProvider.H3C.name().equals(vendor.getType())) {
                    return;
                }
                String projectName = JSONObject.parseObject(vendor.getAuthentication()).getString("projectName");
                if (CloudProvider.SMIS_STORAGE.name().equals(vendor.getType()) && !"HUAWEI".equals(projectName)) {
                    //这里只处理华为存储
                    return;
                }
                if (CloudProvider.SMIS_STORAGE.name().equals(vendor.getType()) && "HUAWEI".equals(projectName)) {
                    //TODO
                } else {
                    if (CloudProvider.VMWARE.name().equals(vendor.getType())) {
                        telegrafService.deleteConfigFile(null, vendor);
                    }
                }
                if (CloudProvider.XSKY.name().equals(vendor.getType())) {
                    vendorService.closeXsky(vendor);
                }
                alarmService.solveByVendor(vendor.getId(), 1L, "系统解决");
            } else if (Storage.class.getSimpleName().equals(type)) {
                Storage storage = jsonObject.getObject("data", Storage.class);
                storageService.destroy(storage, MonitorResourceType.MONITOR_STORAGE_HUAWEI_STORAGE.name());
            } else if (Volume.class.getSimpleName().equals(type)) {
                Volume volume = jsonObject.getObject("data", Volume.class);
                if (CloudProvider.XSKY.name().equalsIgnoreCase(volume.getVendorType())) {
                    MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_VOLUME).destroy(jsonObject.getObject("data", Volume.class));
                } else if (CloudProvider.SMIS_STORAGE.name().equalsIgnoreCase(volume.getVendorType())) {
                    storageService.destroy(volume, MonitorResourceType.MONITOR_STORAGE_HUAWEI_VOLUME.name());
                }
            } else if (Disk.class.getSimpleName().equals(type)) {
                Disk disk = jsonObject.getObject("data", Disk.class);
                CloudVendor vendor = imsCloudVendorService.detail(disk.getVendorId()).getData();
                if (CloudProvider.XSKY.name().equalsIgnoreCase(vendor.getType())) {
                    MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_DISK).destroy(jsonObject.getObject("data", Volume.class));
                } else if (CloudProvider.SMIS_STORAGE.name().equalsIgnoreCase(vendor.getType())) {
                    storageService.destroy(disk, MonitorResourceType.MONITOR_STORAGE_HUAWEI_DISK.name());
                }
            } else if (KubeNode.class.getSimpleName().equals(type)) {
                KubeNode kubeNode = jsonObject.getObject("data", KubeNode.class);
                kubernetesService.destroy(kubeNode, MonitorResourceType.MONITOR_KUBERNETES_NODE.name());
            } else if (KubePod.class.getSimpleName().equals(type)) {
                KubePod kubePod = jsonObject.getObject("data", KubePod.class);
                kubernetesService.destroy(kubePod, MonitorResourceType.MONITOR_KUBERNETES_POD.name());
            } else if (HmcServer.class.getSimpleName().equals(type)) {
                HmcServer hmcServer = jsonObject.getObject("data", HmcServer.class);
                hmcService.destroy(hmcServer, MonitorResourceType.MONITOR_HMC_SERVER.name());
            } else if (HmcPartition.class.getSimpleName().equals(type)) {
                HmcPartition partition = jsonObject.getObject("data", HmcPartition.class);
                hmcService.destroy(partition, MonitorResourceType.MONITOR_HMC_SERVER.name());
            } else if (ObjectUser.class.getSimpleName().equals(type)) {
                MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_OBJECT_USER).destroy(jsonObject.getObject("data", ObjectUser.class));
            } else if (ObjectRouter.class.getSimpleName().equals(type)) {
                MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_OBJECT_ROUTER).destroy(jsonObject.getObject("data", ObjectRouter.class));
            } else if (Bucket.class.getSimpleName().equals(type)) {
                MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_OBJECT_BUCKET).destroy(jsonObject.getObject("data", Bucket.class));
            } else if (CephOsd.class.getSimpleName().equals(type)) {
                MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_CEPH_OSD).destroy(jsonObject.getObject("data", CephOsd.class));
            } else if (VolumePool.class.getSimpleName().equals(type)) {
                MonitoringAbleRegistry.getInstance(MonitorResourceType.MONITOR_XSKY_VOLUME_POOL).destroy(jsonObject.getObject("data", VolumePool.class));
            }
        } catch (Exception e) {
            logger.error("资源被删除后删除关联告警规则数据失败：" + e.getMessage(), e);
        }
    }
}
