package com.bocloud.cms.booter.listener;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;

/**
 * 回收租户云主机
 *
 * @author zhangyf
 * @since 2019-12-05
 */
@Component
public class VmReclaimListener implements MessageListener {

    private Logger logger = LoggerFactory.getLogger(VmReclaimListener.class);


    /**
     * @param message {
     *                "data": 云主机entity对象
     *                }
     */
    @RabbitListener(queues = "cmp.vm.reclaim.queue", admin = "rabbitAdmin", autoStartup = "true")
    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            JSONObject jsonObject = JSONObject.parseObject(content);
            CloudServer cloudServer = jsonObject.getObject("data", CloudServer.class);
            MonitorResourceType resourceType;
            switch (CloudProvider.valueOf(cloudServer.getVendorType())) {
                case VMWARE:
                    resourceType = MonitorResourceType.MONITOR_VCENTER_VM;
                    break;
                case OPENSTACK:
                    resourceType = MonitorResourceType.MONITOR_OPENSTACK_VM;
                    break;
                case MANAGEONE:
                case FUSIONCLOUD:
                    resourceType = MonitorResourceType.MONITOR_FUSIONCLOUD_VM;
                    break;
                case TIANYI:
                    resourceType = MonitorResourceType.MONITOR_TIANYI_VM;
                    break;
                case HUAWEI:
                    resourceType = MonitorResourceType.MONITOR_HUAWEI_VM;
                    break;
                case H3C:
                    resourceType = MonitorResourceType.MONITOR_H3C_VM;
                    break;
                case ALIYUN:
                    resourceType = MonitorResourceType.MONITOR_ALIYUN_VM;
                    break;
                default:
                    return;
            }
            //TODO
        } catch (Exception e) {
            logger.error("failed to delete rule relation data after reclaim tenant virtual machine：{}", ExceptionUtils.getStackTrace(e));
        }
    }
}
