package com.bocloud.cms.booter.scheduler;

import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.AlarmSendPolicy;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.intf.AlarmNoticeService;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.repository.RuleGroupRepository;
import com.bocloud.cms.repository.RuleRepository;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.service.core.CurrentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 监控告警分发定时任务
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月11日
 */
@Component
public class AlarmScheduler {

    @Autowired
    private CurrentService service;
    @Autowired
    private AlarmService alarmService;
    @Autowired
    private AlarmSendPolicyService alarmSendPolicyService;
    @Autowired
    private AlarmNoticeService alarmNoticeService;
    @Autowired
    private RuleRepository ruleRepository;
    @Autowired
    private RuleGroupRepository ruleGroupRepository;

    private static Logger logger = LoggerFactory.getLogger(AlarmScheduler.class);

    /**
     * 查询所有未确认并且超出未处理时间的告警消息
     */
    @Scheduled(cron = "${alarm.dispatch:0 0/1 * * * ?}")
    protected void execute() {
        if (service.isLeader()) {
            // 本次扫描到的所有还未解决并且需要发送通知的告警
            List<Alarm> alarms = this.alarmService.unSolvesForSendNotice();
            for (Alarm alarm : alarms) {
                Rule rule = ruleRepository.get(Rule.class, alarm.getRuleId());
                RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, rule.getGroupId());
                BocloudResult policyResult = alarmSendPolicyService.detail(ruleGroup.getPolicyId());
                if (policyResult.isFailed()) {
                    logger.error("query send policy error,policy id is" + ruleGroup.getPolicyId());
                    return;
                }
                AlarmSendPolicy policy = (AlarmSendPolicy) policyResult.getData();
                alarmNoticeService.sendNotice(alarm, policy, false);
            }
        }
    }

}
