package com.bocloud.cms.common.utils;

import com.bocloud.cms.model.LabelParam;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * prometheus监控工具类
 *
 * @author zhangyf
 * @since 2018-12-27
 */
public class PrometheusUtils {

    private PrometheusUtils() {
    }

    /**
     * @param expr
     * @param params
     * @return
     */
    public static String putParams(String expr, Map<String, String> params) {
        if (params == null || params.isEmpty()) {
            return expr;
        }
        String endParam = getEndParam(expr);
        if (endParam != null && endParam.contains("{")) {
            endParam = putParams(endParam, params);
        }
        if (expr.contains("{")) {
            expr = expr.substring(0, expr.indexOf('{'));
        }
        StringBuilder builder = new StringBuilder();
        builder.append("{");
        params.forEach((key, value) -> builder.append(",").append(key).append("=").append("'").append(value).append("'"));
        expr += builder.toString().replaceFirst(",", "");

        if (null != endParam) {
            expr += endParam;
        } else {
            expr += '}';
        }
        return expr;
    }

    /**
     * @param expr
     * @param params
     * @return
     */
    public static String putParams(String expr, List<LabelParam> params) {
        if (params == null || params.isEmpty()) {
            return expr;
        }
        String endParam = getEndParam(expr);
        if (endParam != null && endParam.contains("{")) {
            endParam = putParams(endParam, params);
        }
        if (expr.contains("{")) {
            expr = expr.substring(0, expr.indexOf('{'));
        }
        StringBuilder builder = new StringBuilder();
        builder.append("{");
        params.forEach(param -> builder.append(",").append(param.getLabel()).append(param.getOperator().getOperator()).append("'").append(param.getValue()).append("'"));
        expr += builder.toString().replaceFirst(",", "");

        if (null != endParam) {
            expr += endParam;
        } else {
            expr += '}';
        }
        return expr;
    }

    /**
     * 全局告警恢复拼接表达式
     *  全局表达式可能为vsphere_cpu_usage[5m] 或者为 vsphere_ram_usage{xxx=''}[5m]
     *  不同的表达式要做对应的修改语句
     * @param expr
     * @param params
     * @param resourceType
     * @return
     */
    public static String alarmRecoverPutParams(String expr, Map<String, String> params, String resourceType) {
        StringBuilder builder = new StringBuilder(expr);

        // openstackService特殊处理
        if ("MONITOR_OPENSTACK_SERVICE".equalsIgnoreCase(resourceType)){
            int size = params.size();
            for (String key : params.keySet()) {
                builder.append("{").append(key).append("=").append("'").append(params.get(key)).append("'").append("}");
            }
            return builder.toString();
        }

        if (builder.toString().contains("{")){
            StringBuilder stringBuilder = new StringBuilder(",");
            int size = params.size();
            for (String key : params.keySet()) {
                stringBuilder.append(key).append("=").append("'").append(params.get(key)).append("'");
                if (--size>0){
                    stringBuilder.append(",");
                }
            }
            builder.insert(builder.indexOf("}"), stringBuilder);
        }else if (builder.toString().contains("[")){
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("{");
            int size = params.size();
            for (String key : params.keySet()) {
                stringBuilder.append(key).append("=").append("'").append(params.get(key)).append("'");
                if (--size>0){
                    stringBuilder.append(",");
                }
            }
            stringBuilder.append("}");
            builder.insert(builder.indexOf("["), stringBuilder);
        }
        return builder.toString();
    }

    /**
     * 截取尾部参数 即表达示“{”后的参数
     *
     * @param expr
     * @return
     */
    private static String getEndParam(String expr) {
        if (!expr.contains("{")) {
            return null;
        }
        String endParam = expr.substring(expr.indexOf('{') + 1);
        return "," + endParam;
    }
}
