package com.bocloud.cms.common.utils;

import ch.ethz.ssh2.ChannelCondition;
import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;

/**
 * @author zhangyf
 * @since 2021-06-04
 */
@Data
public class RemoteShellExecutor implements Closeable {

    private Connection conn;
    private String host;
    private int port;
    private String username;
    private String password;
    private boolean login = false;

    private static final int TIME_OUT = 1000 * 5 * 60;

    public RemoteShellExecutor(String host, int port, String username, String pasword) {
        this.host = host;
        this.username = username;
        this.password = pasword;
        this.port = port;
    }

    @Override
    public void close() throws IOException {
        if (conn != null) {
            conn.close();
        }
    }

    /**
     * 登录
     *
     * @return
     * @throws IOException
     */
    private boolean login() throws IOException {
        conn = new Connection(host, port);
        conn.connect();
        login = conn.authenticateWithPassword(username, password);
        return login;
    }


    /**
     * 执行脚本
     *
     * @param cmd
     * @return
     * @throws Exception
     */
    public void execute(String cmd) throws Exception {
        InputStream stdErr = null;
        try {
            if (login || login()) {
                Session session = conn.openSession();
                session.execCommand(cmd);
                stdErr = new StreamGobbler(session.getStderr());
                String outErr = processStream(stdErr, Charset.defaultCharset().toString());
                session.waitForCondition(ChannelCondition.EXIT_STATUS, TIME_OUT);
                session.close();
                session.getExitStatus();
                if (StringUtils.isNotBlank(outErr)) {
                    throw new Exception("执行远程命令失败：" + outErr);
                }
            } else {
                throw new Exception("登录远程机器失败" + host);
            }
        } catch (Exception e) {
            throw e;
        } finally {
            if (stdErr != null) {
                stdErr.close();
            }
        }
    }

    private String processStream(InputStream in, String charset) throws Exception {
        byte[] buf = new byte[1024];
        StringBuilder sb = new StringBuilder();
        while (in.read(buf) != -1) {
            sb.append(new String(buf, charset));
        }
        return sb.toString();
    }

}