
package com.bocloud.cms.entity;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.model.BlockPolicy;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.bocloud.sms.entity.User;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 告警分发策略类
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月11日
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Table("monitor_alarm_send_policy")
public class AlarmSendPolicy extends Generic {

    /**
     * 规则ID
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 策略名称
     */
    @Column("name")
    private String name;
    /**
     * 超出未处理时间
     */
    @Column("wait_time")
    private Long waitTime;
    /**
     * 紧急告警级别
     */
    @Column("is_urgent")
    private Boolean urgent;
    /**
     * 重要告警级别
     */
    @Column("is_primary")
    private Boolean primary;
    /**
     * 次要告警级别
     */
    @Column("is_secondary")
    private Boolean secondary;
    /**
     * 提示告警级别
     */
    @Column("is_remind")
    private Boolean remind;
    /**
     * 发送站内信
     */
    @Column("is_message")
    private Boolean message;
    /**
     * 发送邮件
     */
    @Column("is_email")
    private Boolean email;

    /**
     * 企业微信
     */
    @Column("is_weixin")
    private Boolean weixin;
    /**
     * 钉钉
     */
    @Column("is_dingding")
    private Boolean dingding;
    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonDeserialize(using = DateDeserializer.class)
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;
    /**
     * 创建者id
     */
    @Column("creator_id")
    private Long createId;
    /**
     * 修改者id
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 是否启用屏蔽策略
     */
    @Column("is_action")
    private Boolean action;
    /**
     * 原生告警分发策略
     */
    @Column("native_alarm")
    private Boolean nativeAlarm;
    /**
     * 屏蔽策略
     */
    @Column("block_policy")
    private String stringBlockPolicy;
    @IgnoreAll
    @Column("block_policy_ignore")
    private List<BlockPolicy> blockPolicy;
    @Column("tenant_id")
    private Long tenantId;
    @Column("role_ids")
    private String roleIds;
    /**
     * 所有分发人
     */
    @IgnoreAll
    private List<Long> receiverIds;
    /**
     * 所有分发人的对象结构
     */
    @IgnoreAll
    private List<User> receiverInfos;
    @IgnoreAll
    private List<RequestContext> receivers;

    public List<BlockPolicy> getBlockPolicy() {
        if (StringUtils.isNotBlank(stringBlockPolicy)) {
            return JSONObject.parseArray(stringBlockPolicy).toJavaList(BlockPolicy.class);
        }
        return new ArrayList<>();
    }

    public void setBlockPolicy(List<BlockPolicy> blockPolicy) {
        // 引用问题, 在setBlockPolicy中向stringBlockPolicy赋值
        this.stringBlockPolicy = JSONObject.toJSONString(blockPolicy);
        this.blockPolicy = blockPolicy;
    }

}
