package com.bocloud.cms.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 告警发送模板
 *
 * @author fzx
 * @Version 1.0
 * @since 2021年5月7日
 */
@Data
@Table("monitor_alarm_template")
@AllArgsConstructor
@NoArgsConstructor
public class AlarmTemplate extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 资源类型
     */
    @Column("resource_type")
    private String resourceType;

    /**
     * Email(告警模板)
     */
    @Column("email")
    private String email;

    /**
     * Email(恢复模板)
     */
    @Column("email_recovery")
    private String emailRecovery;

    /**
     * 企业微信(告警模板)
     */
    @Column("weixin")
    private String weixin;

    /**
     * 企业微信(恢复模板)
     */
    @Column("weixin_recovery")
    private String weixinRecovery;

    /**
     * 钉钉(告警模板)
     */
    @Column("dingding")
    private String dingding;

    /**
     * 钉钉(恢复模板)
     */
    @Column("dingding_recovery")
    private String dingdingRecovery;

    /**
     * 站内信(告警模板)
     */
    @Column("message")
    private String message;

    /**
     * 站内信(恢复模板)
     */
    @Column("message_recovery")
    private String messageRecovery;

    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;

    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;

    /**
     * 租户ID
     */
    @Column("tenant_id")
    private Long tenantId;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * 是否删除(1为删除)
     */
    @Column("is_deleted")
    private Boolean isDeleted;

    @IgnoreAll
    private String createName;
}
