package com.bocloud.cms.entity;

import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 监控Rule实体类
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@EqualsAndHashCode(callSuper = false)
@Data
@Table("monitor_rule")
public class Rule extends Generic {

    /**
     * 主键
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 规则组ID 对应metric_rule_group主键
     */
    @Column("group_id")
    private Long groupId;

    /**
     * 监控指标id 对应monitor_metric表主键
     */
    @Column("metric_id")
    private Long metricId;

    /**
     * 统计函数( max/min/avg)
     */
    @Column("fun")
    private String fun;

    /**
     * 统计时长
     */
    @Column("interval_time")
    private String intervalTime;

    /**
     * 持续时长
     */
    @Column("for1")
    private String for1;

    /**
     * 比较符 == != > < >= <=
     */
    @Column("operator")
    private String operator;

    /**
     * 阈值
     */
    @Column("threshold")
    private String threshold;

    /**
     * 告警级别
     *  紧急，重要，次要，提示
     */
    @Column("level")
    private String level;

    /**
     * metirc监控指标名称
     */
    @IgnoreAll
    private String name;

    /**
     * metircs分组标签
     */
    @IgnoreAll
    private String tag;

    /**
     * 单位
     */
    @IgnoreAll
    private String unit;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * 是否已删除
     */
    @Column("is_deleted")
    private Boolean deleted;

}