package com.bocloud.cms.model;

import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 折线图
 *
 * @param
 * @author zhangyf
 * @since 2018-08-10
 */
@EqualsAndHashCode(callSuper = false)
@Data
public class LineChartResult {

    /**
     * key列表
     */
    private List<String> keys;
    /**
     * 对应的value数组
     */
    private Values[] values;
    /**
     * 单位
     */
    private String unit;
    private Map<String, TopParams> topParams;

    public LineChartResult(List<String> keys) {
        super();
        this.keys = keys;
    }

    public LineChartResult(List<String> keys, Values[] values) {
        super();
        this.keys = keys;
        this.values = values;
    }

    public LineChartResult(List<String> keys, Values[] values, Map<String, TopParams> topParams) {
        super();
        this.keys = keys;
        this.values = values;
        this.topParams = topParams;
    }

    @EqualsAndHashCode(callSuper = false)
    @Data
    public static class Values {

        private String name;
        private String stack;
        private List<Object> data;

        public Values(String name) {
            super();
            this.name = name;
            this.data = new ArrayList<>();
        }

        public Values(String name, List<Object> data) {
            super();
            this.name = name;
            this.data = data;
        }

        public Values(String name, List<Object> data, String stack) {
            super();
            this.name = name;
            this.data = data;
            this.stack = stack;
        }

    }

    /**
     * 折线图传参
     */
    @EqualsAndHashCode(callSuper = false)
    @Data
    public static class LineChartParam {

        /**
         * 表达式
         */
        private String expr;
        /**
         * 开始时间
         */
        private String start;
        /**
         * 结束时间
         */
        private String end;
        /**
         * 时间间隔
         */
        private String step;
        /**
         * 指标名称
         */
        private String name;
    }

    /**
     * top5详情显示所需参数
     */
    @EqualsAndHashCode(callSuper = false)
    @Data
    public static class TopParams {
        /**
         * 主机或云主机id
         */
        private Long id;
        /**
         * 唯一标识符：uuid或instanceId或ip etc;
         */
        private String identifier;
        /**
         * 机器类型 VMWARE or OPENSTACK
         */
        private String type;
        private Long vendorId;
        /**
         * 主机名称
         */
        private String hostName;
    }


}
