package com.bocloud.ims.entity;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.dms.entity.Disk;
import com.bocloud.entity.bean.GenericEntity;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 物理机器类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.02.26
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class Server extends GenericEntity {

    /**
     * ID
     */
    private Long id;
    /**
     * 过期时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredTime;
    /**
     * 资产类别
     */
    private String assetCategory;
    /**
     * 类别
     */
    private String category;
    private String catalog;
    /**
     * UUID
     */
    private String uuid;
    /**
     * 管理IP
     */
    private String manageIp;
    /**
     * 数据中心ID
     */
    private Long dcId;
    /**
     * 机房ID
     */
    private Long roomId;
    /**
     * 机柜ID
     */
    private Long rackId;
    private String hostname;
    private String serviceIp;
    private Long tenantId;
    private Long vendorId;
    /**
     * 连接状态
     */
    private Boolean connected;
    /**
     * 维护模式
     */
    private Boolean maintained;
    /**
     * 启用ipv6
     */
    private Boolean ipv6Enabled;
    /**
     * 电源状态
     */
    private String powerState;
    private Long clusterId;
    private Long vdcId;
    /**
     * 资源ID
     */
    private String resourceId;
    private boolean accurated;
    private Long rootDiskId;
    private String cpuModel;
    private String modelName;
    /**
     * WINDOWS, LINUX
     */
    /**
     * 系统类别
     */
    private String osCategory;
    /**
     * CENTOS, UBUNTU, REDHAT, ESXI, WINDOWSSERVER, SUSE
     */
    /**
     * 操作系统详细信息
     */
    private String os;
    /**
     * 资源分组
     */
    private String resourceGroup;
    private String vmotionIp;
    private Boolean exporterInstalled;

    private String resourceGroupName;
    /**
     * 所属资源池
     */
    private String groupName;
    /**
     * 云供应商名称
     */
    private String vendorName;
    /**
     * 云供应商类型
     */
    private String vendorType;

    private String roomname;

    private String rackname;

    private String dcname;

    private String clusterName;

    private String vdcName;

    private List<String> stores;
    /**
     * 云主机
     */
    private List<CloudServer> servers;

    private List<DataStore> dataStores;
    /**
     * 宿主机（物理机）上的云主机数量
     */
    private Integer vmNum;
    /**
     * 宿主机（物理机）上的云主机数量
     */
    private Integer templateNum;
    /**
     * 数据存储数量
     */
    private Integer datastoreNum;

    private ServerConfig config;
    /**
     * 全局唯一标识
     */
    private String guid;

    /** 物理装机相关开始 */
    /**
     * cop的地址信息
     * <p>
     * 192.168.2.85:18084
     */
    private String instance;
    /**
     * 该节点在rackhd里面的id
     * <p>
     * 5c3e9bd5c324b5b4163c3643
     */
    private String rackhdNodeId;
    /**
     * 装机使用的image id
     */
    private Long imageId;
    private String imageName;
    private String imageIsoName;
    /**
     * 发现任务开始的时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date discoverTime;
    /**
     * 该节点是否是被rackhd发现过
     * <p>
     * 如果被发现过，catalog信息都是ok的
     * <p>
     * 如果没有被发现过，只提供一部分功能，如设备监控
     */
    private Boolean discovered;
    /**
     * 发现该物理时，使用的dhcp网卡的mac地址
     * <p>
     * f8:bc:12:3e:dc:64
     */
    private String identifier;
    /**
     * 物理装机相关结束
     */

    /**
     * 角色
     */
    private String role;
    /**
     * 运行的服务
     */
    private String services;
    /**
     * 服务器时间差，毫秒
     */
    private Long clockDiff;
    /**
     * 标签
     */
    private String resourceLabel;
    private Boolean reboot;
    /**
     * CPU可分配
     */
    private Integer cpuDistributable;
    /**
     * 内存可分配
     */
    private Integer memDistributable;
    /**
     * cpu剩余
     */
    private Long cpuRemainder;
    /**
     * 内存剩余
     */
    private Long memoryRemainder;
    /**
     * 磁盘剩余
     */
    private Long diskRemainder;
    private Disk rootDisk;
    private Integer diskCnt;
    private String iqn;
    private List<String> roles;
}
