package com.bocloud.sms.entity;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.bean.GenericEntity;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 租户类
 *
 * @author weiwei
 * @version 1.0
 * @since 2016.08.04
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class Tenant extends GenericEntity {


    /**
     * ID
     */
    private Long id;
    /**
     * 租户邮箱
     */
    private String email;
    /**
     * 租户邮箱
     */
    private String displayName;
    /**
     * 编码
     */
    private String groupCode;
    /**
     * 登录账号
     */
    private String account;

    /**
     * 密码
     */
    private String password;
    /**
     * 登录状态
     */
    private Boolean loginStatus;
    /**
     * sessionId
     */
    private String sessionId;
    /**
     * 租户类型
     */
    private String origin;
    /**
     * 城市
     */
    private String city;
    /**
     * 区县
     */
    private String town;
    private String cityName;
    private String townName;

    /**
     * 计费号
     */
    private String paymentCode;
    /**
     * 客户联系人姓名
     */
    private String contactName;
    /**
     * 头像
     */
    private String portrait;
    /**
     * 客户联系人邮箱
     */
    private String contactEmail;
    /**
     * 客户联系人电话
     */
    private String contactPhone;
    /**
     * 父租户id
     */
    private Long parentId;
    /**
     * 父租户id
     */
    private String parentResourceId;
    /**
     * 合同开始时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date contractBegin;
    /**
     * 合同到期时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date contractFinish;
    /**
     * 账号到期时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredDate;
    /**
     * 账户折扣费率
     */
    private BigDecimal discount;
    /**
     * 客户经理ID
     */
    private Long managerId;
    /**
     * 编码
     */
    private String code;
    /**
     * 配额
     */
    private JSONArray quotas;
    /**
     * 优惠券数量
     */
    private Integer coupons;
    /**
     * CPU使用率
     */
    private BigDecimal cpuRate;
    /**
     * 内存使用率
     */
    private BigDecimal memoryRate;
    /**
     * 存储使用率
     */
    private BigDecimal diskRate;
    /**
     * 最后一次密码修改时间
     */
    @Column("last_pwd_modify_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastPwdModifyDate;
    /**
     * 最后一次登录时间
     */
    @Column("last_login_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastLoginDate;
    /**
     * 客户经理工号
     */
    private String managerCode;
    /**
     * 客户经理姓名
     */
    private String managerName;
    /**
     * 客户经理邮箱
     */
    private String managerEmail;
    /**
     * 客户经理电话
     */
    private String managerPhone;

    private String managerMobile;
    /**
     * 验证码
     */
    private String captcha;
    /**
     * 运营平台订单数量
     */
    private Long cmpOrders;
    /**
     * esop订单数量
     */
    private Long esopOrders;
    /**
     * 稽核状态
     */
    private String auditStatus;
    /**
     * 最大快照数
     */
    private Integer maxSnapshotCount;
    /**
     * 组织机构类型
     */
    private String type;

    private List<Tenant> children;
}
