package com.bocloud.cms.repository;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.AlarmReceiver;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.sms.entity.User;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 告警分发人DAO实现
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月11日
 */
@Repository
public class AlarmReceiverRepository extends BasicGenericDao<AlarmReceiver, Long> {

    public AlarmReceiverRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * @param sendPolicyId
     * @return
     */
    public List<User> listBySendPolicyId(Long sendPolicyId) {
        String sql = "select b.id,b.name,b.account,b.status,b.mobile,b.email from monitor_alarm_receiver a,users b where a.catalog = '" + RequestContext.Catalog.User + "' and a.receiver_id=b.id and a.send_policy_id=:sendPolicyId";
        Map<String, Object> paramMap = MapTools.simpleMap("sendPolicyId", sendPolicyId);
        List<Map<String, Object>> objects = this.list(sql, paramMap);
        List<User> users = JSONArray.parseArray(JSONArray.toJSONString(objects), User.class);
        List<User> senders = new ArrayList<>();
        for (User user : users) {
            senders.add(user);
        }
        String sql2 = "select b.id,b.name,b.account,b.status,b.email from monitor_alarm_receiver a,tenant b  where a.catalog = '" + RequestContext.Catalog.Tenant + "' and a.receiver_id=b.id and a.send_policy_id=:sendPolicyId";
        List<Map<String, Object>> objects2 = this.list(sql2, paramMap);
        if (objects2.size() > 0) {
            User user = JSONObject.parseObject(JSONObject.toJSONString(objects2.get(0)), User.class);
            user.setId(0L);
            senders.add(user);
        }
        return senders;
    }

    public boolean removeByPolicyId(Long sendPolicyId) {
        String sql = "delete from monitor_alarm_receiver where send_policy_id = :sendPolicyId";
        Map<String, Object> paramMap = MapTools.simpleMap("sendPolicyId", sendPolicyId);
        return this.execute(sql, paramMap) > 0;
    }


    public List<RequestContext> listReceivers(Long sendPolicyId) {
        String sql = " select * from monitor_alarm_receiver a WHERE a.send_policy_id = :sendPolicyId ";
        Map<String, Object> paramMap = MapTools.simpleMap("sendPolicyId", sendPolicyId);
        List<AlarmReceiver> list = this.list(AlarmReceiver.class, sql, paramMap);
        List<RequestContext> receivers = new ArrayList<>();
        for (AlarmReceiver receiver : list) {
            RequestContext context = new RequestContext();
            context.setCatalog(RequestContext.Catalog.valueOf(receiver.getCatalog()));
            context.setTarget(receiver.getReceiverId());
            receivers.add(context);
        }
        return receivers;
    }

}