package com.bocloud.cms.repository;

import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.enums.AlarmStatus;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 告警
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
@Repository("alarmRepository")
public class AlarmRepository extends BasicGenericDao<Alarm, Long> {

    public AlarmRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<Alarm> list(Pager pager, RequestContext context, List<String> types, String vendorType) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("select a.*,b.name vendor_name from monitor_alarm a left join cloud_vendor b on a.vendor_id = b.id and b.is_deleted = 0 where a.is_deleted = 0 and a.tenant_id = :tenantId ");
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        if (RequestContext.Catalog.User == context.getCatalog()) {
            stringBuilder.append("and a.owner_id = :ownerId ");
            paramMap.put("ownerId", context.getTarget());
        }
        if (!StringUtils.isBlank(vendorType)) {
            stringBuilder.append("and b.type=:vendorType ");
            paramMap.put("vendorType", vendorType);
        }
        if (null != types && !types.isEmpty()) {
            stringBuilder.append(" and a.type in (:types) ");
            paramMap.put("types", types);
        }
        String sql = QueryBuilder.buildRaw(stringBuilder.toString(), pager, "a");
        return this.list(Alarm.class, sql, paramMap);
    }

    public int count(List<Param> params, RequestContext context, List<String> types, String vendorType) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("select count(*) from (select a.*,b.name vendor_name,b.type vendor_type from monitor_alarm a left join cloud_vendor b on a.vendor_id = b.id and b.is_deleted = 0 where a.is_deleted = 0 and a.tenant_id =:tenantId ");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", context.getTenant());
        if (RequestContext.Catalog.User == context.getCatalog()) {
            stringBuilder.append("and a.owner_id = :ownerId ");
            paramMap.put("ownerId", context.getTarget());
        }
        if (!StringUtils.isBlank(vendorType)) {
            stringBuilder.append("and b.type=:vendorType ");
            paramMap.put("vendorType", vendorType);
        }
        stringBuilder.append(") a where 1 = 1 ");
        if (null != types && !types.isEmpty()) {
            stringBuilder.append(" and a.type in (:types) ");
            paramMap.put("types", types);
        }
        String sql = QueryBuilder.buildRaw(stringBuilder.toString(), params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean delete(Long id, Long userId) {
        String sql = "update monitor_alarm set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Alarm query(Long id) {
        String sql = "select a.*,b.name vendor_name from monitor_alarm a left join cloud_vendor b on a.vendor_id = b.id and b.is_deleted = 0 where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Alarm> alarms = this.list(Alarm.class, sql, params);
        return ListTool.isEmpty(alarms) ? null : alarms.get(0);
    }

    public boolean confirm(Long id, Long userId, String status, String confirmer) {
        String sql = "update monitor_alarm a set a.mender_id = :menderId,a.status = :status,a.gmt_modify = :gmtModify,a.confirmer = :confirmer,a.gmt_confirm = :gmtConfirm where a.id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("status", status);
        paramMap.put("menderId", userId);
        paramMap.put("gmtModify", new Date());
        paramMap.put("confirmer", confirmer);
        paramMap.put("gmtConfirm", new Date());
        return this.execute(sql, paramMap) > 0;
    }

    public boolean solve(Long id, Long userId, String solver, String dealResult) {
        String sql = "update monitor_alarm a set a.status = :status,a.solver = :solver,a.gmt_solve = :gmtSolve,deal_result = :dealResult,a.mender_id = :menderId,a.gmt_modify = :gmtModify where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("solver", solver);
        params.put("gmtSolve", new Date());
        params.put("dealResult", dealResult);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("status", AlarmStatus.SOLVED.name());
        return this.execute(sql, params) > 0;
    }

    public void solveByServer(Long id, Long userId, String solver, String dealResult) {
        String sql = "update monitor_alarm a set a.status = :status,a.solver = :solver,a.gmt_solve = :gmtSolve,deal_result = :dealResult,a.mender_id = :menderId,a.gmt_modify = :gmtModify where status != 'SOLVED' and a.type = 'HOST' and a.target = :id";
        String sql2 = "update monitor_alarm a set a.status = :status,a.solver = :solver,a.gmt_solve = :gmtSolve,deal_result = :dealResult,a.mender_id = :menderId,a.gmt_modify = :gmtModify where status != 'SOLVED' and a.type = 'VM' and a.target in (select id from cloud_server where host_id = :id)";
        Map<String, Object> params = MapTools.simpleMap("id", id.toString());
        params.put("solver", solver);
        params.put("gmtSolve", new Date());
        params.put("dealResult", dealResult);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("status", AlarmStatus.SOLVED.name());
        this.execute(sql, params);
        this.execute(sql2, params);
    }

    public void solveByCloudServer(Long id, String resourceType, Long userId, String solver, String dealResult) {
        String sql = "update monitor_alarm a set a.status = :status,a.solver = :solver,a.gmt_solve = :gmtSolve,deal_result = :dealResult,a.mender_id = :menderId,a.gmt_modify = :gmtModify where status != 'SOLVED' and a.type = :resourceType and a.target = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id.toString());
        params.put("solver", solver);
        params.put("gmtSolve", new Date());
        params.put("dealResult", dealResult);
        params.put("resourceType", resourceType);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("status", AlarmStatus.SOLVED.name());
        this.execute(sql, params);
    }

    public void solveByVendor(Long id, Long userId, String solver, String dealResult) {
        String sql = "update monitor_alarm a set a.status = :status,a.solver = :solver,a.gmt_solve = :gmtSolve,a.deal_result = :dealResult,a.mender_id = :menderId,a.gmt_modify = :gmtModify where a.status != 'SOLVED' and  a.vendor_id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("solver", solver);
        params.put("gmtSolve", new Date());
        params.put("dealResult", dealResult);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("status", AlarmStatus.SOLVED.name());
        this.execute(sql, params);
    }

    public Alarm queryAlarmMain(String name, String level, Long ruleId, String target, Long tenantId, Date trigger) {
        String sql = "select a.id,a.status,a.remark from monitor_alarm a where a.is_deleted = 0 and a.name = :name and a.tenant_id = :tenantId and a.level = :level and a.rule_id = :ruleId and a.target = :target and a.gmt_trigger =:gmtTrigger and a.status != 'SOLVED' order by id desc ";
        Map<String, Object> paramMap = MapTools.simpleMap("level", level);
        paramMap.put("name", name);
        paramMap.put("tenantId", tenantId);
        paramMap.put("ruleId", ruleId);
        paramMap.put("target", target);
        paramMap.put("gmtTrigger", trigger);
        List<Alarm> alarms = this.list(Alarm.class, sql, paramMap);
        return ListTool.isEmpty(alarms) ? null : alarms.get(0);
    }

    public Alarm queryNativeAlarm(String type, String name, String target, String level, Long vendorId) {
        String sql = "select a.* from monitor_alarm a where a.is_deleted = 0 and a.type = :type and a.name = :name and a.tenant_id = :tenantId and a.level = :level and a.target = :target and a.vendor_id = :vendorId and a.status != 'SOLVED' order by id desc ";
        Map<String, Object> paramMap = MapTools.simpleMap("level", level);
        paramMap.put("name", name);
        paramMap.put("type", type);
        paramMap.put("tenantId", 0L);
        paramMap.put("vendorId", vendorId);
        paramMap.put("target", target);
        List<Alarm> alarms = this.list(Alarm.class, sql, paramMap);
        return ListTool.isEmpty(alarms) ? null : alarms.get(0);
    }

    public List<Alarm> unSolvesForSendNotice() {
        String sql = "SELECT a.*  FROM monitor_alarm a, monitor_alarm_send_policy p, monitor_rule r, monitor_rule_group g  " +
                "WHERE r.id = a.rule_id AND g.id = r.group_id AND g.policy_id = p.id AND a.rule_id IS NOT NULL AND a.STATUS != 'SOLVED' " +
                "AND a.is_deleted = 0 AND ( a.send_notice = 0 OR a.send_notice IS NULL ) " +
                "AND p.wait_time != 0 AND p.wait_time < ( unix_timestamp( now( ) ) - unix_timestamp( gmt_trigger ) )/ 60";
        return this.list(Alarm.class, sql);
    }

    public List<Map<String, Object>> alarmChart(List<Param> params, Long tenantId) {
        String sql = "select DATE_FORMAT(gmt_trigger,'%Y-%m-%d') date, count(*) count from (select a.*,b.name vendor_name,b.type vendor_type from monitor_alarm a left join cloud_vendor b on a.vendor_id = b.id and b.is_deleted = 0 where a.is_deleted = 0 and a.tenant_id = :tenantId) a where 1 = 1 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        sql += " group by DATE_FORMAT(gmt_trigger,'%Y-%m-%d') ";
        return this.list(sql, MapTools.simpleMap("tenantId", tenantId));
    }

    public void solveBySoftware(Long id, Long userId, String solver, String monitorResourceType, String dealResult) {
        String sql = "update monitor_alarm a set a.status = :status,a.solver = :solver,a.gmt_solve = :gmtSolve,deal_result = :dealResult,a.mender_id = :menderId,a.gmt_modify = :gmtModify where a.status != 'SOLVED' and a.type like :type and a.target = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id.toString());
        params.put("solver", solver);
        params.put("gmtSolve", new Date());
        params.put("dealResult", dealResult);
        params.put("type", monitorResourceType + "%");
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("status", AlarmStatus.SOLVED.name());
        this.execute(sql, params);
    }

    public List<Map<String, Object>> vendorAlarmStats(RequestContext context,List<Param> params1, List<Param> params2) {
        String sql = "select a.vendor_id,b.name,sum(case when a.level ='URGENT' THEN 1 ELSE 0 END ) as urgent ,sum(case when a.level = 'PRIMARY' THEN 1 ELSE 0 END ) as major,sum(case when a.level = 'SECONDARY' THEN 1 ELSE 0 END ) as secondary,sum(case when a.level = 'REMIND' THEN 1 ELSE 0 END ) as remind from monitor_alarm a,cloud_vendor b where b.is_deleted = 0 and a.is_deleted = 0 and a.vendor_id= b.id and a.tenant_id = :tenantId";
        Map<String, Object> paramMap =MapTools.simpleMap("tenantId", context.getTenant());
        sql = QueryBuilder.buildRaw(sql, params1, null, "b");
        sql = QueryBuilder.buildRaw(sql, params2, null, "a");
        sql += " group by a.vendor_id,b.name";
        return this.list(sql,paramMap);
    }

    /**
     * 删除云平台下机器的同时 同步删除该机器下的告警
     * @param id
     * @param context
     * @return
     */
    public List<Alarm> listByTarget(Long id, RequestContext context) {
        String sql = "select * from  monitor_alarm  where target = :target";
        Map<String, Object> paramMap =MapTools.simpleMap("target",id);
        return this.list(Alarm.class, sql, paramMap);
    }
}