package com.bocloud.cms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cms.entity.AlarmSendPolicy;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 告警分发策略DAO实现
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月11日
 */
@Repository("alarmSendPolicyRepository")
public class AlarmSendPolicyRepository extends BasicGenericDao<AlarmSendPolicy, Long> {

    public AlarmSendPolicyRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<AlarmSendPolicy> list(Pager pager, RequestContext context) {
        String sql = "select a.* from monitor_alarm_send_policy a where tenant_id = :tenantId ";
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        sql = QueryBuilder.buildRaw(sql, pager, "a");
        return this.list(AlarmSendPolicy.class, sql, paramMap);
    }

    public int count(List<Param> params, RequestContext context) {
        String sql = "select count(1) from monitor_alarm_send_policy a where tenant_id = :tenantId  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean remove(Long id) {
        String sql = "delete from monitor_alarm_send_policy  where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        return this.execute(sql, paramMap) > 0;
    }

    public AlarmSendPolicy query(Long id) {
        String sql = "select a.* from monitor_alarm_send_policy a where a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<AlarmSendPolicy> policies = this.list(AlarmSendPolicy.class, sql, paramMap);
        return ListTool.isEmpty(policies) ? null : policies.get(0);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, RequestContext context) {

        String sql = "select a.id,a.name from monitor_alarm_send_policy a where a.tenant_id = :tenantId  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<AlarmSendPolicy> policies = this.list(AlarmSendPolicy.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (AlarmSendPolicy alarmSendPolicy : policies) {
            beans.add(new SimpleBean(alarmSendPolicy.getId(), alarmSendPolicy.getName()));
        }
        return beans;
    }

    public boolean existsSameName(String name, RequestContext context) {
        String sql = "select count(1) from monitor_alarm_send_policy where name = :name and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        return this.countQuery(sql, paramMap).intValue() > 0;
    }

    /**
     * 除当前id数据外，其他数据的原生告警字段都重置为false
     *
     * @param id
     */
    public void updateNativeAlarm(Long id) {
        super.execute("update monitor_alarm_send_policy set native_alarm = 0 where id != :id", MapTools.simpleMap("id", id));
    }

}
