package com.bocloud.cms.repository;

import com.bocloud.cms.entity.AlarmTemplate;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 告警模板
 * @author fanzx
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
@Repository("alarmTemplateRepository")
public class AlarmTemplateRepository extends BasicGenericDao<AlarmTemplate, Long> {

    public AlarmTemplateRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params, RequestContext context) {
        String sql = "select count(1) from monitor_alarm_template where is_deleted = 0 and tenant_id = :tenantId  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", context.getTenant());
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<AlarmTemplate> list(Pager pager, RequestContext context) {
        String sql = "select t.*,u.name as create_name from monitor_alarm_template t left join users u on u.id = t.creator_id where t.is_deleted = 0 and t.tenant_id = :tenantId ";
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        sql = QueryBuilder.buildRaw(sql, pager, "");
        return this.list(AlarmTemplate.class, sql, paramMap);
    }

    public boolean delete(Long id) {
        String sql = "update monitor_alarm_template set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }

    public AlarmTemplate getByType(String resourceType,Long tenantId){
        String sql = "select * from monitor_alarm_template where is_deleted = 0 and resource_type = :resourceType and tenant_id = :tenantId ORDER BY gmt_create desc ";
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        paramMap.put("tenantId", tenantId);
        List<AlarmTemplate> alarmTemplates = this.list(AlarmTemplate.class, sql, paramMap);
        return ListTool.isEmpty(alarmTemplates) ? null : alarmTemplates.get(0);
    }

}
