package com.bocloud.cms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 监控分析监控分析Dao层实现类的处理
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
@Repository("analysisRuleRepository")
public class AnalysisRuleRepository extends BasicGenericDao<MonitorAnalysisRule, Long> {

    public AnalysisRuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from monitor_analysis_rule a where is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, null).intValue();
    }

    public List<MonitorAnalysisRule> list(Pager pager) {
        String sql = "select * from monitor_analysis_rule where is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, pager, "");
        return this.list(MonitorAnalysisRule.class, sql);
    }

    public MonitorAnalysisRule query(Long id) {
        String sql = "select * from monitor_analysis_rule where is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<MonitorAnalysisRule> rules = this.list(MonitorAnalysisRule.class, sql, paramMap);
        return ListTool.isEmpty(rules) ? null : rules.get(0);
    }

    public boolean nameExist(Long id, String name) {
        String sql = "select * from monitor_analysis_rule where is_deleted = 0 and name = :name ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        if (null != id) {
            sql += "and id <> :id ";
            paramMap.put("id", id);
        }
        List<MonitorAnalysisRule> rules = this.list(MonitorAnalysisRule.class, sql, paramMap);
        return !rules.isEmpty();
    }

}
