package com.bocloud.cms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cms.entity.AnalysisSnapshot;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author zhangyf
 * @since 2019-08-15
 */
@Repository("analysisSnapshotRepository")
public class AnalysisSnapshotRepository extends BasicGenericDao<AnalysisSnapshot, Long> {

    public AnalysisSnapshotRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from monitor_analysis_snapshot a where is_deleted = 0   ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<AnalysisSnapshot> list(Integer page, Integer rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select s.*,r.name as rule_name from monitor_analysis_snapshot s left join monitor_analysis_rule r on s.rule_id = r.id where s.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "s");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(AnalysisSnapshot.class, sql, paramMap);
    }

    public boolean delete(Long id, Long userId) {
        String sql = "update monitor_analysis_snapshot set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }

    public AnalysisSnapshot query(Long id) {
        String sql = "select * from monitor_analysis_snapshot where is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<AnalysisSnapshot> snapshots = this.list(AnalysisSnapshot.class, sql, paramMap);
        return ListTool.isEmpty(snapshots) ? null : snapshots.get(0);
    }

}
