package com.bocloud.cms.repository;

import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cms.entity.IpRelation;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * IpRelationRepository
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Repository("ipRelationRepository")
public class IpRelationRepository extends BasicGenericDao<IpRelation, Long> {

    public IpRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public IpRelation query(Long targetId, Long tenantId) {
        String sql = "select * from monitor_ip_relation where tenant_id = :tenantId and target_id = :targetId";
        Map<String, Object> paramMap = MapTools.simpleMap("targetId", targetId);
        paramMap.put("tenantId", tenantId);
        List<IpRelation> ipRelations = this.list(IpRelation.class, sql, paramMap);
        return ListTool.isEmpty(ipRelations) ? null : ipRelations.get(0);
    }

    public List<IpRelation> list(List<Long> targetIds, Long tenantId) {
        String sql = "select * from monitor_ip_relation where tenant_id = :tenantId and target_id in (:targetIds)";
        Map<String, Object> paramMap = MapTools.simpleMap("targetIds", targetIds);
        paramMap.put("tenantId", tenantId);
        return this.list(IpRelation.class, sql, paramMap);
    }

    public void deleteIpRelationsByTarget(Long targetId) {
        String sql = "delete from monitor_ip_relation where target_id = :targetId";
        this.execute(sql, MapTools.simpleMap("targetId", targetId));
    }

}
