package com.bocloud.cms.repository;

import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cms.entity.MonitorHostIpmi;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 主机监控
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Repository("monitorHostDaoRepository")
public class MonitorHostIpmiRepository extends BasicGenericDao<MonitorHostIpmi, Long> {

    public MonitorHostIpmiRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public MonitorHostIpmi query(Long hostId) {
        String sql = "select * from monitor_host_ipmi where is_deleted = 0 and host_id = :hostId ";
        Map<String, Object> paramMap = MapTools.simpleMap("hostId", hostId);
        List<MonitorHostIpmi> ipRelations = this.list(MonitorHostIpmi.class, sql, paramMap);
        return ListTool.isEmpty(ipRelations) ? null : ipRelations.get(0);
    }


    public boolean delete(Long id, Long userId) {
        String sql = "update monitor_host_ipmi set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("gmtModify", new Date());
        paramMap.put("menderId", userId);
        return this.execute(sql, paramMap) > 0;
    }

    public List<MonitorHostIpmi> list() {
        String sql = "select * from monitor_host_ipmi where is_deleted = 0 ";
        return this.list(MonitorHostIpmi.class, sql);
    }
}
