package com.bocloud.cms.repository;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 开启关闭告警状态
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Repository("resourceAlarmStatusRepository")
public class ResourceAlarmStatusRepository extends BasicGenericDao<ResourceAlarmStatus, Long> {

    public ResourceAlarmStatusRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ResourceAlarmStatus> list(String resourceType, List<String> targets, RequestContext context) {
        if (targets == null || targets.isEmpty()) {
            return new ArrayList<>();
        }
        StringBuilder builder = new StringBuilder();
        builder.append("select * from monitor_resource_alarm_status where resource_type = :resourceType and tenant_id = :tenantId ");
        builder.append("  and resource_id in (:resourceId) ");
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        paramMap.put("tenantId",context.getTenant());
        if (targets.size()>0){
            paramMap.put("resourceId", targets);
        }
        return this.list(ResourceAlarmStatus.class, builder.toString(), paramMap);
    }

    /**
     * 根据id和属性删除对象
     * @param resourceId
     * @param resourceType
     */
    public boolean delete(String resourceId, String resourceType,RequestContext context) {
        String sql = "delete from monitor_resource_alarm_status where resource_type = :resourceType  and resource_id = :resourceId and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        if (resourceId!=null){
            paramMap.put("resourceId", resourceId);
        }
        paramMap.put("tenantId",context.getTenant());
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 是否开启告警(默认开启)
     *
     * @param resourceType
     * @param target
     * @param tenantId
     * @return
     */
    public boolean alarmEnable(String resourceType, String target, Long tenantId) {
        StringBuilder builder = new StringBuilder();
        builder.append("select * from monitor_resource_alarm_status where resource_type = :resourceType and resource_id = :resourceId and tenant_id = :tenantId ");
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        paramMap.put("resourceId", target);
        paramMap.put("tenantId", tenantId);
        List<ResourceAlarmStatus> list = this.list(ResourceAlarmStatus.class, builder.toString(), paramMap);
        return !Optional.ofNullable(list).orElse(new ArrayList<>()).stream().findAny().isPresent();
    }

    public ResourceAlarmStatus getByTargetId( String resourceId,Long tenantId ){
        String sql = "select * from monitor_resource_alarm_status where resource_id = :resourceId and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("resourceId", resourceId);
        paramMap.put("tenantId", tenantId);
        List<ResourceAlarmStatus> list = this.list(ResourceAlarmStatus.class, sql, paramMap);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }
}
