package com.bocloud.cms.repository;

import com.bocloud.cms.entity.ResourceRelation;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author fanzx
 * @Date 2021/09/18
 */
@Repository( "resourceRelationRepository" )
public class ResourceRelationRepository extends BasicGenericDao<ResourceRelation, Long> {

    public ResourceRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 根据ruleGroupId查询对象集合
     * @param id
     * @return
     */
    public List<ResourceRelation> listByRuleGroupId(Long id) {
        String sql = "select * from monitor_resource_relation where is_deleted = 0 and group_id = :groupId  ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        List<ResourceRelation> resourceRelations = this.list(ResourceRelation.class, sql, paramMap);
        if (resourceRelations != null) {
            return resourceRelations;
        }
        return null;
    }

    /**
     * 根据ruleGroupId"删除"表信息
     *     将is_delete字段修改为1
     * @param id
     * @return
     */
    public boolean delete(Long id) {
        String sql = "update monitor_resource_relation set is_deleted = 1, gmt_modify = :gmtModify where group_id = :groupId";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }


    /**
     * 根据ruleGroupID 去删除monitor_resource_relation表信息
     * @param id
     */
    public Boolean deleteByGroupId(Long id) {
        String sql = "delete from monitor_resource_relation  where group_id = :groupId  ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 根据ruleGroupId,和resourceId查询出单条ResourceRelation对象
     * @param ruleGroupId
     * @param resourceId
     * @return
     */
    public ResourceRelation find(Long ruleGroupId, String resourceId) {
        String sql = "select * from monitor_resource_relation where is_deleted = 0 and  target = :target and group_id = :groupId ";
        Map<String, Object> paramMap = MapTools.simpleMap("target", resourceId);
        if (null != ruleGroupId) {
            paramMap.put("groupId", ruleGroupId);
        }
        List<ResourceRelation> ruleGroups = this.list(ResourceRelation.class, sql, paramMap);
        return ListTool.isEmpty(ruleGroups) ? null : ruleGroups.get(0);
    }

    /**
     * 根据resourceId,去查询到所关联的ruleGroupId集合
     * @param resourceId
     * @return
     */
    public List<ResourceRelation> listByTarget(String resourceId) {
        String sql = "select * from monitor_resource_relation where is_deleted = 0 and target = :target";
        Map<String, Object> paramMap = MapTools.simpleMap("target", resourceId);
        return this.list(ResourceRelation.class, sql, paramMap);
    }
}
