package com.bocloud.cms.repository;

import com.bocloud.cms.entity.Resource;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 监控资源
 *
 * @author zhangyf
 * @since 2019-07-15
 */
@Repository("resourceRepository")
public class ResourceRepository extends BasicGenericDao<Resource, Long> {

    public ResourceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params, RequestContext context, List<String> types) {
        String sql = "select count(1) from monitor_resource where is_deleted = 0 and tenant_id = :tenantId ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", context.getTenant());
        paramMap.put("types", types);
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        if (null != types && !types.isEmpty()) {
            sql += " and type in (:types) ";
            paramMap.put("types", types);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<Resource> list(Pager pager, RequestContext context, List<String> types) {
        String sql = "select * from monitor_resource a where tenant_id = :tenantId ";
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        if (null != types && !types.isEmpty()) {
            sql += " and type in (:types) ";
            paramMap.put("types", types);
        }
        sql = QueryBuilder.buildRaw(sql, pager, "a");
        return this.list(Resource.class, sql, paramMap);
    }

    public List<Resource> list(Pager pager) {
        String sql = "select * from monitor_resource a ";
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        sql = QueryBuilder.buildRaw(sql, pager, "a");
        return this.list(Resource.class, sql, paramMap);
    }


    public boolean delete(Long id, Long contextId) {
        String sql = "update monitor_resource set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("gmtModify", new Date());
        paramMap.put("menderId", contextId);
        return this.execute(sql, paramMap) > 0;
    }

    public Resource query(Long id) {
        String sql = "select * from monitor_resource where is_deleted = 0 and id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<Resource> resources = this.list(Resource.class, sql, paramMap);
        return ListTool.isEmpty(resources) ? null : resources.get(0);
    }

    public Resource getByName(Long id, String name, RequestContext context) {
        String sql = "select * from monitor_resource where is_deleted = 0 and name = :name and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        if (null != id) {
            sql += "and id <> :id ";
            paramMap.put("id", id);
        }
        List<Resource> resources = this.list(Resource.class, sql, paramMap);
        return ListTool.isEmpty(resources) ? null : resources.get(0);
    }

}
