package com.bocloud.cms.repository;

import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author fanzx
 * @Date 2021/09/18
 */
@Repository("ruleGroupRepository")
public class RuleGroupRepository extends BasicGenericDao<RuleGroup, Long> {

    public RuleGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 根据policyId去查询总数
     * @param id
     * @param context
     * @return
     */
    public Long countByPolicy(Long id, RequestContext context) {
        String sql = "select count(*) from monitor_rule_group a where a.is_deleted = 0 and a.policy_id = :policyId and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("policyId", id);
        paramMap.put("tenantId", context.getTenant());
        return this.countQuery(sql, paramMap);
    }

    /**
     * 根据name查询RuleGroup对象
     *
     * @param id
     * @param name
     * @param context
     * @return
     */
    public RuleGroup getByName(Long id, String name, RequestContext context) {
        String sql = "select * from monitor_rule_group where is_deleted = 0 and name = :name and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", context.getTenant());
        if (null != id) {
            sql += "and id = :id ";
            paramMap.put("id", id);
        }
        List<RuleGroup> ruleGroups = this.list(RuleGroup.class, sql, paramMap);
        return ListTool.isEmpty(ruleGroups) ? null : ruleGroups.get(0);
    }

    /**
     * 查询告警策略列表
     *
     * @param params
     * @param context
     * @return
     */
    public int count(List<Param> params, RequestContext context) {
        String sql = "select count(1) from monitor_rule_group where is_deleted = 0 and tenant_id = :tenantId  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", context.getTenant());
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }


    /**
     * 查询告警策略列表
     *
     * @param pager
     * @param context
     * @return
     */
    public List<RuleGroup> list(Pager pager, RequestContext context) {
        String sql = "select g.*,p.name as policy_name from monitor_rule_group g left join monitor_alarm_send_policy p on g.policy_id = p.id where g.is_deleted = 0 and g.tenant_id = :tenantId ";
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and g.creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        sql = QueryBuilder.buildRaw(sql, pager, "g");
        return this.list(RuleGroup.class, sql, paramMap);
    }

    /**
     * 根据ruleGroupId"删除"表信息
     *     将is_delete字段修改为1
     * @param id
     * @return
     */
    public boolean delete(Long id) {
        String sql = "update monitor_rule_group set is_deleted = 1, gmt_modify = :gmtModify where id = :groupId";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }


    /**
     * 根据id和传入的状态 开启/关闭告警状态
     * @param groupId
     * @param enable
     * @param context
     */
    public Boolean update(Long groupId, Boolean enable, RequestContext context) {
        String sql = "update monitor_rule_group set enable = :enable where  id = :id  ";
        Map<String, Object> paramMap = MapTools.simpleMap("enable", enable);
        if ( groupId!=null ){
            paramMap.put("id",groupId );
        }
        return this.execute(sql, paramMap) > 0;
    }

    public boolean existsSameName(String name, RequestContext context) {
        String sql = "select count(1) from monitor_rule_group where name = :name and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        return this.countQuery(sql, paramMap).intValue() > 0;
    }
}
