package com.bocloud.cms.repository;

import com.bocloud.cms.entity.Rule;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author fanzx
 * @Date 2021/09/18
 */
@Repository("ruleRepository")
public class RuleRepository extends BasicGenericDao<Rule, Long> {

    public RuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 根据groupId查询Rule集合对象
     * @param groupId
     * @return
     */
    public List<Rule> listByRuleGroupId(Long groupId) {
        String sql = "select * from monitor_rule where is_deleted = 0 and group_id = :groupId  ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", groupId);
        List<Rule> rules = this.list(Rule.class, sql, paramMap);
        return ListTool.isEmpty(rules) ? null : rules;
    }

    /**
     * 根据ruleGroupId"删除"表信息
     *     将is_delete字段修改为1
     * @param id
     * @return
     */
    public boolean delete(Long id) {
        String sql = "update monitor_rule set is_deleted = 1, gmt_modify = :gmtModify  where group_id = :groupId";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }
}
