package com.bocloud.cms.intf;

import com.bocloud.common.model.*;
import com.bocloud.cms.entity.Alarm;

import java.util.Date;
import java.util.List;

/**
 * 告警服务service接口
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月13日
 */
public interface AlarmService {

    /**
     * 分页查询
     * @param pager
     * @param context
     * @return
     */
    GridBean list(Pager pager, RequestContext context);

    /**
     * 添加告警信息
     *
     * @param alarm
     * @return
     */
    void create(Alarm alarm);

    /**
     * 批量添加告警信息
     *
     * @param alarms
     * @return
     */
    void create(List<Alarm> alarms);

    /**
     * 获取详细信息
     *
     * @param id 告警ID
     * @return 告警详细信息
     */
    BocloudResult detail(Long id);

    /**
     * 查询未解决并且需要发送通知的告警
     *
     * @return
     */
    List<Alarm> unSolvesForSendNotice();

    /**
     * 告警确认
     *
     * @param ids    告警ID
     * @param context 操作者
     */
    BocloudResult confirm(List<Long> ids, RequestContext context);

    /**
     * 告警解决
     *
     * @param id
     * @param context
     * @param dealResult
     * @return
     */
    void solve(Long id, RequestContext context, String dealResult);

    /**
     * 告警解决(批量)
     *
     * @param ids
     * @param context
     * @param dealResult
     */
    void solve(List<Long> ids, RequestContext context, String dealResult);

    /**
     * 按宿主机解决告警
     *
     * @param id
     * @param userId
     * @param dealResult
     */
    void solveByServer(Long id, Long userId, String dealResult);

    /**
     * 按云主机解决告警
     *
     * @param id
     * @param userId
     * @param resourceType
     * @param dealResult
     */
    void solveByCloudServer(Long id, String resourceType, Long userId, String dealResult);

    /**
     * 按平台解决告警
     *
     * @param id
     * @param userId
     * @param dealResult
     */
    void solveByVendor(Long id, Long userId, String dealResult);

    /**
     * 按宿主机解决告警
     *
     * @param id
     * @param monitorResourceType
     * @param userId
     * @param dealResult
     */
    void solveBySoftware(Long id, String monitorResourceType, Long userId, String dealResult);

    /**
     * 查询资源告警主记录
     *
     * @param name
     * @param level
     * @param ruleId
     * @param target
     * @param tenantId
     * @param trigger  触发时间
     * @return
     */
    Alarm queryAlarmMain(String name, String level, Long ruleId, String target, Long tenantId, Date trigger);

    /**
     * 统计接口
     * @param params
     * @param context
     * @return
     */
    BocloudResult stats(List<Param> params, RequestContext context);

    /**
     * 告警统计饼状图
     * @param context
     * @param start
     * @param end
     * @param vendorId
     * @param target
     * @param s
     * @param type
     * @param vendorType
     * @return
     */
    BocloudResult alarmLevel(RequestContext context, Long start, Long end, Long vendorId, Object target, String s, String type, String vendorType);

    /**
     * 告警统计柱状堆积图
     * @param context
     * @param type
     * @param vendorId
     * @param target
     * @param start
     * @param end
     * @param vendorType
     * @return
     */
    BocloudResult alarmStats(RequestContext context, String type, Long vendorId, Object target, Long start, Long end, String vendorType);

    /**
     * 平台告警柱状图
     * @param context
     * @param start
     * @param end
     * @param vendorType
     * @return
     */
    BocloudResult vendorAlarmStats(RequestContext context, Long start, Long end, String vendorType);

}
