package com.bocloud.cms.intf;

import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.common.model.RequestContext;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;

/**
 * 告警资源对象接口
 *
 * @author zhangyf
 * @since 2019-12-23
 */
public interface MonitoringAble<T> {

    /**
     * 获取默认的唯一标识，如果实现类T中不是使用id作为唯一标识，则需要复写此方法
     *
     * @param target       目标资源对象
     * @param resourceType 资源类型
     * @return
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     */
    default String getTargetObjectId(T target, MonitorResourceType resourceType) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        return String.valueOf(target.getClass().getMethod("getId").invoke(target));
    }

    /**
     * 获取告警目标对象
     *
     * @param targetIds     目标资源id
     * @param resourceType 资源类型
     * @return
     */
    List<T> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType);

    /**
     * 获取表达式参数<br />
     * 用于生成告警规则时匹配资源
     *
     * @param targetObject 告警目标对象
     * @param metric
     * @return
     */
    Map<String, String> getPrometheusExprParam(T targetObject, Metric metric);

    /**
     * 从prometheus告警信息中解析资源对象
     *
     * @param alert
     * @param ruleGroup
     * @param rule
     * @param metric
     * @return
     */
    T parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric);

    /**
     * 资源被删除
     *
     * @param object
     * @throws Exception
     */
    default void destroy(T object) throws Exception {
        destroy(object, null);
    }


    /**
     * 资源被删除
     *
     * @param object
     * @param catalog //资源下级类型
     * @throws Exception
     */
    void destroy(T object, String catalog) throws Exception;

    /**
     * 接收到告警后设置对应资源个性化内容
     *
     * @param alarm
     * @param object parseObjectFromAlert 方法返回的对象
     */
    void wrapAlarm(Alarm alarm, T object);

    /**
     * 资源告警接收状态
     *
     * @param ruleGroup
     * @param object
     * @return
     */
    boolean alarmEnable(RuleGroup ruleGroup, T object);

}
