package com.bocloud.cms.service;

import com.bocloud.cms.entity.*;
import com.bocloud.cms.intf.AlarmTemplateService;
import com.bocloud.cms.repository.AlarmTemplateRepository;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 告警模板
 * @author fanzx
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
@Service
public class AlarmTemplateServiceImpl implements AlarmTemplateService {

    private static final Logger logger = LoggerFactory.getLogger(AlarmSendPolicyServiceImpl.class);

    @Autowired
    private AlarmTemplateRepository alarmTemplateRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = this.alarmTemplateRepository.count(pager.getParams(), context);
            List<AlarmTemplate> alarmTemplates = this.alarmTemplateRepository.list(pager, context);
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, alarmTemplates);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("query template list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(AlarmTemplate alarmTemplate, RequestContext context) {

        try {
            Assert.notNull(alarmTemplate, "未获取到任何数据!");
            alarmTemplate.setCreatorId(context.getTarget());
            alarmTemplate.setTenantId(context.getTenant());
            alarmTemplate.setGmtCreate(new Date());
            alarmTemplate.setIsDeleted(false);
            this.alarmTemplateRepository.save(alarmTemplate);
            return new BocloudResult(true, "新增成功");
        }catch (Exception e){
            logger.error("create templates fail:", e);
            return new BocloudResult(false, "新增失败");
        }
    }

    @Override
    public BocloudResult modify(AlarmTemplate alarmTemplate, RequestContext context) {
        try {
            AlarmTemplate template = this.alarmTemplateRepository.get(AlarmTemplate.class, alarmTemplate.getId());
            Assert.notNull(template, "未查询到任何数据!");
            alarmTemplate.setCreatorId(template.getCreatorId());
            alarmTemplate.setMenderId(context.getTarget());
            alarmTemplate.setTenantId(context.getTenant());
            alarmTemplate.setIsDeleted(false);
            this.alarmTemplateRepository.update(alarmTemplate);
            return new BocloudResult(true, "修改成功");
        }catch (Exception e){
            logger.error("modify templates fail:", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = RuleGroup.class.getSimpleName() + "_" + id;
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            org.springframework.util.Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            AlarmTemplate alarmTemplate = this.alarmTemplateRepository.get(AlarmTemplate.class, id);
            Assert.notNull(alarmTemplate, " 数据不存在");
            alarmTemplate.setIsDeleted(true);
            this.alarmTemplateRepository.update(alarmTemplate);
            return new BocloudResult(true, "告警模板删除成功!");

        } catch (Exception e) {
            logger.error("remove template error:", e);
            return new BocloudResult(false, "告警模板删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            AlarmTemplate alarmTemplate = this.alarmTemplateRepository.get(AlarmTemplate.class, id);
            if (alarmTemplate.getIsDeleted()){
                return new BocloudResult(false, new AlarmTemplate(), "查询失败,数据不存在");
            }
            Assert.notNull(alarmTemplate, "未查询到任何数据!");
            return new BocloudResult(true, alarmTemplate, "查询成功");
        }catch (Exception e){
            logger.error("Query alarm template detail error:", e);
            return new BocloudResult(false, new AlarmTemplate(), "查询失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult batchRemove(List<Long> ids, Long target) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            Assert.notEmpty(ids, " 至少选择一个告警模板删除");
            for (Long id : ids) {
                this.alarmTemplateRepository.delete(id);
                rightCount++;
            }
        } catch (Exception e) {
            logger.error("alarm template batch delete error!", e);
            return new BocloudResult(false, "批量删除告警模板失败:" + e.getMessage());
        }
        return new BocloudResult(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");

    }
}
