package com.bocloud.cms.service;

import com.bocloud.cms.entity.AnalysisSnapshot;
import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.cms.intf.AnalysisSnapshotService;
import com.bocloud.cms.repository.AnalysisRuleRepository;
import com.bocloud.cms.repository.AnalysisSnapshotRepository;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author zhangyf
 * @since 2019-08-15
 */
@Service
public class AnalysisSnapshotServiceImpl implements AnalysisSnapshotService {

    private static final Logger logger = LoggerFactory.getLogger(AnalysisSnapshotServiceImpl.class);
    @Autowired
    private AnalysisSnapshotRepository analysisSnapshotRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AnalysisRuleRepository analysisRuleRepository;
    @Autowired
    private ImsCloudVendorService imsCloudVendorService;

    @Override
    public BocloudResult list(Integer page, Integer rows, List<Param> params, Map<String, String> sorter,
                              Boolean simple, RequestContext context) {
        GridBean gridBean;
        try {
            if (null == params) {
                params = Lists.newArrayList();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            int total = this.analysisSnapshotRepository.count(params);
            List<AnalysisSnapshot> list = this.analysisSnapshotRepository.list(page, rows, params, sorter);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query snapshot list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, Long userId) {
        String path = AnalysisSnapshot.class.getSimpleName() + "_" + id;
        try (BocloudACLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), " 请求超时");
            AnalysisSnapshot object = this.analysisSnapshotRepository.query(id);
            Assert.notNull(object, " 数据不存在");
            analysisSnapshotRepository.delete(id, userId);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove snapshot error:", e);
            return new BocloudResult(false, "删除失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            AnalysisSnapshot snapshot = this.analysisSnapshotRepository.query(id);
            Assert.notNull(snapshot, " 数据不存在");
            MonitorAnalysisRule analysisRule = analysisRuleRepository.query(snapshot.getRuleId());
            CloudVendor vendor = imsCloudVendorService.detail(snapshot.getVendorId()).getData();
            snapshot.setType(analysisRule.getType());
            snapshot.setRuleName(analysisRule.getName());
            snapshot.setVendorName(vendor.getName());
            return new BocloudResult(true, snapshot, "详情展示成功");
        } catch (Exception e) {
            logger.error("snapshot detail error!", e);
            return new BocloudResult(false, "详情展示失败" + e.getMessage());
        }
    }

}
