package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.CloudTowerHostService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

@Service
public class CloudTowerHostServiceImpl implements CloudTowerHostService, InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(CloudTowerHostServiceImpl.class);

    @Autowired
    CloudTowerHostService cloudTowerHostService;
    /**
     * 查询宿主机列表时使用
     */
    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_CLOUDTOWER_HOST, this);
    }
    @Override
    public List<ServerModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        BocloudResult result = imsServerService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(ServerModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("name", targetObject.getName());
        params.put("vendorId",targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public ServerModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String instanceId = String.valueOf(alert.getLabels().get("instanceId"));
        // 通过主机名称去查找对应的主机信息
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("name", alert.getLabels().get("name")), Sign.EQ));
        BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<ServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
        Assert.isTrue(list != null && !list.isEmpty(), "cloudTowerHost not found,instanceId:" + instanceId);
        return list.get(0);
    }

    @Override
    public void destroy(ServerModel object, String catalog) throws Exception {

    }

    @Override
    public void wrapAlarm(Alarm alarm, ServerModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getManageIp() == null ? object.getName() : object.getName() + "(" + object.getManageIp() + ")";
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, ServerModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

}
