package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.CloudTowerVmService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.ImsCloudServerService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

@Service
public class CloudTowerVmServiceImpl implements CloudTowerVmService, InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(CloudTowerVmServiceImpl.class);

    @Autowired
    CloudTowerVmService cloudTowerVmService;
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmService alarmService;
    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_CLOUDTOWER_VM, this);
    }
    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        BocloudResult result = imsCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("instanceId", targetObject.getInstanceId());
        params.put("vendorId",targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String instanceId = String.valueOf(alert.getLabels().get("instanceId"));
        // 通过vmName去查找对应的云主机信息
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
        BocloudResult result = imsCloudServerService.list(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        Assert.isTrue(list != null && !list.isEmpty(), "cloudTowerVm not found,instanceId:" + instanceId);
        return list.get(0);
    }

    @Override
    public void destroy(CloudServer cloudServer, String catalog) throws Exception {
        logger.info("destroy cloudTower vm : " + cloudServer.getId());
        alarmService.solveByCloudServer(cloudServer.getId(),catalog,1L,"系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, CloudServer object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getManagerIp() == null ? object.getName() : object.getName() + "(" + object.getManagerIp() + ")";
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

}
