package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.FusionCloudService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ImsCloudServerService;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.entity.Server;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

/**
 * FileSystem
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service("fusionCloudService")
public class FusionCloudServiceImpl implements FusionCloudService, InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(FusionCloudServiceImpl.class);
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public BocloudResult listOfHost(Pager pager, RequestContext context) {
        BocloudResult result = imsServerService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<ServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(ServerModel.class);
        List<String> targets = new ArrayList<>();
        List<String> nameList = new ArrayList<>();
        for (Server server : list) {
            targets.add(String.valueOf(server.getId()));
            nameList.add(server.getName());
        }

        String names = StringUtils.join(nameList.toArray(), "|");
        List<PrometheusResult> cpuList = prometheusService.query("round(bocloud_fusioncloud_host_cpuUsage{id=~'" + names + "'},0.01)");
        List<PrometheusResult> memoryList = prometheusService.query("round(bocloud_fusioncloud_host_memoryUsage{id=~'" + names + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.name(), target, context);
        for (ServerModel serverModel : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(serverModel.getId().toString()))
                    .findAny().isPresent();
            serverModel.setAlarmEnable(alarmEnable);
        }

        //CPU利用率赋值
        list.forEach(server -> cpuList.stream().filter(cpuUsage -> cpuUsage.getMetric().get("name").equals(server.getName()))
                .forEach(cpuUsage -> server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]))));
        //内存利用率赋值
        list.forEach(server -> memoryList.stream().filter(memoryUsage -> memoryUsage.getMetric().get("name").equals(server.getName()))
                .forEach(memoryUsage -> server.setMemUsage(String.valueOf(memoryUsage.getValue()[1]))));
        data.put(Common.ROWS, list);
        return result;
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List listTargetObjects(List targetIds, MonitorResourceType resourceType) {
        // 适配达梦数据库
        List<Integer> targetList = new ArrayList<>();
        for (Object targetId : targetIds) {
            targetList.add(Integer.valueOf(targetId.toString()));
        }
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        // 去掉.name()操作 保持equals比较时两边类型一致 防止always false问题
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = imsCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        }
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_VM.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = imsCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        }
        return null;
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Object targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        //主机
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.name().equals(metric.getResourceType())) {
            Server server = (Server) targetObject;
            params.put("id", server.getName());//id、name值一样
            params.put("name", server.getName());
            params.put("vendorId", String.valueOf(server.getVendorId()));
            //云主机
        } else if (MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name().equals(metric.getResourceType())) {
            CloudServer cloudServer = (CloudServer) targetObject;
            params.put("id", cloudServer.getInstanceId());
            params.put("vendorId", String.valueOf(cloudServer.getVendorId()));
        }
        return params;
    }

    @Override
    public Object parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.name().equals(metric.getResourceType())) {
            String name = String.valueOf(alert.getLabels().get("name"));
            List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("name", name), Sign.EQ));
            BocloudResult result = imsServerService.list(new Pager(1, 1, params, null));
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
            Assert.isTrue(list != null && !list.isEmpty(), "server not found,name:" + name);
            return list.get(0);
        }
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name().equals(metric.getResourceType())) {
            String instanceId = String.valueOf(alert.getLabels().get("id"));
            List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
            BocloudResult result = imsCloudServerService.list(new Pager(1, 1, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
            Assert.isTrue(list != null && !list.isEmpty(), "vm not found,instanceId:" + instanceId);
            return list.get(0);
        }
        return null;
    }

    @Override
    public void destroy(Object object, String catalog) throws Exception {

    }

    @Override
    public void wrapAlarm(Alarm alarm, Object object) {
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.name().equals(alarm.getType())) {
            Server server = (Server) object;
            alarm.setTarget(server.getId());
            alarm.setVendorId(server.getVendorId());
            alarm.setOwnerId(server.getCreatorId());
            if (server.getName()!=null & server.getManageIp()!=null){
                alarm.setTargetName(server.getName() + "("+ server.getManageIp()+")");
            }else {
                String targetName = server.getManageIp() == null ? server.getName() : server.getManageIp();
                alarm.setTargetName(targetName);
            }
        }
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name().equals(alarm.getType())) {
            CloudServer cloudServer = (CloudServer) object;
            alarm.setTarget(cloudServer.getId());
            alarm.setVendorId(cloudServer.getVendorId());
            alarm.setOwnerId(cloudServer.getCreatorId());
            if (cloudServer.getName()!=null & cloudServer.getManagerIp()!=null){
                alarm.setTargetName(cloudServer.getName() + "("+ cloudServer.getManagerIp()+")");
            }else {
                String targetName = cloudServer.getManagerIp() == null ? cloudServer.getName() : cloudServer.getManagerIp();
                alarm.setTargetName(targetName);
            }
        }
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Object object) {
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_HOST.name().equals(ruleGroup.getResourceType())) {
            Server server = (Server) object;
            return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(server.getId()), ruleGroup.getTenantId());
        }
        if (MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name().equals(ruleGroup.getResourceType())) {
            CloudServer cloudServer = (CloudServer) object;
            return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(cloudServer.getId()), ruleGroup.getTenantId());
        }
        return true;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_FUSIONCLOUD_VM, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_FUSIONCLOUD_HOST, this);
    }
}
