package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.H3cVmService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.ImsCloudServerService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

/**
 * h3c云主机
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class H3cVmServiceImpl implements H3cVmService, InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(H3cVmService.class);

    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmService alarmService;

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_H3C_VM, this);
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        BocloudResult result = imsCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("id", targetObject.getInstanceId());
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String instanceId = String.valueOf(alert.getLabels().get("id"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
        BocloudResult result = imsCloudServerService.list(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        Assert.isTrue(list != null && !list.isEmpty(), "vm not found,instanceId:" + instanceId);
        return list.get(0);
    }

    @Override
    public void destroy(CloudServer cloudServer, String catalog) throws Exception {
        logger.info("destroy h3c vm : " + cloudServer.getId());
        alarmService.solveByCloudServer(cloudServer.getId(),catalog,1L,"系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, CloudServer object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        if (object.getName()!=null & object.getManagerIp()!=null){
            alarm.setTargetName(object.getName() + "("+ object.getManagerIp()+")");
        }else {
            String targetName = object.getManagerIp() == null ? object.getName() : object.getManagerIp();
            alarm.setTargetName(targetName);
        }
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }
}
