package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.entity.Resource;
import com.bocloud.cms.intf.ResourceService;
import com.bocloud.cms.repository.ResourceRepository;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 告警规则 绑定资源
 * @author fanzhenxi
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class ResourceServiceImpl implements ResourceService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceServiceImpl.class);
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    StorageConfig storageConfig;


    @Override
    public BocloudResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(16, 0.75F)));
            //获取监控的数据类型
            List<String> types = Lists.newArrayList();
            Iterator<Param> iterator = pager.getParams().iterator();
            while (iterator.hasNext()) {
                Param param = iterator.next();
                Map<String, Object> map = param.getParam();
                if (map.containsKey("types")) {
                    types = JSONArray.parseArray(String.valueOf(map.get("types")), String.class);
                    iterator.remove();
                }
            }
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = this.resourceRepository.count(pager.getParams(), context, types);
            List<Resource> list = this.resourceRepository.list(pager, context, types);
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query resource list fail:", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
