package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.StorageService;
import com.bocloud.cms.model.DiskModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.StorageModel;
import com.bocloud.cms.model.VolumeModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.DmsStorageService;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.dms.entity.Disk;
import com.bocloud.dms.entity.Storage;
import com.bocloud.dms.entity.Volume;
import com.bocloud.ims.entity.CloudVendor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 存储Service实体类
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service
public class StorageServiceImpl implements StorageService, InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(StorageService.class);
    @Autowired
    private DmsStorageService dmsStorageService;
    @Autowired
    private ImsCloudVendorService imsCloudVendorService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public BocloudResult listDevices(Pager pager, RequestContext context, HttpServletRequest request) {

        BocloudResult result = dmsStorageService.listDevices(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<StorageModel> list = data.getJSONArray(Common.ROWS).toJavaList(StorageModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_STORAGE_HUAWEI_STORAGE.name(), targets, context);
        for (StorageModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public BocloudResult listDisks(Pager pager, RequestContext context, HttpServletRequest request) {
        BocloudResult result = dmsStorageService.listDisks(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<DiskModel> list = data.getJSONArray(Common.ROWS).toJavaList(DiskModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_STORAGE_HUAWEI_DISK.name(), targets, context);
        for (DiskModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public BocloudResult listVolumes(Pager pager, RequestContext context, String action, HttpServletRequest request) {
        BocloudResult result = dmsStorageService.listVolumes(pager, request.getHeader(Common.TOKEN), action);
        JSONObject data = (JSONObject) result.getData();
        List<VolumeModel> list = data.getJSONArray(Common.ROWS).toJavaList(VolumeModel.class);
        if (list.isEmpty()) {
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_STORAGE_HUAWEI_VOLUME.name(), targets, context);
        for (VolumeModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    /**
     * 获取资源列表
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List listTargetObjects(List targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        for (Object targetId : targetIds) {
            targetList.add(Integer.valueOf(targetId.toString()));
        }
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        if (MonitorResourceType.MONITOR_STORAGE_HUAWEI_STORAGE.equals(resourceType)
                || MonitorResourceType.MONITOR_STORAGE_DELL_STORAGE.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = dmsStorageService.listDevices(new Pager(1, Integer.MAX_VALUE, params, null),null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(Storage.class);
        }
        if (MonitorResourceType.MONITOR_STORAGE_HUAWEI_DISK.equals(resourceType)
                || MonitorResourceType.MONITOR_STORAGE_DELL_DISK.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = dmsStorageService.listDisks(new Pager(1, Integer.MAX_VALUE, params, null),null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(Disk.class);
        }
        if (MonitorResourceType.MONITOR_STORAGE_HUAWEI_VOLUME.equals(resourceType)
                || MonitorResourceType.MONITOR_STORAGE_DELL_VOLUME.equals(resourceType)) {
            List<Param> params = Collections.singletonList(param);
            BocloudResult result = dmsStorageService.listVolumes(new Pager(1, Integer.MAX_VALUE, params, null),null,"smis");
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            return data.getJSONArray(Common.ROWS).toJavaList(Volume.class);
        }
        return null;
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Object targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        switch (MonitorResourceType.valueOf(metric.getResourceType())) {
            case MONITOR_STORAGE_HUAWEI_STORAGE:
            case MONITOR_STORAGE_DELL_STORAGE:
                Storage storage = (Storage) targetObject;
                params.put("instanceUuid", String.valueOf(storage.getStorageUuid()));
                params.put("vendorId", String.valueOf(storage.getVendorId()));
                params.put("instanceType", "storage");
                break;
            case MONITOR_STORAGE_HUAWEI_DISK:
            case MONITOR_STORAGE_DELL_DISK:
                Disk disk = (Disk) targetObject;
                params.put("instanceUuid", String.valueOf(disk.getDiskUuid()));
                params.put("instanceType", "disk");
                break;
            case MONITOR_STORAGE_HUAWEI_VOLUME:
            case MONITOR_STORAGE_DELL_VOLUME:
                Volume volume = (Volume) targetObject;
                params.put("instanceUuid", volume.getVolumeUuid());
                params.put("vendorId", String.valueOf(volume.getVendorId()));
                params.put("instanceType", "volume");
                break;
            default:
                break;
        }
        return params;
    }

    @Override
    public Object parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_STORAGE_HUAWEI_STORAGE == MonitorResourceType.valueOf(ruleGroup.getResourceType())
                || MonitorResourceType.MONITOR_STORAGE_DELL_STORAGE == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String storageUuid = String.valueOf(alert.getLabels().get("instanceUuid"));
            String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
            List<Param> params = new ArrayList<>(2);
            params.add(new Param(MapTools.simpleMap("storageUuid", storageUuid), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("vendorId", Long.valueOf(vendorId)), Sign.EQ));
            BocloudResult result = dmsStorageService.listDevices(new Pager(1, 1, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<Storage> list = data.getJSONArray(Common.ROWS).toJavaList(Storage.class);
            Assert.isTrue(list != null && !list.isEmpty(), "storage not found,storageUuid:" + storageUuid + ", vendorId:" + vendorId);
            return list.get(0);
        } else if (MonitorResourceType.MONITOR_STORAGE_HUAWEI_DISK == MonitorResourceType.valueOf(ruleGroup.getResourceType())
                || MonitorResourceType.MONITOR_STORAGE_DELL_DISK == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String diskUuid = String.valueOf(alert.getLabels().get("instanceUuid"));
            List<Param> params = new ArrayList<>(1);
            params.add(new Param(MapTools.simpleMap("diskUuid", diskUuid), Sign.EQ));
            BocloudResult result = dmsStorageService.listDisks(new Pager(1, 1, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<Disk> list = data.getJSONArray(Common.ROWS).toJavaList(Disk.class);
            Assert.isTrue(list != null && !list.isEmpty(), "storage not found,diskUuid:" + diskUuid);
            return list.get(0);
        } else if (MonitorResourceType.MONITOR_STORAGE_HUAWEI_VOLUME == MonitorResourceType.valueOf(ruleGroup.getResourceType())
                || MonitorResourceType.MONITOR_STORAGE_DELL_VOLUME == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String volumeUuid = String.valueOf(alert.getLabels().get("instanceUuid"));
            String vendorId = String.valueOf(alert.getLabels().get("vendorId"));
            List<Param> params = new ArrayList<>(2);
            params.add(new Param(MapTools.simpleMap("volumeUuid", volumeUuid), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("vendorId", Long.valueOf(vendorId)), Sign.EQ));
            BocloudResult result = dmsStorageService.listDisks(new Pager(1, 1, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<Volume> list = data.getJSONArray(Common.ROWS).toJavaList(Volume.class);
            Assert.isTrue(list != null && !list.isEmpty(), "storage not found,volumeUuid:" + volumeUuid + ", vendorId:" + vendorId);
            return list.get(0);
        } else {
            throw new IllegalArgumentException(ruleGroup.getResourceType());
        }
    }

    @Override
    public void destroy(Object object, String catalog) throws Exception {
        BocloudResult<CloudVendor> vendorBocloudResult;
        CloudVendor vendor;
        switch (MonitorResourceType.valueOf(catalog)) {
            case MONITOR_STORAGE_HUAWEI_STORAGE:
            case MONITOR_STORAGE_DELL_STORAGE:
                Storage storage = (Storage) object;
                logger.info("storage : " + storage.getId());
                vendorBocloudResult = imsCloudVendorService.detail(storage.getVendorId());
                if (vendorBocloudResult.isFailed()) {
                    logger.warn("query vendor failed, vendorId:" + storage.getVendorId() + ", " + vendorBocloudResult.getMessage());
                    return;
                }
                vendor = vendorBocloudResult.getData();
                if (null == vendor) {
                    logger.warn("query vendor failed, vendorId:" + storage.getVendorId() + ", maybe this vendor has been deleted!");
                    return;
                }
                //TODO
                break;
            case MONITOR_STORAGE_HUAWEI_DISK:
            case MONITOR_STORAGE_DELL_DISK:
                Volume volume = (Volume) object;
                logger.info("volume : " + volume.getId());
                vendorBocloudResult = imsCloudVendorService.detail(volume.getVendorId());
                if (vendorBocloudResult.isFailed()) {
                    logger.warn("query vendor failed, vendorId:" + volume.getVendorId() + ", " + vendorBocloudResult.getMessage());
                    return;
                }
                vendor = vendorBocloudResult.getData();
                if (null == vendor) {
                    logger.warn("query vendor failed, vendorId:" + volume.getVendorId() + ", maybe this vendor has been deleted!");
                    return;
                }
                //TODO
                break;
            case MONITOR_STORAGE_HUAWEI_VOLUME:
            case MONITOR_STORAGE_DELL_VOLUME:
                Disk disk = (Disk) object;
                logger.info("disk : " + disk.getId());
                vendorBocloudResult = imsCloudVendorService.detail(disk.getVendorId());
                if (vendorBocloudResult.isFailed()) {
                    logger.warn("query vendor failed, vendorId:" + disk.getVendorId() + ", " + vendorBocloudResult.getMessage());
                    return;
                }
                vendor = vendorBocloudResult.getData();
                if (null == vendor) {
                    logger.warn("query vendor failed, vendorId:" + disk.getVendorId() + ", maybe this vendor has been deleted!");
                    return;
                }
                //TODO
                break;
            default:
                break;
        }
    }

    @Override
    public void wrapAlarm(Alarm alarm, Object object) {
        switch (MonitorResourceType.valueOf(alarm.getType())) {
            case MONITOR_STORAGE_HUAWEI_STORAGE:
            case MONITOR_STORAGE_DELL_STORAGE:
                Storage storage = (Storage)object;
                alarm.setTarget(storage.getId());
                alarm.setTargetName(storage.getName());
                break;
            case MONITOR_STORAGE_HUAWEI_DISK:
            case MONITOR_STORAGE_DELL_DISK:
                Disk disk = (Disk)object;
                alarm.setTarget(disk.getId());
                alarm.setTargetName(disk.getName());
                break;
            case MONITOR_STORAGE_HUAWEI_VOLUME:
            case MONITOR_STORAGE_DELL_VOLUME:
                Volume volume = (Volume)object;
                alarm.setTarget(volume.getId());
                alarm.setTargetName(volume.getName());
                break;
            default:
                break;
        }

    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Object object) {
        switch (MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            case MONITOR_STORAGE_HUAWEI_STORAGE:
            case MONITOR_STORAGE_DELL_STORAGE:
                Storage storage = (Storage) object;
                return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(storage.getId()), ruleGroup.getTenantId());
            case MONITOR_STORAGE_HUAWEI_DISK:
            case MONITOR_STORAGE_DELL_DISK:
                Disk disk = (Disk) object;
                return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(disk.getId()), ruleGroup.getTenantId());
            case MONITOR_STORAGE_HUAWEI_VOLUME:
            case MONITOR_STORAGE_DELL_VOLUME:
                Volume volume = (Volume) object;
                return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(volume.getId()), ruleGroup.getTenantId());
            default:
                break;
        }
        return true;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_STORAGE_HUAWEI_STORAGE, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_STORAGE_HUAWEI_DISK, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_STORAGE_HUAWEI_VOLUME, this);

        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_STORAGE_DELL_STORAGE, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_STORAGE_DELL_DISK, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_STORAGE_DELL_VOLUME, this);
    }

}
