package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.TopBaseEntity;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.TopService;
import com.bocloud.cms.model.LineChartResult;
import com.bocloud.cms.model.LineChartResult.TopParams;
import com.bocloud.cms.model.LineChartResult.Values;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.service.internal.ImsCloudServerService;
import com.bocloud.cms.service.internal.ImsCloudVendorService;
import com.bocloud.cms.service.internal.ImsServerService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * Top5监控图表
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Service("topService")
public class TopServiceImpl implements TopService {

    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("top-pool-%d").build();
    private final ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);
    private static final Logger logger = LoggerFactory.getLogger(TopServiceImpl.class);
    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private ImsServerService imsServerService;
    @Autowired
    private ImsCloudVendorService imsCloudVendorService;
    @Autowired
    private PrometheusService prometheusService;

    /**
     * 主机CpuTop
     * @param vendorId
     * @param limit
     * @return
     */
    @Override
    public LineChartResult topOfHostCpu(Long vendorId, Integer limit) {
        if (null == vendorId) {
            return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",vsphere_host_cpu_usage_average{cpu='instance-total'} or (1 - avg(irate(node_cpu_seconds_total{mode='idle',vendor_id!=''}[5m])) by (host_id))*100 or bocloud_fusioncloud_host_cpuUsage)),0.01)");
        }
        CloudVendor vendor = imsCloudVendorService.detail(vendorId).getData();
        switch (CloudProvider.valueOf(vendor.getType())) {
            case OPENSTACK:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",(1 - avg(irate(node_cpu_seconds_total{mode='idle',vendor_id='" + vendor.getId() + "'}[5m])) by (host_id))*100)),0.01)");
            case VMWARE:
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",vsphere_host_cpu_usage_average{cpu='instance-total',vcenter='" + address + "'})),0.01)");
            case MANAGEONE:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",bocloud_fusioncloud_host_cpuUsage{vendorId='" + vendorId + "'})),0.01)");
            case SMARTX:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",smartx_host_cpu_overall_usage_percent{vendorId='" + vendorId + "'})),0.01)");
            case CLOUDTOWER:
                return hostLineChart("宿主机CPU利用率", "round(sort_desc(topk(" + limit + ",bocloud_cloudtower_host_host_cpu_overall_usage_percent{vendorId='" + vendorId + "'})),0.01)");
            default:
                return new LineChartResult(null, new Values[]{new Values("宿主机cpu利用率", null)}, null);
        }
    }

    /**
     * 主机MemTop
     * @param vendorId
     * @param limit
     * @return
     */
    @Override
    public LineChartResult topOfHostMem(Long vendorId, Integer limit) {
        if (null == vendorId) {
            return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",vsphere_host_mem_usage_average or 100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes{vendor_id!=''})/node_memory_MemTotal_bytes) * 100 or bocloud_fusioncloud_host_memoryUsage)),0.01)");
        }
        CloudVendor vendor = imsCloudVendorService.detail(vendorId).getData();
        switch (CloudProvider.valueOf(vendor.getType())) {
            case OPENSTACK:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ", 100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes{vendor_id!=''})/node_memory_MemTotal_bytes) * 100)),0.01)");
            case VMWARE:
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",vsphere_host_mem_usage_average{vcenter='" + address + "'})),0.01)");
            case MANAGEONE:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",bocloud_fusioncloud_host_memoryUsage{vendorId='" + vendorId + "'})),0.01)");
            case SMARTX:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",smartx_host_memory_usage_percent{vendorId='" + vendorId + "'})),0.01)");
            case CLOUDTOWER:
                return hostLineChart("宿主机内存利用率", "round(sort_desc(topk(" + limit + ",bocloud_cloudtower_host_host_memory_usage_percent{vendorId='" + vendorId + "'})),0.01)");
            default:
                return new LineChartResult(null, new Values[]{new Values("宿主机内存利用率", null)}, null);
        }
    }

    /**
     * 虚拟机CpuTop
     * @param context
     * @param vendorId
     * @param limit
     * @param request
     * @return
     */
    @Override
    public LineChartResult topOfvmCpu(RequestContext context, Long vendorId, Integer limit, HttpServletRequest request) {
        String expr = null;
        if (null == vendorId) {
            expr = "round(sort(topk(" + limit + ",vsphere_vm_cpu_usage_average{cpu='instance-total'} or avg(irate(libvirt_domain_info_cpu_time_seconds_total[5m])) by (vendor_id,instanceId,instanceName)/ avg(libvirt_domain_info_virtual_cpus) by (vendor_id,instanceId,instanceName)*100 or bocloud_fusioncloud_vm_cpuUsage)),0.01)";
        } else {
            CloudVendor vendor = imsCloudVendorService.detail(vendorId).getData();
            switch (CloudProvider.valueOf(vendor.getType())) {
                case OPENSTACK:
                    expr = "round(sort(topk(" + limit * 3 + ",avg(irate(libvirt_domain_info_cpu_time_seconds_total{vendor_id='" + vendor.getId() + "'}[5m])) by (vendor_id,instanceId,instanceName)/ avg(libvirt_domain_info_virtual_cpus) by (vendor_id,instanceId,instanceName))*100),0.01)";
                    break;
                case VMWARE:
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    expr = "round(sort(topk(" + limit * 3 + ",vsphere_vm_cpu_usage_average{cpu='instance-total',vcenter='" + address + "'})),0.01)";
                    break;
                case MANAGEONE:
                    expr = "round(sort(topk(" + limit * 3 + ",bocloud_fusioncloud_vm_cpuUsage{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                case SMARTX:
                    expr = "round(sort(topk(" + limit * 3 + ",smartx_elf_vm_cpu_overall_usage_percent{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                case CLOUDTOWER:
                    expr = "round(sort(topk(" + limit * 3 + ",bocloud_cloudtower_vm_elf_vm_cpu_overall_usage_percent{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                default:
                    break;
            }
        }
        return vmLineChart(request, "云主机CPU利用率", expr, limit);
    }

    /**
     * 虚拟机MemTop
     * @param context
     * @param vendorId
     * @param limit
     * @param request
     * @return
     */
    @Override
    public LineChartResult topOfvmMem(RequestContext context, Long vendorId, Integer limit, HttpServletRequest request) {
        String expr = null;
        if (null == vendorId) {
            expr = "round(sort(topk(" + limit * 3 + ",vsphere_vm_mem_usage_average or (1-libvirt_domain_memory_stat_unused/libvirt_domain_memory_stat_available)*100 or bocloud_fusioncloud_vm_memoryUsage)),0.01)";
        } else {
            CloudVendor vendor = imsCloudVendorService.detail(vendorId).getData();
            switch (CloudProvider.valueOf(vendor.getType())) {
                case OPENSTACK:
                    expr = "round(sort(topk(" + limit * 3 + ",(1-libvirt_domain_memory_stat_unused/libvirt_domain_memory_stat_available{vendor_id='" + vendor.getId() + "'})*100)),0.01)";
                    break;
                case VMWARE:
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    expr = "round(sort(topk(" + limit * 3 + ",vsphere_vm_mem_usage_average{vcenter='" + address + "'})),0.01)";
                    break;
                case MANAGEONE:
                    expr = "round(sort(topk(" + limit * 3 + ",bocloud_fusioncloud_vm_memoryUsage{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                case SMARTX:
                    expr = "round(sort(topk(" + limit * 3 + ",smartx_elf_vm_memory_usage_percent{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                case CLOUDTOWER:
                    expr = "round(sort(topk(" + limit * 3 + ",bocloud_cloudtower_vm_elf_vm_memory_usage_percent{vendorId='" + vendor.getId() + "'})),0.01)";
                    break;
                default:
                    break;
            }
        }
        return vmLineChart(request, "云主机内存利用率", expr, limit);
    }

    /**
     * 主机Top图表
     * @param name
     * @param expr
     * @return
     */
    private LineChartResult hostLineChart(String name, String expr) {
        // 获取当前时间戳
        String time = Long.toString(System.currentTimeMillis()).substring(0, 10);
        List<PrometheusResult> list = prometheusService.query(expr,time);
        List<String> keys = new ArrayList<>(list.size());
        List<Object> data = new ArrayList<>(list.size());
        // ids列表，用于作监控详情信息展示
        Map<String, TopParams> params = new HashMap<>(16, 0.75F);
        List<Future<TopBaseEntity>> futures = new ArrayList<>();
        for (PrometheusResult result : list) {
            List<Param> paramList = new ArrayList<>();
            Future<TopBaseEntity> future = executorService.submit(() -> {
                String hostName = "";
                if (result.getMetric().containsKey("vcenter")) {
                    String esxhostname = String.valueOf(result.getMetric().get("esxhostname"));
                    paramList.add(new Param(MapTools.simpleMap("name", esxhostname), Sign.EQ));
                } else if (result.getMetric().containsKey("host_id")) {
                    //openstack 使用id查询
                    String id = String.valueOf(result.getMetric().get("host_id"));
                    paramList.add(new Param(MapTools.simpleMap("id", id), Sign.EQ));
                } else if (result.getMetric().containsKey("exported_instance")) {
                    //smartX
                    String smartxName = String.valueOf(result.getMetric().get("name"));
                    paramList.add(new Param(MapTools.simpleMap("name", smartxName), Sign.EQ));
                } else {
                    //manageone 使用名称查询
                    hostName = String.valueOf(result.getMetric().get("name"));
                    paramList.add(new Param(MapTools.simpleMap("name", hostName), Sign.EQ));
                }
                BocloudResult serverResult = imsServerService.list(new Pager(1, 1, paramList, null));
                return new TopBaseEntity(serverResult, hostName, result);
            });
            futures.add(future);
        }
        futures.forEach(future -> {
            try {
                TopBaseEntity topBaseEntity = future.get();
                TopParams topParams = new TopParams();
                String resultName = topBaseEntity.getResourceName();
                if (topBaseEntity.getBocloudResult().isSuccess()) {
                    List<Server> hosts = ((JSONObject) topBaseEntity.getBocloudResult().getData()).getJSONArray(Common.ROWS).toJavaList(Server.class);
                    if (!hosts.isEmpty()) {
                        Server server = hosts.get(0);
                        resultName = server.getName();
                        topParams.setId(server.getId());
                        topParams.setType(server.getVendorType());
                        topParams.setVendorId(server.getVendorId());
                        topParams.setIdentifier(server.getVendorType().equals(CloudProvider.MANAGEONE.name()) ? server.getName() : server.getManageIp());
                    }
                    params.put(resultName, topParams);
                }
                keys.add(resultName);
                data.add(topBaseEntity.getPrometheusResult().getValue()[1]);
            } catch (Exception e) {
                logger.error("查询宿主机Top列表失败！" + e);
            }
        });
        return new LineChartResult(keys, new Values[]{new Values(name, data)}, params);
    }

    /**
     * 虚拟机Top图表
     * @param name
     * @param expr
     * @return
     */
    private LineChartResult vmLineChart(HttpServletRequest request, String name, String expr, int limit) {
        // 获取当前时间戳
        String time = Long.toString(System.currentTimeMillis()).substring(0, 10);
        List<PrometheusResult> list = prometheusService.query(expr,time);
        List<String> keys = new ArrayList<>(list.size());
        List<Object> data = new ArrayList<>(list.size());
        List<Future<TopBaseEntity>> futures = new ArrayList<>();
        // id列表，用于作监控详情信息展示
        Map<String, TopParams> params = new HashMap<>(16, 0.75F);
        for (PrometheusResult result : list) {
            Future<TopBaseEntity> future = executorService.submit(() -> {
                CloudProvider vendorType = null;
                if (result.getMetric().containsKey("vendor_id")) {
                    BocloudResult<CloudVendor> vendorBocloudResult = imsCloudVendorService.detail(Long.valueOf(String.valueOf(result.getMetric().get("vendor_id"))));
                    vendorType = CloudProvider.valueOf(vendorBocloudResult.getData().getType());
                } else if (result.getMetric().containsKey("vendorId")) {
                    BocloudResult<CloudVendor> vendorBocloudResult = imsCloudVendorService.detail(Long.valueOf(String.valueOf(result.getMetric().get("vendorId"))));
                    vendorType = CloudProvider.valueOf(vendorBocloudResult.getData().getType());
                }
                List<Param> paramList = new ArrayList<>();
                String vmName;
                if (result.getMetric().containsKey("vcenter")) {
                    //vmware 使用name查询
                    vmName = String.valueOf(result.getMetric().get("vmname"));
                    String uuid = String.valueOf(result.getMetric().get("uuid"));
                    paramList.add(new Param(MapTools.simpleMap("biosUuid", uuid), Sign.EQ));
                } else if (result.getMetric().containsKey("vendor_id") && CloudProvider.OPENSTACK == vendorType) {
                    //openstack 使用instanceId查询
                    vmName = String.valueOf(result.getMetric().get("instanceName"));
                    String instanceId = String.valueOf(result.getMetric().get("instanceId"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("vendorId") && CloudProvider.SMARTX == vendorType) {
                    //smartx 使用vm查询
                    vmName = String.valueOf(result.getMetric().get("vm"));
                    String instanceId = String.valueOf(result.getMetric().get("vm"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else if (result.getMetric().containsKey("vendorId") && CloudProvider.CLOUDTOWER == vendorType) {
                    //cloudTower 使用vm查询
                    vmName = String.valueOf(result.getMetric().get("name"));
                    String instanceId = String.valueOf(result.getMetric().get("instanceId"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                } else {
                    //manageone 使用instanceId查询
                    vmName = String.valueOf(result.getMetric().get("name"));
                    String instanceId = String.valueOf(result.getMetric().get("id"));
                    paramList.add(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
                }
                BocloudResult bocloudResult = imsCloudServerService.list(new Pager(1, 1, paramList, null),
                        request.getHeader(Common.TOKEN));
                return new TopBaseEntity(bocloudResult,vmName,result);
            });
            futures.add(future);
        }
        int count = 0;
        for (Future<TopBaseEntity> future : futures) {
            try {
                TopBaseEntity topBaseEntity = future.get();
                TopParams topParams = new TopParams();
                if (topBaseEntity.getBocloudResult().isSuccess()){
                    count++;
                    List<CloudServer> vms = ((JSONObject) topBaseEntity.getBocloudResult().getData()).getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
                    if (!vms.isEmpty()) {
                        CloudServer cloudServer = vms.get(0);
                        topParams.setId(cloudServer.getId());
                        topParams.setType(cloudServer.getVendorType());
                        topParams.setVendorId(cloudServer.getVendorId());
                        topParams.setHostName(cloudServer.getHostName());
                        topParams.setIdentifier(cloudServer.getVendorType().equals(CloudProvider.VMWARE.name()) ? cloudServer.getName() : cloudServer.getInstanceId());
                        params.put(topBaseEntity.getResourceName(), topParams);
                        if ("SMARTX".equals(cloudServer.getVendorType())){
                            keys.add(cloudServer.getName());
                        }else{
                            keys.add(topBaseEntity.getResourceName());
                        }
                        data.add(topBaseEntity.getPrometheusResult().getValue()[1]);
                    }
                }
            } catch (Exception e) {
                logger.error("查询宿主机Top列表失败！" + e);
            }
        }
        List<String> keys2 = new ArrayList<>();
        List<Object> data2 = new ArrayList<>();
        int resultCount = limit;
        for (int i = keys.size() - 1; i >= 0; i--) {
            if (resultCount>0){
                keys2.add(keys.get(i));
                data2.add(data.get(i));
                resultCount--;
            }
        }
        return new LineChartResult(keys2, new Values[]{new Values(name, data2)}, params);
    }
}
