package com.bocloud.cms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.CloudServerStatusEnum;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.intf.VmService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.IpRelationRepository;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.ImsCloudServerService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 云主机Service层
 *
 * @author zhangyf
 * @version 5.6.0-SNAPSHOT
 * @date 2022/2/21 18:01
 */
@Service
public class VmServiceImpl implements VmService, InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(VmServiceImpl.class);

    @Autowired
    private ImsCloudServerService imsCloudServerService;
    @Autowired
    private IpRelationRepository ipRelationRepository;
    @Autowired
    private AlarmService alarmService;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public BocloudResult list(Pager pager, RequestContext context, HttpServletRequest request) {
        BocloudResult result = imsCloudServerService.list(pager, request.getHeader(Common.TOKEN));
        List<String> targets = new ArrayList<>();
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<CloudServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServerModel.class);
        if (list.isEmpty()) {
            return result;
        }
        // 默认全部开启
        for (CloudServerModel cloudServerModel : list) {
            cloudServerModel.setAlarmEnable(true);
        }
        handleVmwareList(context, list);
        handleOpenStackList(list, context);
        handleHuaweiList(list, context);
        handleFusionCloudList(list, context);
        handleManageOneList(list, context);
        handleTianyiList(list, context);
        handleH3cList(list, context);
        handleAliyunList(list, context);
        handleHmcList(list, context);
        handleTencentList(list, context);
        handleSmartxList(list, context);
        handleCloudTower(list, context);
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public BocloudResult vmOverview(Long id) {
        BocloudResult<CloudServer> cloudServer = imsCloudServerService.detail(id);
        Assert.isTrue(cloudServer.isSuccess(), " 未查询到该云主机相关信息");
        HashMap<Object, Object> result = new HashMap<>(16, 0.75F);
        Date date = cloudServer.getData().getGmtCreate();
        result.put("name", cloudServer.getData().getName());
        result.put("gmtCreate", DateTools.formatTime(date));
        result.put("ip", cloudServer.getData().getManagerIp());
        result.put("powerStatus", cloudServer.getData().getStatus());
        return new BocloudResult(true, result, "查询成功");
    }

    @Override
    public void saveMonitorIp(RequestContext context, Long id, String ip) {
        IpRelation relation = ipRelationRepository.query(id, context.getTenant());
        relation = Optional.ofNullable(relation).orElse(new IpRelation());
        relation.setTargetId(id);
        relation.setIp(ip);
        relation.setTenantId(context.getTenant());
        if (relation.getId() == null) {
            ipRelationRepository.save(relation);
        } else {
            ipRelationRepository.update(relation);
        }
    }

    @Override
    public String queryMonitorIp(RequestContext context, Long id) {
        //监控IP
        String monitorIp = null;
        if (context.getTenant() != 0L) {
            BocloudResult<CloudServer> cloudServerResult = imsCloudServerService.detail(id);
            monitorIp = cloudServerResult.getData().getManagerIp();
        }
        IpRelation relation = ipRelationRepository.query(id, context.getTenant());
        if (Optional.ofNullable(relation).isPresent()) {
            return relation.getIp();
        } else {
            if (StringUtils.isBlank(monitorIp) && context.getTenant() == 0L) {
                BocloudResult<CloudServer> cloudServerResult = imsCloudServerService.detail(id);
                monitorIp = cloudServerResult.getData().getManagerIp();
            }
            return monitorIp;
        }
    }

    @Override
    public void deleteIpRelationsByTarget(Long targetId) {
        ipRelationRepository.deleteIpRelationsByTarget(targetId);
    }

    @Override
    public BocloudResult<Map<String, Object>> listDown(Pager pager, RequestContext context, HttpServletRequest request) {

        SXSSFWorkbook workbook = new SXSSFWorkbook();
        try {
            SXSSFSheet sheet = workbook.createSheet("sheet");
            sheet.setRandomAccessWindowSize(-1);
            // 样式一，标题数据蓝色填充
            CellStyle style = workbook.createCellStyle();
            style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
            style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            style.setBorderBottom(BorderStyle.THIN); // 下边框
            style.setBorderLeft(BorderStyle.THIN);// 左边框
            style.setBorderTop(BorderStyle.THIN);// 上边框
            style.setBorderRight(BorderStyle.THIN);// 右边框
            style.setAlignment(HorizontalAlignment.CENTER);
            // 样式二，内容样式
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderTop(BorderStyle.THIN);// 上边框
            cellStyle.setBorderBottom(BorderStyle.THIN); // 下边框
            cellStyle.setBorderLeft(BorderStyle.THIN);// 左边框
            cellStyle.setBorderRight(BorderStyle.THIN);// 右边框
            cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平对齐方式
            cellStyle.setVerticalAlignment(VerticalAlignment.TOP);// 垂直对齐方式
            // 标题写入
            createVmReport(sheet, style);
            pager.setPage(1);
            pager.setRows(Integer.MAX_VALUE);
            BocloudResult result = this.list(pager, context, request);
            if (result.isFailed()) {
                return result;
            }
            JSONObject data = (JSONObject) result.getData();
            List<CloudServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServerModel.class);
            if (list.isEmpty()) {
                return new BocloudResult<>(false, "暂无租户");
            }
            Integer count = 2;
            for (CloudServerModel cloudServerModel : list) {
                SXSSFRow sheetRow = sheet.createRow(count);
                excelExporter(cloudServerModel, sheet, cellStyle, sheetRow, count);
                count++;
            }

            String fileName = "云主机监控列表";
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage(), e);
            return new BocloudResult<>(false, e.getMessage());
        } finally {
            if (null != workbook) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    logger.error("error message：", e);
                }
            }
        }
    }

    private void excelExporter(CloudServerModel cloudServerModel, SXSSFSheet sheet, CellStyle cellStyle, SXSSFRow sheetRow, Integer count) {

        sheetRow.createCell(0).setCellValue(cloudServerModel.getName());
        sheet.getRow(count).getCell(0).setCellStyle(cellStyle);

        String ip = "";
        try {
            List<JSONObject> ipList = JSON.parseObject(cloudServerModel.getPrivateIps(), List.class);
            if(null != ipList && !ipList.isEmpty()){
                for (JSONObject ipInfo : ipList) {
                    ip += "(内网)" + ipInfo.getString("address") + "\n";
                }
            }
        }catch (Exception e){
            logger.warn("解析深信服的IP地址报错:{}", e.getMessage());
            ip = "(内网)" + cloudServerModel.getPrivateIps();
        }

        sheetRow.createCell(1).setCellValue(ip.endsWith("\n") ? ip.substring(0, ip.length() - 1) : ip);
        sheet.getRow(count).getCell(1).setCellStyle(cellStyle);
        sheetRow.createCell(2).setCellValue(cloudServerModel.getVendorType());
        sheet.getRow(count).getCell(2).setCellStyle(cellStyle);

        sheetRow.createCell(3).setCellValue(CloudServerStatusEnum.getNameByType(cloudServerModel.getStatus()));
        sheet.getRow(count).getCell(3).setCellStyle(cellStyle);
        String spec = cloudServerModel.getCpu() + "C" + cloudServerModel.getMemory().stripTrailingZeros().toPlainString() + "G" + cloudServerModel.getDisk() + "G";
        sheetRow.createCell(4).setCellValue(spec);
        sheet.getRow(count).getCell(4).setCellStyle(cellStyle);
        sheetRow.createCell(5).setCellValue(cloudServerModel.getOsName());
        sheet.getRow(count).getCell(5).setCellStyle(cellStyle);
        sheetRow.createCell(6).setCellValue(cloudServerModel.getCpuUsage());
        sheet.getRow(count).getCell(6).setCellStyle(cellStyle);
        sheetRow.createCell(7).setCellValue(cloudServerModel.getMemUsage());
        sheet.getRow(count).getCell(7).setCellStyle(cellStyle);
        sheetRow.createCell(8).setCellValue(cloudServerModel.getTenantName());
        sheet.getRow(count).getCell(8).setCellStyle(cellStyle);
        sheetRow.createCell(9).setCellValue(cloudServerModel.getProjectName());
        sheet.getRow(count).getCell(9).setCellStyle(cellStyle);
        sheetRow.createCell(10).setCellValue(cloudServerModel.getRemark());
        sheet.getRow(count).getCell(10).setCellStyle(cellStyle);
    }


    /**
     * 创建标题
     *
     * @param sheet
     * @param style
     * @param action
     */
    public void createVmReport(SXSSFSheet sheet, CellStyle style) {
        // 模板标题处理
        String[] titles = new String[]{"名称", "IP", "平台", "状态", "规格", "操作系统", "CPU利用率(%)", "内存利用率(%)", "所属租户", "所属项目","备注"};
        int number = -1;
        SXSSFRow row = sheet.createRow(0);
        for (String title : titles) {
            number++;
            row.createCell(number).setCellValue(title);
            sheet.getRow(0).getCell(number).setCellStyle(style);
            sheet.setColumnWidth(number, 30 * 256);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, number, number));
        }
    }

    /**
     * 获取资源列表
     *
     * @param targetIds    目标资源集合 适配dm特殊化处理string->Integer
     * @param resourceType 资源类型
     * @return
     */
    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        BocloudResult result = imsCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer cloudServer, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        if (MonitorResourceType.MONITOR_VCENTER_VM.name().equals(metric.getResourceType())) {
            params.put("uuid", cloudServer.getBiosUuid());
        } else if (MonitorResourceType.MONITOR_OPENSTACK_VM.name().equals(metric.getResourceType())) {
            // openStack云主机连接状态特殊处理
            if ("bocloud_openstack_vm_success".equals(metric.getExpr())) {
                params.put("instanceId", cloudServer.getInstanceId());
                params.put("name", cloudServer.getName());
            } else if (metric.getExpr().contains("libvirt_domain_")) {
                params.put("instanceId", cloudServer.getInstanceId());
            } else {
                params.put("id", cloudServer.getInstanceId());
            }
        } else {
            throw new IllegalArgumentException(metric.getResourceType());
        }
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        if (MonitorResourceType.MONITOR_VCENTER_VM == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String uuid = String.valueOf(alert.getLabels().get("uuid"));
            List<Param> params = new ArrayList<>(1);
            params.add(new Param(MapTools.simpleMap("biosUuid", uuid), Sign.EQ));
            BocloudResult result = imsCloudServerService.list(new Pager(1, 1, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
            Assert.isTrue(list != null && !list.isEmpty(), "not found vm,biosUuid:" + uuid);
            return list.get(0);
        } else if (MonitorResourceType.MONITOR_OPENSTACK_VM == MonitorResourceType.valueOf(ruleGroup.getResourceType())) {
            String instanceId = String.valueOf(alert.getLabels().get("instanceId"));
            List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", instanceId), Sign.EQ));
            BocloudResult result = imsCloudServerService.list(new Pager(1, 1, params, null), null);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            JSONObject data = (JSONObject) result.getData();
            List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
            Assert.isTrue(list != null && !list.isEmpty(), "not found vm,instanceId:" + instanceId);
            return list.get(0);
        } else {
            throw new IllegalArgumentException(ruleGroup.getResourceType());
        }
    }

    @Override
    public void destroy(CloudServer cloudServer, String catalog) throws Exception {
        logger.info("destroy vm : " + cloudServer.getId());
        alarmService.solveByCloudServer(cloudServer.getId(), catalog, 1L, "系统解决");
        // 删除监控IP设置数据
        deleteIpRelationsByTarget(cloudServer.getId());
    }

    @Override
    public void wrapAlarm(Alarm alarm, CloudServer vm) {
        alarm.setTarget(vm.getId().toString());
        if (vm.getName() != null & vm.getManagerIp() != null) {
            alarm.setTargetName(vm.getName() + "(" + vm.getManagerIp() + ")");
        } else {
            String targetName = vm.getManagerIp() == null ? vm.getName() : vm.getManagerIp();
            alarm.setTargetName(targetName);
        }
        alarm.setOsCategory(vm.getOsCategory());
        alarm.setVendorId(vm.getVendorId());
        alarm.setOwnerId(vm.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_OPENSTACK_VM, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_VCENTER_VM, this);
    }

    private void handleCloudTower(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.CLOUDTOWER.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }

        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<PrometheusResult> cpuList = prometheusService.query("round(bocloud_cloudtower_vm_elf_vm_cpu_overall_usage_percent{instanceId=~'" + instanceId + "'},0.01)");
        List<PrometheusResult> memoryList = prometheusService.query("round(bocloud_cloudtower_vm_elf_vm_memory_usage_percent{instanceId=~'" + instanceId + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_CLOUDTOWER_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            cloudServer.setMemTotal(String.valueOf(cloudServer.getMemory()));
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instanceId").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instanceId").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }

            //告警启用状态，默认开启
            if ( alarmStatuses.size()>0 ){
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())){
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleVmwareList(RequestContext context, List<CloudServerModel> list) {
        List<CloudServerModel> vmwareVms = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.VMWARE.name())).collect(Collectors.toList());
        if (ListTool.isEmpty(vmwareVms)){
            return;
        }

        List<List<CloudServerModel>> pages = Lists.partition(vmwareVms, 200);

        pages.parallelStream().forEach(page->{
            String vmnames = page.stream().map(GenericEntity::getName).map(name -> name.replace("(", "\\\\(").replace(")", "\\\\)")).collect(Collectors.joining("|"));
            List<PrometheusResult> memoryList = prometheusService.query("round(vsphere_vm_mem_usage_average{vmname=~'" + vmnames + "'},0.01)");
            List<PrometheusResult> cpu = prometheusService.query("round(vsphere_vm_cpu_usage_average{vmname=~'" + vmnames + "'},0.01)");
            List<String> target = page.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
            List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_VCENTER_VM.name(), target, context);

            Map<String, String> vNameMapMem = memoryList.stream().collect(Collectors.toMap(s -> String.valueOf(s.getMetric().get("vmname")), s -> String.valueOf(s.getValue()[1])));
            Map<String, String> vNameMapCpu = cpu.stream().collect(Collectors.toMap(s -> String.valueOf(s.getMetric().get("vmname")), s -> String.valueOf(s.getValue()[1])));
            Map<String, Boolean> rIdMapAlarmEnable = alarmStatuses.stream().collect(Collectors.toMap(a -> String.valueOf(a.getResourceId()), ResourceAlarmStatus::getEnable));

            for (CloudServerModel cloudServer : page) {
                cloudServer.setMemTotal(String.valueOf(cloudServer.getMemory()));
                //CPU、内存利用率
                cloudServer.setCpuUsage(vNameMapCpu.get(cloudServer.getName()));
                cloudServer.setMemUsage(vNameMapMem.get(cloudServer.getName()));
                //告警启用状态，默认开启
                cloudServer.setAlarmEnable(rIdMapAlarmEnable.get(cloudServer.getId().toString()));
            }

            List<String> targets = page.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.VMWARE.name()))
                    .map(s -> s.getId().toString()).collect(Collectors.toList());
            //通断状态、响应时长
            warpProbeFiled(context, page, targets);
        });


    }

    private void handleOpenStackList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.OPENSTACK.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> labelParams = Arrays.asList(new LabelParam("instanceId", instanceId, Operator.RE_EQ));
        //拼接参数
        String memoryExpr = "round((1-libvirt_domain_memory_stat_unused/libvirt_domain_memory_stat_available{})*100,0.01)";
        String cpuExpr = "round((avg(irate(libvirt_domain_info_cpu_time_seconds_total{}[5m])) by (instanceId))*100/ avg(libvirt_domain_info_virtual_cpus) by (instanceId),0.01)";
        List<PrometheusResult> memory = prometheusService.query(memoryExpr, labelParams);
        List<PrometheusResult> cpu = prometheusService.query(cpuExpr, labelParams);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_OPENSTACK_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            cloudServer.setMemTotal(String.valueOf(cloudServer.getMemory()));
            //CPU、内存利用率
            Optional.ofNullable(cpu).orElse(new ArrayList<>()).stream()
                    .filter(s -> s.getMetric().get("instanceId").equals(cloudServer.getInstanceId()))
                    .findFirst().ifPresent(s -> cloudServer.setCpuUsage(String.valueOf(s.getValue()[1])));
            Optional.ofNullable(memory).orElse(new ArrayList<>()).stream()
                    .filter(s -> s.getMetric().get("instanceId").equals(cloudServer.getInstanceId()))
                    .findFirst().ifPresent(s -> cloudServer.setMemUsage(String.valueOf(s.getValue()[1])));

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
        //云主机ip集合 查询云主机 响应时长和能通断状态
        List<String> ipList = new ArrayList<>();
        //通过遍历获取所有云主机的ip集合
        for (CloudServerModel server : list) {
            ipList.add(server.getManagerIp());
        }
        String ips = StringUtils.join(ipList.toArray(), "|");
        //云主机通断状态
        List<PrometheusResult> status = prometheusService.query("probe_success{instance=~'" + ips + "'}");
        //云主机响应时长
        List<PrometheusResult> time = prometheusService.query("round(probe_duration_seconds{instance=~'" + ips + "'}*1000,0.001)");
        for (CloudServerModel server : list) {
            //通断状态赋值
            if (!ListTool.isEmpty(status)) {
                for (PrometheusResult onOffStatus : status) {
                    if (onOffStatus.getMetric().get("instance").equals(server.getManagerIp())) {
                        server.setOnOffStatus(String.valueOf(onOffStatus.getValue()[1]));
                    }
                }
            }
            //响应时长赋值
            if (!ListTool.isEmpty(time)) {
                for (PrometheusResult memUsage : time) {
                    if (memUsage.getMetric().get("instance").equals(server.getManagerIp())) {
                        server.setResponseTime(String.valueOf(memUsage.getValue()[1]));
                    }
                }
            }
        }
    }

    private void handleHuaweiList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.HUAWEI.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("instance_id", instanceId, Operator.RE_EQ));
        String cpuExpr = "round(huawei_agt_ecs_cpu_usage{} or huawei_sys_ecs_cpu_util{},0.01)";
        String memoryExpr = "round(huawei_agt_ecs_mem_usedPercent{} or huawei_sys_ecs_mem_util{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HUAWEI_VM.name(), target, context);
        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleFusionCloudList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.FUSIONCLOUD.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<PrometheusResult> cpuList = prometheusService.query("round(bocloud_fusioncloud_vm_cpuUsage{id=~'" + instanceId + "'},0.01)");
        List<PrometheusResult> memoryList = prometheusService.query("round(bocloud_fusioncloud_vm_memoryUsage{id=~'" + instanceId + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleManageOneList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.MANAGEONE.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<PrometheusResult> cpuList = prometheusService.query("round(bocloud_fusioncloud_vm_cpuUsage{id=~'" + instanceId + "'},0.01)");
        List<PrometheusResult> memoryList = prometheusService.query("round(bocloud_fusioncloud_vm_memoryUsage{id=~'" + instanceId + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_FUSIONCLOUD_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleTianyiList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.TIANYI.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }

        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("instance_id", instanceId, Operator.RE_EQ));
        String cpuExpr = "round(tianyi_agt_ecs_cpu_usage{} or tianyi_sys_ecs_cpu_util{},0.01)";
        String memoryExpr = "round(tianyi_agt_ecs_mem_usedPercent{} or tianyi_sys_ecs_mem_util{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_TIANYI_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleH3cList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.H3C.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<PrometheusResult> cpuList = prometheusService.query("round(h3c_vm_cpu_rate{id=~'" + instanceId + "'},0.01)");
        List<PrometheusResult> memoryList = prometheusService.query("round(h3c_vm_mem_rate{id=~'" + instanceId + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_H3C_VM.name(), target, context);
        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }

    }

    private void handleAliyunList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.ALIYUN.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("instance_id", instanceId, Operator.RE_EQ));

        String cpuExpr = "round(aliyun_CPUUtilization{} or aliyun_cpu_total{},0.01)";
        String memoryExpr = "round(aliyun_memory_usedutilization{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_ALIYUN_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleHmcList(List<CloudServerModel> list, RequestContext context) {
        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        // TODO 待开发 MONITOR_HMC_VM参数传递有问题 需要和前端联调
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HMC_VM.name(), target, context);
        for (CloudServerModel cloudServer : list) {
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleTencentList(List<CloudServerModel> list, RequestContext context) {
        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_TENCENT_VM.name(), target, context);
        for (CloudServerModel cloudServer : list) {
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }

    private void handleSmartxList(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.SMARTX.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("vm", instanceId, Operator.RE_EQ));

        String cpuExpr = "round(smartx_elf_vm_cpu_overall_usage_percent{},0.01)";
        String memoryExpr = "round(smartx_elf_vm_memory_usage_percent{},0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_SMARTX_VM.name(), targets, context);
        for (CloudServerModel cloudServer : list) {

            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(cloudServer.getId().toString()))
                    .findAny().isPresent();
            cloudServer.setAlarmEnable(alarmEnable);

            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("vm").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("vm").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
        }
    }

    /**
     * 通断状态、响应时长（当前仅处理了Vmware和OpenStack）
     *
     * @param context
     * @param list
     * @param targets
     */
    private void warpProbeFiled(RequestContext context, List<CloudServerModel> list, List<String> targets) {
        // 从设置的监控ip中取，如果没有设置，则使用管理ip
        List<Long> targetsForIpRelations = targets.stream().map(Long::parseLong).collect(Collectors.toList());
        List<IpRelation> ipRelations = targets.isEmpty() ? null : ipRelationRepository.list(targetsForIpRelations, context.getTenant());
        List<String> ips = Optional.ofNullable(list).orElse(new ArrayList<>()).stream().map(cloudServer ->
                Optional.ofNullable(ipRelations).orElse(new ArrayList<>()).stream()
                        .filter(ipRelation -> ipRelation.getTargetId().equals(cloudServer.getId()))
                        .map(IpRelation::getIp).findFirst().orElse(cloudServer.getManagerIp())).collect(Collectors.toList());
        //拼接参数
        String ip = StringUtils.join(ips.toArray(), "|");
        //云主机通断状态
        List<PrometheusResult> status = prometheusService.query("probe_success{instance=~'" + ip + "'}");
        //云主机响应时长
        List<PrometheusResult> time = prometheusService.query("round(probe_duration_seconds{instance=~'" + ip + "'}*1000,0.001)");

        Map<Long, String> ipRelationsMap = ipRelations.stream().collect(Collectors.toMap(IpRelation::getTargetId, IpRelation::getIp));
        Map<String, String> statusMap = status.stream().collect(Collectors.toMap(result -> String.valueOf(result.getMetric().get("instance")), result -> String.valueOf(result.getValue()[1])));
        Map<String, String> timeMap = time.stream().collect(Collectors.toMap(result -> String.valueOf(result.getMetric().get("instance")), result -> String.valueOf(result.getValue()[1])));

        for (CloudServer cloudServer : list) {
            String monitorIp = ipRelationsMap.getOrDefault(cloudServer.getId(), cloudServer.getManagerIp());
            cloudServer.setOnOffStatus(statusMap.get(monitorIp));
            cloudServer.setResponseTime(timeMap.get(monitorIp));
            cloudServer.setMonitorIp(ipRelationsMap.get(cloudServer.getId()));
        }
    }
}
