package com.bocloud.cms.service.core;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.utils.Constant;
import com.bocloud.cms.entity.AnalysisSnapshot;
import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.cms.model.NotifyMessage;
import com.bocloud.cms.service.AnalysisServiceImpl;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.DateTools;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;

import java.util.Date;

/**
 * 监控分析的处理
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0-SNAPSHOT
 */
public class AnalysisHandleRunable implements Runnable {


    private static final Logger logger = LoggerFactory.getLogger(AnalysisHandleRunable.class);

    private Long id;
    private Long vendorId;
    private String startTime;
    private String endTime;

    private RequestContext context;
    private AnalysisServiceImpl monitorAnalysisService;
    private AmqpTemplate amqpTemplate;
    private String token;

    public AnalysisHandleRunable(Long id, Long vendorId, String startTime, String endTime,
                                 AnalysisServiceImpl monitorAnalysisService, AmqpTemplate amqpTemplate,
                                 RequestContext context, String token) {
        this.id = id;
        this.vendorId = vendorId;
        this.startTime = startTime;
        this.endTime = endTime;
        this.amqpTemplate = amqpTemplate;
        this.monitorAnalysisService = monitorAnalysisService;
        this.context = context;
        this.token = token;
    }

    @Override
    public void run() {
        try {
            Date start = DateTools.stringToDateTime(startTime);
            Date end = DateTools.stringToDateTime(endTime);
            CloudVendor vendor = monitorAnalysisService.getImsCloudVendorService().detail(vendorId).getData();
            MonitorAnalysisRule analysaisRule = monitorAnalysisService.getAnalysisRuleRepository().query(id);
            String type = analysaisRule.getType();
            AnalysisSnapshot snapshot;
            //flag prometheus查询时平台唯一标识
            String flag;
            switch (CloudProvider.valueOf(vendor.getType())) {
                case OPENSTACK:
                    flag = String.valueOf(vendor.getId());
                    break;
                case VMWARE:
                    flag = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    break;
                case MANAGEONE:
                case TIANYI:
                case CLOUDTOWER:
                    flag = String.valueOf(vendorId);
                    break;
                default:
                    throw new IllegalStateException("Unexpected value: " + CloudProvider.valueOf(vendor.getType()));
            }
            switch (type) {
                case "cloudServerRecycle":
                case "wholeMachineRecycle":
                case "cloudServerCapacityExpansion":
                    snapshot = monitorAnalysisService.vmAnalysis(JSONObject.parseArray(analysaisRule.getRules()), type, start, end, flag, vendor.getType(), token);
                    break;
                case "serverCapacityExpansion":
                    snapshot = monitorAnalysisService.hostAnalysis(JSONObject.parseArray(analysaisRule.getRules()), type, start, end, flag, vendor.getType(), vendor.getId());
                    break;
                default:
                    logger.error("未知的类型");
                    return;
            }
            snapshot.setCreatorId(context.getTarget());
            snapshot.setRuleId(analysaisRule.getId());
            snapshot.setRule(analysaisRule.getRules());
            snapshot.setGmtCreate(new Date());
            snapshot.setDeleted(false);
            snapshot.setRuleName(analysaisRule.getName());
            snapshot.setStartTime(start);
            snapshot.setEndTime(end);
            snapshot.setVendorId(vendorId);
            monitorAnalysisService.getAnalysisSnapshotRepository().save(snapshot);
            String content = "您提交的监控分析请求已分析完成，请到监控分析历史记录中查看分析结果。\r\n";
            content += "规则：" + analysaisRule.getName() + "\r\n";
            content += "平台：" + vendor.getName() + "\r\n";
            content += "时间：" + startTime + " - " + endTime + "\r\n";
            NotifyMessage message = new NotifyMessage(context, "监控分析结果通知", content, null,
                    true, false);
            amqpTemplate.convertAndSend(Constant.EVENT_NOTICE_QUEUE, message);
        } catch (Exception e) {
            logger.error("analysis error:" + e.getMessage(), e);
        }
    }
}
