package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.dms.entity.CephOsd;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.Map;

/**
 * Ceph硬盘
 *
 * @author zhangyf
 * @since 2020-07-08
 */
@Component
public class DmsCephOsdService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static Service service = Service.create(BoCloudService.DMS);

    private Map<String, Object> headers = new HashMap<>(4);

    @Autowired
    public DmsCephOsdService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    public CephOsd detail(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/ceph/osds/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), CephOsd.class);
    }

    public BocloudResult list(Pager pager) {
        BocloudResult result = serviceFactory.build(service, "/v1/ceph/osds", BoCloudMethod.GET, headers, pager.toMap()).invoke();
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return result;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
