package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.common.utils.Common;
import com.bocloud.dms.entity.Disk;
import com.bocloud.dms.entity.Storage;
import com.bocloud.dms.entity.Volume;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 磁盘实体
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class DmsStorageService implements InitializingBean {
    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private static Service service = Service.create(BoCloudService.DMS);
    private Map<String, Object> headers = new HashMap<>(4);

    public BocloudResult listDevices(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(6);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(service, "/v1/storages", BoCloudMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

    public BocloudResult listDisks(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(6);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(service, "/v1/storages/disks", BoCloudMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

    public BocloudResult listVolumes(Pager pager, String token, String action) {
        Map<String, Object> headers = new HashMap<>(6);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        Map<String, Object> paramMap = pager.toMap(false);
        paramMap.put("action", action);
        RemoteService remoteService = serviceFactory.build(service, "/v1/volumes", BoCloudMethod.GET, headers, paramMap);
        return remoteService.invoke();
    }

    public BocloudResult<Storage> detailDevice(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/storages/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, Storage.class), "success");
    }

    public BocloudResult<Disk> detailDisk(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/storages/disks/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, Disk.class), "success");
    }

    public BocloudResult<Volume> detailVolume(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/volumes/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, Volume.class), "success");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
