package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ims云服务
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class ImsCloudServerService {

    @Autowired
    private ServiceFactory serviceFactory;
    private static Service service = Service.create(BoCloudService.IMS);

    @Autowired
    private CurrentService currentService;

    public BocloudResult<CloudServer> detail(Long id) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        headers.put("source", currentService.getService().getName());
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, CloudServer.class), "success");
    }

    /**
     * 列表查询
     *
     * @param pager
     * @param token 如果为null，则使用internal_token内部调用查询
     * @return
     */
    public BocloudResult list(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(4);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms", BoCloudMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

}
