package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ims云平台调用
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Component
@Slf4j
public class ImsCloudVendorService implements InitializingBean {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private static Service service = Service.create(BoCloudService.IMS);
    private Map<String, Object> headers = new HashMap<>(4);

    public BocloudResult<CloudVendor> detail(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/vendors/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        Assert.isTrue(result.isSuccess(), "查询平台失败:" + result.getMessage());
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, CloudVendor.class), "success");
    }

    /**
     * 修改平台信息
     *
     * @param vendor
     */
    public BocloudResult update(CloudVendor vendor) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/vendors/" + vendor.getId(), BoCloudMethod.PUT,
                headers, JSONObject.toJSONString(vendor));
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();

    }

    /**
     * 开启监控
     *
     * @param ids
     * @return
     */
    public BocloudResult openMonitoring(List<Long> ids) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/vendors/alarm?action=open", BoCloudMethod.PATCH,
                headers, ids);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    /**
     * 关闭监控
     *
     * @param ids
     * @return
     */
    public BocloudResult closeMonitoring(List<Long> ids) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/vendors/alarm?action=close", BoCloudMethod.PATCH,
                headers, ids);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    public BocloudResult list(Pager pager) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/vendors", BoCloudMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
