package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.ims.entity.kubernetes.KubeNode;
import com.bocloud.ims.entity.kubernetes.KubePod;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * ims跨服务远程调用
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class ImsKubernetesService implements InitializingBean {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private static Service service = Service.create(BoCloudService.IMS);
    private Map<String, Object> headers = new HashMap<>(4);

    public BocloudResult listOfNode(Pager pager) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/kubernetes/nodes", BoCloudMethod.GET, headers, pager.toMap(false));
        BocloudResult invoke = remoteService.invoke();
        if (invoke.isFailed()) {
            return new BocloudResult(false, invoke.getErrorMsg());
        }
        return invoke;
    }

    public BocloudResult listOfPod(Pager pager) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/kubernetes/pods", BoCloudMethod.GET, headers, pager.toMap(false));
        BocloudResult invoke = remoteService.invoke();
        if (invoke.isFailed()) {
            return new BocloudResult(false, invoke.getErrorMsg());
        }
        return invoke;
    }

    public BocloudResult<KubeNode> detailOfNode(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/kubernetes/nodes/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, KubeNode.class), "success");
    }

    public BocloudResult<KubePod> detailOfPod(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/kubernetes/pods/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONObject.parseObject(data, KubePod.class), "success");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
