package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.nms.entity.SanFabricPort;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * NMS跨服务调用
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class NmsSanFabricPortService {
    @Autowired
    private ServiceFactory serviceFactory;
    private static Service service = Service.create(BoCloudService.NMS);

    @Autowired
    private CurrentService currentService;

    public BocloudResult listOfPorts(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(4);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(service, "/v1/switches/ports", BoCloudMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

    public BocloudResult<SanFabricPort> detail(Long id) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        headers.put("source", currentService.getService().getName());
        List<Param> paramList = new ArrayList<>();
        paramList.add(new Param(MapTools.simpleMap("id", id), Sign.EQ));
        RemoteService remoteService = serviceFactory.build(service, "/v1/switches/ports", BoCloudMethod.GET, headers, new Pager(1, 1, paramList, null).toMap(false));
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        JSONObject data = (JSONObject) result.getData();
        List<SanFabricPort> ports = data.getJSONArray(Common.ROWS).toJavaList(SanFabricPort.class);
        if (ports.isEmpty()) {
            return new BocloudResult<>(false, "no this port,id=" + id);
        }
        return new BocloudResult<>(true, ports.get(0), "success");
    }
}
