package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.model.Pager;
import com.bocloud.common.utils.Common;
import com.bocloud.pms.entity.Cms;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * PMS远程服务调用
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0-SNAPSHOT
 */
@Component
public class PmsCmsService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static Service service = Service.create(BoCloudService.PMS);
    private Map<String, Object> headers = new HashMap<>(4);

    @Autowired
    public PmsCmsService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    public Cms detail(Long id) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/cms/" + id+ "/basics", BoCloudMethod.GET, headers, null);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            throw new RuntimeException(result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Cms.class);
    }

    public BocloudResult list(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(Common.TOKEN, token);
        return serviceFactory.build(service, "/v1/cms", BoCloudMethod.GET, headers, pager.toMap()).invoke();
    }

    public BocloudResult list(Pager pager) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        return serviceFactory.build(service, "/v1/cms", BoCloudMethod.GET, headers, pager.toMap()).invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
